@@ -2,6 +2,8 @@
 #include <iostream>
 
 int main (int argc, char** argv) {
+  APEX_UNUSED(argc);
+  APEX_UNUSED(argv);
   apex::init("apex::version unit test", 0, 1);
   std::cout << apex::version() << std::endl;
   apex::finalize();
