@@ -17,6 +17,11 @@
 #ifndef _RTSLAYER_H_
 #define _RTSLAYER_H_
 
+#include <map>
+#include <string>
+#include <functional>
+#include <utility>
+
 //////////////////////////////////////////////////////////////////////
 //
 // class RtsLayer
@@ -29,7 +34,7 @@
 // restrained to this class. 
 //////////////////////////////////////////////////////////////////////
 
-typedef std::map<std::string, TauGroup_t, std::less<string> > ProfileMap_t;
+typedef std::map<std::string, TauGroup_t, std::less<std::string> > ProfileMap_t;
 
 
 double TauWindowsUsecD(void);
@@ -41,10 +46,14 @@ public:
   RtsLayer () { }  // defaults
   ~RtsLayer () { } 
 
-	static int createThread(void);
-	static void destroyThread(int id);
-	static void recycleThread(int id);
+  static int _createThread(void);
+  static int createThread(void);
+  static void destroyThread(int id);
+  static void recycleThread(int id);
 
+#ifdef JAVA
+  static bool& TheUsingJNI(void);
+#endif
 
   static TauGroup_t & TheProfileMask(void);
   static bool& TheEnableInstrumentation(void);
@@ -53,32 +62,33 @@ public:
   static int& TheContext(void);
   static long GenerateUniqueId(void);
   static ProfileMap_t& TheProfileMap(void);
-  static TauGroup_t getProfileGroup(char *  ProfileGroup) ;
+  static TauGroup_t getProfileGroup(char const *  ProfileGroup) ;
   static TauGroup_t enableProfileGroup(TauGroup_t  ProfileGroup) ;
   static TauGroup_t disableProfileGroup(TauGroup_t  ProfileGroup) ;
   static TauGroup_t generateProfileGroup(void) ;
-  static TauGroup_t enableProfileGroupName(char * ProfileGroup) ;
-  static TauGroup_t disableProfileGroupName(char * ProfileGroup) ;
+  static TauGroup_t enableProfileGroupName(char const * ProfileGroup) ;
+  static TauGroup_t disableProfileGroupName(char const * ProfileGroup) ;
   static TauGroup_t enableAllGroups(void) ;
   static TauGroup_t disableAllGroups(void) ;
   static TauGroup_t resetProfileGroup(void) ;
   static int setAndParseProfileGroups (char *prog, char *str) ;
   static bool isEnabled(TauGroup_t  ProfileGroup) ; 
   static void ProfileInit(int& argc, char**& argv);
-  static string PrimaryGroup(const char *ProfileGroupName);
+  static std::string PrimaryGroup(const char *ProfileGroupName);
   static bool isCtorDtor(const char *name);
 
   static std::string GetRTTI(const char *name); 
   inline static const char * CheckNotNull(const char * str) {
-    if (str == 0) return "  ";
-    else return str;
+    if (str) return str;
+    else return "  ";
   }
 
+	static void Initialize(void);
 
   static int 	SetEventCounter(void);
   static double GetEventCounter(void);
 
-  static void   getUSecD(int tid, double *values);
+  static void   getUSecD(int tid, double *values, int reversed=0);
 
   static void getCurrentValues(int tid, double *values);
 
@@ -86,8 +96,6 @@ public:
 
   static int setMyContext(int ContextId);
 
-  static int setMyThread(int tid);
-
   static const char* getSingleCounterName(); 
   static const char* getCounterName(int i); 
 
@@ -100,8 +108,14 @@ public:
   // Return the number of the 'current' thread. 0..TAU_MAX_THREADS-1
   static int myThread(void);
 
-  static int threadId(void);
-  
+  static int unsafeThreadId(void);
+ 
+ 	// Return the local thread id (ignoring tasks) This is a 
+	// low-overhead call but DO NOT use this call when
+	// accessing Profiler stack or the FunctionInfo DB.
+  static int localThreadId(void);
+  static int unsafeLocalThreadId(void);
+
   static int getPid();
   static int getTid();
 
@@ -114,11 +128,13 @@ public:
   static void RegisterFork(int nodeid, enum TauFork_t opcode);
 
   // This ensure that the FunctionDB (global) is locked while updating
-  static void LockDB(void);
-  static void UnLockDB(void);
+  static int LockDB(void);
+  static int UnLockDB(void);
+  static int getNumDBLocks(void);
 
-  static void LockEnv(void);
-  static void UnLockEnv(void);
+  static int LockEnv(void);
+  static int UnLockEnv(void);
+  static int getNumEnvLocks(void);
 
   static int getTotalThreads();
 
@@ -131,7 +147,7 @@ private:
   static void threadLockEnv(void);
   static void threadUnLockEnv(void);
 
-  static int lockDBcount[TAU_MAX_THREADS];
+  static int lockDBCount[TAU_MAX_THREADS];
   static int lockEnvCount[TAU_MAX_THREADS];
 
   static bool initLocks();
@@ -140,8 +156,8 @@ private:
 
 };
 
-extern "C" int Tau_RtsLayer_getTid();
 extern "C" int Tau_RtsLayer_createThread();
+extern "C" int Tau_RtsLayer_TheEnableInstrumentation();
 
 #endif /* _RTSLAYER_H_  */
 /***************************************************************************
