@@ -95,15 +95,14 @@
 
 #define TAU_PHASE_CREATE_DYNAMIC(var, name, type, group) void *var##finfo = NULL; Tau_profile_c_timer(&var##finfo, name, type, group, Tau_phase_enable_once(#group, &var##finfo))
 
-#define TAU_PHASE_START(var) Tau_start_timer(var##finfo, 1, Tau_get_thread())
-#define TAU_PHASE_STOP(var) Tau_stop_timer(var##finfo, Tau_get_thread())
-
+#define TAU_PHASE_START(var) Tau_start_timer(var##finfo, 1, Tau_get_thread()); Tau_invoke_plugin_phase_entry(var##finfo)
+#define TAU_PHASE_STOP(var) Tau_stop_timer(var##finfo, Tau_get_thread()); Tau_invoke_plugin_phase_exit(var##finfo)
 #define TAU_ENABLE_GROUP(group)			Tau_enable_group(group);
 #define TAU_DISABLE_GROUP(group)		Tau_disable_group(group);
-#define TAU_ENABLE_GROUP_NAME(group)            Tau_enable_group_name(group) 
+#define TAU_ENABLE_GROUP_NAME(group)            Tau_enable_group_name(group)
 #define TAU_DISABLE_GROUP_NAME(group)           Tau_disable_group_name(group)
-#define TAU_ENABLE_ALL_GROUPS()            	Tau_enable_all_groups() 
-#define TAU_DISABLE_ALL_GROUPS()            	Tau_disable_all_groups() 
+#define TAU_ENABLE_ALL_GROUPS()            	Tau_enable_all_groups()
+#define TAU_DISABLE_ALL_GROUPS()            	Tau_disable_all_groups()
 #define TAU_DISABLE_GROUP_NAME(group)           Tau_disable_group_name(group)
 #define TAU_GET_PROFILE_GROUP(group)            Tau_get_profile_group(group)
 
@@ -132,7 +131,7 @@
 #define TAU_PROFILE_TIMER_GET_GROUP(timer) Tau_profile_get_group(timer)
 #define TAU_PROFILE_TIMER_GET_GROUP_NAME(timer) Tau_profile_get_group_name(timer)
 
-#define TAU_REGISTER_THREAD()			Tau_register_thread();	
+#define TAU_REGISTER_THREAD()			Tau_register_thread();
 #define TAU_REGISTER_FORK(nodeid, op) 		Tau_register_fork(nodeid, op);
 #define TAU_ENABLE_INSTRUMENTATION()		Tau_enable_instrumentation();
 #define TAU_DISABLE_INSTRUMENTATION()		Tau_disable_instrumentation();
@@ -163,10 +162,10 @@
 #define TAU_REGISTER_EVENT(event, name)	static void *event = 0; \
                                  if (event == 0) event = Tau_get_userevent(name);
 #define TAU_PROFILER_REGISTER_EVENT(e, msg) TauUserEvent* e () { \
-	static TauUserEvent u(msg); return &u; } 
- 
+	static TauUserEvent u(msg); return &u; }
+
 #define TAU_REGISTER_CONTEXT_EVENT(event, name)	static void *event = 0; \
-                                 if (event == 0) Tau_get_context_userevent(&event, name);  
+                                 if (event == 0) Tau_get_context_userevent(&event, name);
 
 #define TAU_TRIGGER_CONTEXT_EVENT(eventname, eventvalue) Tau_trigger_context_event(eventname, eventvalue)
 #define TAU_TRIGGER_CONTEXT_EVENT_THREAD(eventname, eventvalue, tid) Tau_trigger_context_event_thread(eventname, eventvalue, tid)
@@ -175,7 +174,8 @@
 #define TAU_EVENT_THREAD(event, data, tid)				Tau_userevent_thread(event, data, tid)
 #define TAU_CONTEXT_EVENT(event, data)		Tau_context_userevent(event, data);
 #define TAU_CONTEXT_EVENT_THREAD(event, data, tid) Tau_context_userevent_thread(event, data, tid);
-#define TAU_EVENT_SET_NAME(event, name)	Tau_set_event_name(event, name); 	
+#define TAU_CONTEXT_EVENT_THREAD_TS(event, data, tid, ts) Tau_context_userevent_thread_ts(event, data, tid, ts);
+#define TAU_EVENT_SET_NAME(event, name)	Tau_set_event_name(event, name);
 #define TAU_REPORT_STATISTICS()		Tau_report_statistics();
 #define TAU_REPORT_THREAD_STATISTICS()  Tau_report_thread_statistics();
 #define TAU_EVENT_DISABLE_MIN(event) 	Tau_event_disable_min(event);
@@ -221,7 +221,7 @@
 { static void *ptr = NULL; \
   Tau_profile_c_timer(&ptr, name, type, group, #group); \
   return ptr; \
-} 
+}
 #define TAU_GLOBAL_TIMER_START(timer) { void *ptr = TauGlobal##timer(); \
     Tau_start_timer(ptr, 0, Tau_get_thread()); }
 #define TAU_GLOBAL_TIMER_STOP()  Tau_global_stop();
@@ -231,7 +231,7 @@
 { static void *ptr = NULL; \
   Tau_profile_c_timer(&ptr, name, type, group, #group); \
   return ptr; \
-} 
+}
 
 #define TAU_GLOBAL_PHASE_START(timer) { void *ptr = TauGlobalPhase##timer(); \
     Tau_start_timer(ptr, 1, Tau_get_thread()); }
@@ -312,7 +312,7 @@
 #ifdef TAU_PROFILEPARAM
 #define TAU_PROFILE_PARAM1L(data,name)  	Tau_profile_param1l(data,name)
 #else  /* TAU_PROFILEPARAM */
-#define TAU_PROFILE_PARAM1L(data,name)  	
+#define TAU_PROFILE_PARAM1L(data,name)
 #endif /* TAU_PROFILEPARAM */
 
 #define TAU_TRACE_SENDMSG(type, destination, length) \
@@ -325,6 +325,16 @@
 #define TAU_TRACE_RECVMSG_REMOTE(type, source, length, remoteid) \
         Tau_trace_recvmsg_remote(type, source, length, remoteid);
 
+#define TAU_TRACE_BARRIER_ALL_START(tag) \
+        Tau_trace_barrier_all_start(tag);
+#define TAU_TRACE_BARRIER_ALL_END(tag) \
+        Tau_trace_barrier_all_end(tag);
+
+#define TAU_TRACE_RMA_COLLECTIVE_BEGIN(tag, type, start, stride, size, data_in, data_out, root) \
+        Tau_trace_rma_collective_begin(tag, type, start, stride, size, data_in, data_out, root);
+#define TAU_TRACE_RMA_COLLECTIVE_END(tag, type, start, stride, size, data_in, data_out, root) \
+        Tau_trace_rma_collective_end(tag, type, start, stride, size, data_in, data_out, root);
+
 #define TAU_PROFILER_CREATE(handle, name, type, group)  handle=Tau_get_function_info(name, type, group, #group);
 #define TAU_PROFILER_START(handle) Tau_start_timer(handle, 0, Tau_get_thread());
 #define TAU_PROFILER_STOP(handle) Tau_stop_timer(handle, Tau_get_thread());
@@ -349,8 +359,8 @@
 
 #define TAU_BCAST_DATA(data)  	                Tau_bcast_data(data)
 #define TAU_REDUCE_DATA(data)  	                Tau_reduce_data(data)
-#define TAU_ALLTOALL_DATA(data)                 Tau_alltoall_data(data) 
-#define TAU_SCATTER_DATA(data)                  Tau_scatter_data(data) 
+#define TAU_ALLTOALL_DATA(data)                 Tau_alltoall_data(data)
+#define TAU_SCATTER_DATA(data)                  Tau_scatter_data(data)
 #define TAU_GATHER_DATA(data)  	                Tau_gather_data(data)
 #define TAU_ALLREDUCE_DATA(data)  	        Tau_allreduce_data(data)
 #define TAU_WAIT_DATA(data)  	        	Tau_wait_data(data)
@@ -367,6 +377,21 @@
 
 #endif /* TAU_FAPI */
 
+/* Plugin API */
+#define TAU_ENABLE_PLUGIN_FOR_SPECIFIC_EVENT Tau_enable_plugin_for_specific_event
+#define TAU_DISABLE_PLUGIN_FOR_SPECIFIC_EVENT Tau_disable_plugin_for_specific_event
+#define TAU_ENABLE_ALL_PLUGINS_FOR_SPECIFIC_EVENT Tau_enable_all_plugins_for_specific_event
+#define TAU_DISABLE_ALL_PLUGINS_FOR_SPECIFIC_EVENT Tau_disable_all_plugins_for_specific_event
+#define TAU_ENABLE_PLUGIN_FOR_TRIGGER_EVENT Tau_enable_plugin_for_trigger_event
+#define TAU_DISABLE_PLUGIN_FOR_TRIGGER_EVENT Tau_disable_plugin_for_trigger_event
+#define TAU_ENABLE_ALL_PLUGINS_FOR_TRIGGER_EVENT Tau_enable_all_plugins_for_trigger_event
+#define TAU_DISABLE_ALL_PLUGINS_FOR_TRIGGER_EVENT Tau_disable_all_plugins_for_trigger_event
+#define TAU_ADD_REGEX Tau_add_regex
+#define TAU_CHECK_FOR_MATCHING_REGEX Tau_check_for_matching_regex
+#define TAU_CREATE_TRIGGER Tau_create_trigger
+#define TAU_TRIGGER Tau_trigger
+/* Plugin API */
+
 #endif /* TAU_ENABLED */
 
 /* TAU_PLUGIN API */
@@ -392,8 +417,9 @@ void Tau_enable_group(TauGroup_t group);
 
 void Tau_start(const char *name);
 void Tau_stop(const char *name);
-void *Tau_pure_search_for_function(const char *name);
+void *Tau_pure_search_for_function(const char *name, int create);
 void Tau_pure_start_task(const char *name, int tid);
+void Tau_pure_start_task_group(const char *name, int tid, const char * group);
 void Tau_pure_stop_task(const char *name, int tid);
 void Tau_stop_current_timer();
 void Tau_stop_current_timer_task(int tid);
@@ -417,7 +443,7 @@ void Tau_get_exclusive_values(void *handle, double* values, int tid);
 void Tau_set_exclusive_values(void *handle, double* values, int tid);
 void Tau_get_counter_info(const char ***counterlist, int *numcounters);
 
-int TAUDECL Tau_get_current_stack_depth(int tid); 
+int TAUDECL Tau_get_current_stack_depth(int tid);
 
 int TAUDECL Tau_get_local_tid(void);
 int TAUDECL Tau_get_thread(void);
@@ -441,6 +467,8 @@ int Tau_global_decr_insideTAU();
 int Tau_global_getLightsOut();
 void Tau_global_setLightsOut();
 
+  // extern struct GpuThread gThreads[TAU_MAX_THREADS];
+
 long Tau_convert_ptr_to_long(void *ptr);
 unsigned long Tau_convert_ptr_to_unsigned_long(void *ptr);
 
@@ -484,6 +512,23 @@ void TAUDECL Tau_set_node(int node);
 
 void TAUDECL Tau_start_timer(void *profiler, int phase, int tid);
 void TAUDECL Tau_stop_timer(void *profiler, int tid);
+int TAUDECL Tau_invoke_plugin_phase_entry(void *profiler);
+int TAUDECL Tau_invoke_plugin_phase_exit(void *profiler);
+
+/* Plugin API */
+void TAUDECL Tau_add_regex(const char * r);
+void TAUDECL Tau_enable_all_plugins_for_specific_event(int ev, const char *name);
+void TAUDECL Tau_disable_all_plugins_for_specific_event(int ev, const char *name);
+void TAUDECL Tau_disable_plugin_for_specific_event(int ev, const char *name, unsigned int id);
+void TAUDECL Tau_enable_plugin_for_specific_event(int ev, const char *name, unsigned int id);
+void TAUDECL Tau_enable_all_plugins_for_trigger_event(int ev, size_t hash);
+void TAUDECL Tau_disable_all_plugins_for_trigger_event(int ev, size_t hash);
+void TAUDECL Tau_disable_plugin_for_trigger_event(int ev, size_t hash, unsigned int id);
+void TAUDECL Tau_enable_plugin_for_trigger_event(int ev, size_t hash, unsigned int id);
+size_t TAUDECL Tau_create_trigger(const char *name);
+void TAUDECL Tau_trigger(size_t id, void * data);
+/* Plugin API */
+
 void TAUDECL Tau_lite_start_timer(void *profiler, int phase);
 void TAUDECL Tau_lite_stop_timer(void *profiler);
 void TAUDECL Tau_pure_start(const char *name);
@@ -494,6 +539,10 @@ void TAUDECL Tau_trace_sendmsg(int type, int destination, int length);
 void TAUDECL Tau_trace_recvmsg(int type, int source, int length);
 void TAUDECL Tau_trace_recvmsg_remote(int type, int source, int length, int remoteid);
 void TAUDECL Tau_trace_sendmsg_remote(int type, int destination, int length, int remoteid);
+void TAUDECL Tau_trace_barrier_all_start(int tag);
+void TAUDECL Tau_trace_barrier_all_end(int tag);
+void TAUDECL Tau_trace_rma_collective_begin(int tag, int type, int start, int stride, int size, int data_in, int data_out, int root);
+void TAUDECL Tau_trace_rma_collective_end(int tag, int type, int start, int stride, int size, int data_in, int data_out, int root);
 void TAUDECL Tau_create_top_level_timer_if_necessary(void);
 void TAUDECL Tau_create_top_level_timer_if_necessary_task(int task);
 void TAUDECL Tau_stop_top_level_timer_if_necessary(void);
@@ -510,8 +559,8 @@ void TAUDECL Tau_metadata_array_put(Tau_metadata_value_t* array, const int index
 void TAUDECL Tau_metadata_object_put(Tau_metadata_value_t* object, const char *name, Tau_metadata_value_t* value);
 
 
-void TAUDECL Tau_Bg_hwp_counters_start(int *error); 
-void TAUDECL Tau_Bg_hwp_counters_stop(int* numCounters, x_uint64 counters[], int* mode, int *error); 
+void TAUDECL Tau_Bg_hwp_counters_start(int *error);
+void TAUDECL Tau_Bg_hwp_counters_stop(int* numCounters, x_uint64 counters[], int* mode, int *error);
 void TAUDECL Tau_Bg_hwp_counters_output(int* numCounters, x_uint64 counters[], int* mode, int* error);
 
 void Tau_set_user_clock(double value);
@@ -524,6 +573,7 @@ void Tau_set_context(int context);
 void Tau_set_thread(int thread);
 void Tau_callstack(void);
 int Tau_dump(void);
+void Tau_flush_gpu_activity(void);
 int Tau_mergeProfiles_MPI();
 int Tau_mergeProfiles_SHMEM();
 int Tau_dump_incr(void);
@@ -545,10 +595,12 @@ void Tau_get_context_userevent(void **ptr, const char *name);
 void Tau_trigger_context_event(const char *name, double data);
 void Tau_trigger_context_event_thread(const char *name, double data, int tid);
 void Tau_trigger_userevent(const char *name, double data);
+void Tau_trigger_userevent_thread(const char *name, double data, int tid);
 void Tau_userevent(void *event, double data);
 void Tau_userevent_thread(void *event, double data, int tid);
 void Tau_context_userevent(void *event, double data);
 void Tau_context_userevent_thread(void *event, double data, int tid);
+void Tau_context_userevent_thread_ts(void *event, double data, int tid, double ts);
 void Tau_set_event_name(void *event, char * name);
 void Tau_report_statistics(void);
 void Tau_report_thread_statistics(void);
@@ -598,28 +650,38 @@ void Tau_track_class_allocation(const char * name, size_t size);
 void Tau_track_class_deallocation(const char * name, size_t size);
 void Tau_start_class_allocation(const char * name, size_t size, int include_in_parent);
 void Tau_stop_class_allocation(const char * name, int record);
+void Tau_traced_api_call_enter();
+void Tau_traced_api_call_exit();
+int Tau_time_traced_api_call();
+void Tau_disable_pthread_tracking(void);
+void Tau_enable_pthread_tracking(void);
 
 #ifdef __cplusplus
 /* Include the C++ API header */
 #include <Profile/TauCppAPI.h>
 #else /* __cplusplus */
 /* These can't be used in C, only C++, so they are dummy macros here */
-#define TAU_PROFILE(name, type, group) 
-#define TAU_DYNAMIC_PROFILE(name, type, group) 
+#define TAU_PROFILE(name, type, group)
+#define TAU_DYNAMIC_PROFILE(name, type, group)
 #define TYPE_STRING(profileString, str)
 #define TAU_CT(obj)
-#endif /* __cplusplus */ 
+#endif /* __cplusplus */
 
 
 #ifdef __cplusplus
 } /* for extern "C" */
 #endif /* __cplusplus */
 
+// allow this to be called from within TAU
+#ifdef __cplusplus
+bool& Tau_is_pthread_tracking_enabled();
+#endif
+
 /**************************************************************************/
 
 #endif /* _TAU_API_H_ */
 /***************************************************************************
  * $RCSfile: TauAPI.h,v $   $Author: cheelee $
  * $Revision: 1.116 $   $Date: 2010/06/08 01:09:52 $
- * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.116 2010/06/08 01:09:52 cheelee Exp $ 
+ * POOMA_VERSION_ID: $Id: TauAPI.h,v 1.116 2010/06/08 01:09:52 cheelee Exp $
  ***************************************************************************/
