@@ -10,6 +10,7 @@
 enum Memcpy { MemcpyHtoD = 0, MemcpyDtoH = 1, MemcpyDtoD = 2, MemcpyUnknown = 3 };
 enum Unifmem { BytesHtoD = 0, BytesDtoH = 1, CPUPageFault = 2, UnifmemUnknown = 3 };
 enum Instrmix { FlPtOps = 0, MemOps = 1, CtrlOps = 2 };
+enum EnvType { SMClock=0, MemoryClock=1, GPUTemperature=2, PowerUtilization=3, FanSpeed=4 };
 
 #define TAU_GPU_UNKNOWN_TRANSFER_SIZE -1
 #define TAU_GPU_USE_DEFAULT_NAME ""
@@ -35,7 +36,7 @@ typedef struct {
 /* Struct to contain the metadata for each GPU. */
 typedef struct {
 	char *name;
-	const char *value;
+	char *value;
 
 } GpuMetadata;
 
@@ -61,6 +62,9 @@ public:
 	//Name for this event.
 	virtual const char *getName() const = 0;
 
+	//Task id for this event.
+	virtual int getTaskId() const = 0;
+
 	//CPU event callsite for this event. Used to create a the GPU callsite event paths.
 	virtual FunctionInfo* getCallingSite() const = 0;
 
@@ -85,55 +89,6 @@ public:
 	
 };
 
-/* /\* BEGIN: PC Sampling structs *\/ */
-
-/* class InstrSampling */
-/* { */
-/*  public: */
-/*   uint32_t sourceLocatorId; */
-/*   uint32_t functionId; */
-/*   uint32_t pcOffset; */
-/*   uint32_t correlationId; */
-/*   uint32_t executed; */
-/*   uint32_t threadsExecuted; */
-/*   double timestamp_delta; */
-/*   double timestamp_current; */
-/*   double timestamp_recent; */
-/*   }; */
-
-/* class FuncSampling */
-/* { */
-/*  public: */
-/*   uint32_t fid; */
-/*   uint32_t contextId; */
-/*   uint32_t moduleId; */
-/*   uint32_t functionIndex; */
-/*   char* name; */
-/*   const char* demangled; */
-/*   uint32_t calls; // unique function calls */
-/*   uint32_t kernel_launches; // total kernel launches for this function */
-/*   bool funcinfo_created;  // track whether FunctionInfo created for pprof */
-/* }; */
-
-/* class SourceSampling */
-/* { */
-/*  public: */
-/*   uint32_t sid; */
-/*   uint32_t fid; */
-/*   char* fileName; */
-/*   uint32_t lineNumber; */
-/*   double timestamp_delta; */
-/*   double timestamp_recentacc; */
-/*   uint32_t samples; // # samples for sid (file/line) */
-/* }; */
-
-
-/* static std::map<uint32_t, std::list<InstrSampling> > instrSrcMap; */
-/* static std::map<uint32_t, std::list<InstrSampling> > instrFuncMap; */
-/* static std::map<uint32_t, SourceSampling> srcLocMap; */
-/* static std::map<uint32_t, FuncSampling> funcMap; */
-/* END: PC Sampling structs */
-
 /************************************************************************
  * Performance Hooks. The following routines are hooks into the execution
  * of GPU applications. 
@@ -151,6 +106,9 @@ extern "C" void Tau_gpu_exit(void);
 /* Entry point for CPU routines */
 extern "C" void Tau_gpu_enter_event(const char *functionName);
 
+/* Entry point for GPU routines from synchronous callbacks*/
+extern "C" void Tau_gpu_enter_event_from_cpu(const char *functionName, int tid, double start);
+
 /* Entry point for CPU routines that initiate a memory copy to the GPU */
 extern "C" void Tau_gpu_enter_memcpy_event(const char *functionName,
 GpuEvent *gpu, int transferSize, int memcpyType);
@@ -162,6 +120,9 @@ GpuEvent *gpu, int transferSize, int unifmemType);
 /* Exit point for CPU routines */
 extern "C" void Tau_gpu_exit_event(const char *functionName);
 
+/* Exit point for GPU routines from synchronous callbacks */
+extern "C" void Tau_gpu_exit_event_from_cpu(const char *functionName, int tid, double end);
+
 /* Exit point for CPU routines that initiate a memory copy to the GPU */
 extern "C" void Tau_gpu_exit_memcpy_event(const char *functionName,
 GpuEvent *gpu, int memcpyType);
@@ -185,31 +146,15 @@ extern "C" void Tau_gpu_register_memcpy_event(GpuEvent *event, double startTime,
  * program. Times are pre-aligned to the CPU clock. */
 extern "C" void Tau_gpu_register_unifmem_event(GpuEvent *event, double startTime, double endTime, int transferSize, int unifmemType, int direction);
 
+extern "C" void Tau_gpu_register_envt_event(GpuEvent *event, double startTime, double endTime, int transferSize, int dataType);
+
+/* Callback for a GPU synchronization event */
+extern "C" void Tau_gpu_register_sync_event(GpuEvent *event, double startTime, double endTime);
+
 /* Callback for a GPU atomic event that is associated with this gpu event. */
 extern "C" void Tau_gpu_register_gpu_atomic_event(GpuEvent *event);
 
-extern "C" void TauTraceOneSidedMsg(int type, GpuEvent *gpu, int length, int thread);
-
-/* /\* Callback for a SASS event that occurred earlier in the execution of the                                    */
-/*  * program. Timestamps from GPU clock. *\/ */
-/* extern "C" void Tau_gpu_register_func_event(GpuEvent *event, int deviceId, double timeStamp, const char* name, uint32_t contextId, uint32_t functionIndex, uint32_t id, uint32_t moduleid, const char *kname, const char *demangled); */
-
-/* extern "C" void Tau_gpu_register_instruction_event(GpuEvent *event, double start, double stop, double delta_tstamp, const char* name, uint32_t correlationId, uint32_t sourceLocatorId, uint32_t functionId, uint32_t pcOffset, uint32_t executed, uint32_t threadsExecuted); */
-
-/* extern "C" void Tau_gpu_register_source_event(GpuEvent *event, double timestamp, const char* name, uint32_t sourceId, const char *fileName, uint32_t lineNumber); */
-
-/* //extern "C" void printInstrMap(void); */
-/* extern "C" void printSourceMap(void); */
-/* extern "C" void printFuncMap(void); */
-
-/* // routines for calculating kernel level stats */
-/* extern "C" double getKernelExecutionTimes(uint32_t functionIndex); */
-/* extern "C" uint32_t getKernelSamples(uint32_t functionIndex); */
-/* extern "C" const char* getKernelFilePath(uint32_t functionIndex); */
-/* extern "C" uint32_t getKernelLineNo(uint32_t functionIndex); */
-/* extern "C" void resetKernelExecutionTimes(uint32_t functionIndex); */
-/* extern "C" uint32_t getUniqueKernelLaunches(uint32_t functionIndex); */
-
+extern "C" void TauTraceOneSidedMsg(int type, GpuEvent *gpu, int length, int thread, x_uint64 ts);
 
 #endif // __cplusplus
 #endif // _TAU_GPU_INTERFACE
