@@ -21,6 +21,7 @@
 #ifndef _TAU_HANDLER_H_
 #define _TAU_HANDLER_H_
 
+void TauSetupHandler(void);
 void TauSetInterruptInterval(int interval);
 
 int  TauEnableTrackingMemory(void);
@@ -42,6 +43,9 @@ int TauEnableTrackingMemoryHeadroom(void);
 int TauDisableTrackingMemoryHeadroom(void);
 void TauTrackMemoryHeadroomHere(void);
 
+int Tau_open_system_file(const char *filename);
+int Tau_read_load_event(int fd, double *value);
+
 #endif /* _TAU_HANDLER_H_ */
   
 /***************************************************************************
