@@ -12,17 +12,18 @@
  * **      Documentation   : See http://www.cs.uoregon.edu/research/tau       **
  * ***************************************************************************/
 
-#include <Profile/TauPluginTypes.h>
 
 #ifndef _TAU_PLUGIN_H_
 #define _TAU_PLUGIN_H_
 
+#include <Profile/TauPluginTypes.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
 
 void Tau_util_init_tau_plugin_callbacks(Tau_plugin_callbacks_t * cb);
-void Tau_util_plugin_register_callbacks(Tau_plugin_callbacks_t * cb);
+void Tau_util_plugin_register_callbacks(Tau_plugin_callbacks_t * cb, unsigned int plugin_id);
 
 #ifdef __cplusplus
 }
