@@ -8,12 +8,21 @@
 */
 #include <Profile/Profiler.h>
 #include <vector>
-using namespace std;
+// Putting "using namespace" statements in header files can create ambiguity
+// between user-defined symbols and std symbols, creating unparsable code
+// or even changing the behavior of user codes.  This is also widely considered
+// to be bad practice.  Here's a code PDT can't parse because of this line:
+//   EX: #include <complex>
+//   EX: typedef double real;
+//
+//using namespace std;
+
+
 
 void Tau_sampling_outputTraceCallstack(int tid, void *pc, 
 				       void *context);
 void Tau_sampling_unwindTauContext(int tid, void **address);
-vector<unsigned long> *Tau_sampling_unwind(int tid, Profiler *profiler,
+std::vector<unsigned long> *Tau_sampling_unwind(int tid, Profiler *profiler,
 					   void *pc, void *context);
 
 /* *CWL* - Looks like we do need to add common unwinding prototypes here as well. 
