@@ -41,7 +41,7 @@ void Tau_util_destroyOutputDevice(Tau_util_outputDevice *out);
 int Tau_util_getOutputBufferLength(Tau_util_outputDevice *out);
 int Tau_util_output(Tau_util_outputDevice *out, const char *format, ...);
 int Tau_util_readFullLine(char *line, FILE *fp);
-char *Tau_util_removeRuns(char *str);
+char const * Tau_util_removeRuns(char const * str);
 
 void TAU_ABORT(const char *format, ...);
 
@@ -50,6 +50,19 @@ void *Tau_util_malloc(size_t size, const char *file, int line);
 void *Tau_util_calloc(size_t size, const char *file, int line);
 #define TAU_UTIL_CALLOC(size) Tau_util_calloc(size, __FILE__, __LINE__);
 
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+void Tau_plugin_sendmsg(long unsigned int type, long unsigned int destination, long unsigned int length, long unsigned int remoteid);
+void Tau_plugin_recvmsg(long unsigned int type, long unsigned int source, long unsigned int length, long unsigned int remoteid);
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
+
+// Use a macro so we can compile it out if too much overhead.
+
+#define TAU_PLUGIN_SENDMSG(t, d, l, r) if (Tau_plugins_enabled.send) { Tau_plugin_sendmsg((long unsigned int)t, (long unsigned int)d, (long unsigned int)l, (long unsigned int)r); }
+#define TAU_PLUGIN_RECVMSG(t, s, l, r) if (Tau_plugins_enabled.recv) { Tau_plugin_recvmsg((long unsigned int)t, (long unsigned int)s, (long unsigned int)l, (long unsigned int)r); }
 
 /* The following macros help create a local array and assign to elements of 
    the local C array, values from Fortran array after conversion using f2c 
@@ -79,7 +92,6 @@ void *Tau_util_calloc(size_t size, const char *file, int line);
 #define TAU_FREE_LOCAL(l) free(l)
 #endif /* sgi || TAU_MPI_NEEDS_STATUS */
 
-
 /******************************************************/
 #if (defined(sgi) || defined(TAU_MPI_NEEDS_STATUS))
 #define MPI_Status_c2f(c,f) *(MPI_Status *)f=*(MPI_Status *)c 
