@@ -20,10 +20,13 @@
 #include "errno.h"
 #include <pthread.h>
 #include <unistd.h>
+#include <sys/wait.h>
 #include <sys/time.h>
 #include <signal.h>
 #include <algorithm>
 #include <iterator>
+#include <fcntl.h>
+
 
 #ifdef TAU_MPI
 #include <mpi.h>
@@ -41,12 +44,14 @@ unsigned long fi_count = 0;
 unsigned long ue_count = 0;
 static bool done = false;
 static SOS_runtime * _runtime = NULL;
+static int listener_pid = 0;
 pthread_mutex_t _my_mutex; // for initialization, termination
 pthread_cond_t _my_cond; // for timer
 pthread_t worker_thread;
 bool _threaded = false;
 int daemon_rank = 0;
 int my_rank = 0;
+int listener_rank = 0;
 int comm_size = 1;
 bool shutdown_daemon = false;
 int period_microseconds = 2000000;
@@ -91,13 +96,13 @@ void * Tau_sos_thread_function(void* data) {
         pthread_mutex_lock(&_my_mutex);
         int rc = pthread_cond_timedwait(&_my_cond, &_my_mutex, &ts);
         if (rc == ETIMEDOUT) {
-            TAU_VERBOSE("%d Sending data from TAU thread...\n", RtsLayer::myNode()); fflush(stderr);
-            TAU_SOS_send_data();
-            TAU_VERBOSE("%d Done.\n", RtsLayer::myNode()); fflush(stderr);
+            TAU_VERBOSE("[TAU_SOS]%d Sending data from TAU thread...\n", RtsLayer::myNode()); fflush(stderr);
+            TAU_SOS_send_data(false);
+            TAU_VERBOSE("[TAU_SOS]%d Done.\n", RtsLayer::myNode()); fflush(stderr);
         } else if (rc == EINVAL) {
-            TAU_VERBOSE("Invalid timeout!\n"); fflush(stderr);
+            TAU_VERBOSE("[TAU_SOS]Invalid timeout!\n"); fflush(stderr);
         } else if (rc == EPERM) {
-            TAU_VERBOSE("Mutex not locked!\n"); fflush(stderr);
+            TAU_VERBOSE("[TAU_SOS]Mutex not locked!\n"); fflush(stderr);
         }
     }
     // unlock after being signalled.
@@ -120,7 +125,7 @@ void TAU_SOS_make_pub() {
         sprintf(pub_name, "TAU_SOS_SUPPORT");
         sprintf(app_version, "v0.alpha");
         SOS_pub_init(_runtime, &tau_sos_pub, pub_name, SOS_NATURE_DEFAULT);
-        //SOS_pub_config(tau_sos_pub, SOS_PUB_OPTION_CACHE, thePluginOptions().env_sos_cache_depth);
+        SOS_pub_config(tau_sos_pub, SOS_PUB_OPTION_CACHE, thePluginOptions().env_sos_cache_depth);
 
         strcpy(tau_sos_pub->prog_ver, app_version);
         tau_sos_pub->meta.channel       = 1;
@@ -209,18 +214,18 @@ void TAU_SOS_send_shutdown_message(void) {
     header.msg_size = offset;
     offset = 0;
     SOS_buffer_pack(buffer, &offset, (char*)"i", header.msg_size);
-    TAU_VERBOSE("Sending SOS_MSG_TYPE_SHUTDOWN ...\n");
+    TAU_VERBOSE("[TAU_SOS]Sending SOS_MSG_TYPE_SHUTDOWN ...\n");
     SOS_send_to_daemon(buffer, buffer);
     SOS_buffer_destroy(buffer);
 	char * exporting = getenv("SOS_EXPORT_DB_AT_EXIT");
 	if (exporting != NULL) {
-    	TAU_VERBOSE("Waiting %d seconds for SOS to write (if necessary)...\n", thePluginOptions().env_sos_shutdown_delay);
+    	TAU_VERBOSE("[TAU_SOS]Waiting %d seconds for SOS to write (if necessary)...\n", thePluginOptions().env_sos_shutdown_delay);
 		sleep(thePluginOptions().env_sos_shutdown_delay);
 	}
 #endif
 }
 
-void TAU_SOS_fork_exec_sosd(void) {
+bool TAU_SOS_fork_exec_sosd(void) {
 #ifdef TAU_MPI
     // first, figure out who should fork a daemon on this node
     int i;
@@ -248,19 +253,46 @@ void TAU_SOS_fork_exec_sosd(void) {
         }
         host_index = host_index + hostlength;
     }
+    char* forkCommand = NULL;
+    char* ranks_per_node = NULL;
+    char* offset = NULL;
+    forkCommand = getenv ("SOS_FORK_COMMAND");
+    //std::cout << "forkCommand " << forkCommand << std::endl;
+    ranks_per_node = getenv ("SOS_APP_RANKS_PER_NODE");
+    //std::cout << "ranks_per_node " << ranks_per_node << std::endl;
+    offset = getenv ("SOS_LISTENER_RANK_OFFSET");
+    //std::cout << "offset " << offset << std::endl;
+    if (!forkCommand || !ranks_per_node || !offset) {
+        if (my_rank == 0) {
+            std::cerr << "Please set the SOS_FORK_COMMAND, SOS_APP_RANKS_PER_NODE, and SOS_LISTENER_RANK_OFFSET environment variables to spawn SOS in the background." << std::endl;
+            std::cerr << "SOS Listener not found, SOS plugin not configured." << std::endl;
+        }
+        return false;
+    }
     // fork the daemon
     if (my_rank == daemon_rank) {
-        int pid = vfork();
-        if (pid == 0) {
-            char* forkCommand = NULL;
-            char* ranks_per_node = NULL;
-            char* offset = NULL;
-            forkCommand = getenv ("SOS_FORK_COMMAND");
-            //std::cout << "forkCommand " << forkCommand << std::endl;
-            ranks_per_node = getenv ("SOS_APP_RANKS_PER_NODE");
-            //std::cout << "ranks_per_node " << ranks_per_node << std::endl;
-            offset = getenv ("SOS_LISTENER_RANK_OFFSET");
-            //std::cout << "offset " << offset << std::endl;
+        int outfd = 0;
+        int errfd = 0;
+        /* This is disabled unless you need to log the output from listeners */
+        if (ranks_per_node && false) {
+            int rpn = atoi(ranks_per_node);
+            std::stringstream ss;
+            ss << "listener." << my_rank / rpn << ".out";
+            if ((outfd = open(ss.str().c_str(), O_CREAT|O_TRUNC|O_WRONLY, 0644)) < 0) {
+                perror(ss.str().c_str());    /* open failed */
+                exit(1);
+            }
+            std::cout << "writing output of the command " << forkCommand << " to " << ss.str() << std::endl;
+            std::stringstream ss2;
+            ss2 << "listener." << my_rank / rpn << ".err";
+            if ((errfd = open(ss2.str().c_str(), O_CREAT|O_TRUNC|O_WRONLY, 0644)) < 0) {
+                perror(ss2.str().c_str());    /* open failed */
+                exit(1);
+            }
+            std::cout << "writing error of the command " << forkCommand << " to " << ss2.str() << std::endl;
+        }
+        int listener_pid = vfork();
+        if (listener_pid == 0) {
             if (forkCommand) {
                 std::string custom_command(forkCommand);
                 size_t index = 0;
@@ -268,7 +300,7 @@ void TAU_SOS_fork_exec_sosd(void) {
                 if (index != std::string::npos) {
                     if (ranks_per_node) {
                         int rpn = atoi(ranks_per_node);
-                        int listener_rank = my_rank / rpn;
+                        listener_rank = my_rank / rpn;
                         if(offset) {
                             int off = atoi(offset);
                             listener_rank = listener_rank + off;
@@ -278,10 +310,14 @@ void TAU_SOS_fork_exec_sosd(void) {
                         custom_command.replace(index,15,ss.str());
                     }
                 }
-                std::cout << "SOS Listener not found, Rank " << my_rank << " spawning SOS daemon(s): " << custom_command << std::endl;
+                std::cout << "SOS Listener not found, Rank " << my_rank << " spawned SOS daemon(s): " << custom_command << std::endl;
+                if (outfd > 0) {
+                    dup2(outfd, STDOUT_FILENO);    /* fd becomes the standard output */
+                }
+                if (errfd > 0) {
+                    dup2(errfd, STDERR_FILENO);    /* fd becomes the standard output */
+                }
                 TAU_SOS_do_fork(custom_command);
-            } else {
-                std::cerr << "Please set the SOS_FORK_COMMAND environment variable to spawn SOS in the background." << std::endl;
             }
         }
     }
@@ -290,6 +326,7 @@ void TAU_SOS_fork_exec_sosd(void) {
     //
     //wait(2);
 #endif
+    return true;
 }
 
 /*********************************************************************
@@ -432,9 +469,9 @@ void Tau_SOS_parse_selection_file(const char * filename) {
     }
 }
 
-void TAU_SOS_init() {
+bool TAU_SOS_init() {
     static bool initialized = false;
-    TAU_VERBOSE("TAU_SOS_init()...\n");
+    TAU_VERBOSE("[TAU_SOS]TAU_SOS_init()...\n");
     if (!initialized) {
 #ifdef TAU_MPI
         PMPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
@@ -449,12 +486,16 @@ void TAU_SOS_init() {
         // if runtime returns null, wait a bit and try again. If 
         // we fail "too many" times, give an error and continue
         _runtime = NULL;
-        TAU_VERBOSE("TAU_SOS_init() trying to connect...\n");
+        TAU_VERBOSE("[TAU_SOS]TAU_SOS_init() trying to connect...\n");
         SOS_init(&_runtime, SOS_ROLE_CLIENT, SOS_RECEIVES_NO_FEEDBACK, NULL);
         if(_runtime == NULL) {
-            TAU_VERBOSE("Unable to connect to SOS daemon. Spawning...\n");
-            TAU_SOS_fork_exec_sosd();
-            shutdown_daemon = true;
+            TAU_VERBOSE("[TAU_SOS]Unable to connect to SOS daemon. Spawning...\n");
+            shutdown_daemon = TAU_SOS_fork_exec_sosd();
+            if (!shutdown_daemon) { 
+                // failed.  Don't claim initialized.
+                _runtime = NULL;
+                return false;
+            }
         }
         int repeat = 3;
         while(_runtime == NULL) {
@@ -463,17 +504,17 @@ void TAU_SOS_init() {
             TAU_VERBOSE("TAU_SOS_init() trying to connect...\n");
             SOS_init(&_runtime, SOS_ROLE_CLIENT, SOS_RECEIVES_NO_FEEDBACK, NULL);
             if (_runtime != NULL) {
-                TAU_VERBOSE("Connected to SOS daemon. Continuing...\n");
+                TAU_VERBOSE("[TAU_SOS]Connected to SOS daemon. Continuing...\n");
                 break;
             } else if (--repeat < 0) { 
-                TAU_VERBOSE("Unable to connect to SOS daemon. Failing...\n");
-                return;
+                TAU_VERBOSE("[TAU_SOS]Unable to connect to SOS daemon. Failing...\n");
+                return false;
             }
         }
 
         if (thePluginOptions().env_sos_periodic) {
             period_microseconds = thePluginOptions().env_sos_period;
-            TAU_VERBOSE("Spawning thread for SOS.\n");
+            TAU_VERBOSE("[TAU_SOS]Spawning thread for SOS.\n");
             int ret = pthread_create(&worker_thread, NULL, &Tau_sos_thread_function, NULL);
             if (ret != 0) {
                 errno = ret;
@@ -484,6 +525,8 @@ void TAU_SOS_init() {
         initialized = true;
         /* Fixme! Insert all the data that was collected into Metadata */
     }
+    // if we have a runtime, all is good.
+    return (!(_runtime == NULL));
 }
 
 void TAU_SOS_stop_worker(void) {
@@ -493,7 +536,7 @@ void TAU_SOS_stop_worker(void) {
     done = true;
     pthread_mutex_unlock(&_my_mutex);
     if (thePluginOptions().env_sos_periodic) {
-        TAU_VERBOSE("TAU SOS thread joining...\n"); fflush(stderr);
+        TAU_VERBOSE("[TAU_SOS]TAU SOS thread joining...\n"); fflush(stderr);
         pthread_cond_signal(&_my_cond);
         int ret = pthread_join(worker_thread, NULL);
         if (ret != 0) {
@@ -534,18 +577,39 @@ void TAU_SOS_finalize(void) {
         TAU_SOS_stop_worker();
     }
     // flush any outstanding packs
-    TAU_SOS_send_data();
+    TAU_SOS_send_data(true);
     // shutdown the daemon, if necessary
     if (shutdown_daemon) {
         if (my_rank == daemon_rank) {
-            TAU_VERBOSE("Waiting %d seconds for SOS to flush...\n", thePluginOptions().env_sos_shutdown_delay);
+            TAU_VERBOSE("[TAU_SOS]Waiting %d seconds for SOS to flush...\n", thePluginOptions().env_sos_shutdown_delay);
 		    sleep(thePluginOptions().env_sos_shutdown_delay);
+			printf("[TAU_SOS]TAU: rank %d sending shutdown message to listener %d...\n", my_rank, listener_rank);
             TAU_SOS_send_shutdown_message();
+			int returnStatus = 0;
+			pid_t retval = 0;
+			if (listener_pid != 0) {
+				// wait for zombie children, just in case.
+			    retval = waitpid((pid_t) (-1), &returnStatus, 0);
+			} else {
+				retval = waitpid(listener_pid, &returnStatus, 0);
+			}
+            if (WIFEXITED(returnStatus)) {
+                printf("listener %d exited, status=%d\n", listener_rank, WEXITSTATUS(returnStatus));
+            } else if (WIFSIGNALED(returnStatus)) {
+                printf("listener %d killed by signal %d\n", listener_rank, WTERMSIG(returnStatus));
+            } else if (WIFSTOPPED(returnStatus)) {
+                printf("listener %d stopped by signal %d\n", listener_rank, WSTOPSIG(returnStatus));
+            } else if (WIFCONTINUED(returnStatus)) {
+                printf("listener %d continued\n", listener_rank);
+            }
+			if (retval < 0) {
+				perror("waitpid error: ");
+        		fprintf(stderr, "WARNING! SOS listener %d did not exit normally!\n", listener_rank);
+			}
         }
         // shouldn't be necessary, but sometimes the shutdown message is ignored?
         //TAU_SOS_fork_exec_sosd_shutdown();
     }
-        printf("}\n");
     SOS_finalize(_runtime);
 }
 
@@ -553,11 +617,13 @@ void Tau_SOS_pack_current_timer(const char * event_name) {
     if (_runtime == NULL) { return; }
     // first time?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     if (done) { return; }
@@ -579,11 +645,13 @@ void Tau_SOS_pack_string(const char * name, char * value) {
     if (done) { return; }
     // first time?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     // if (strlen(name) > 256) { printf("long string, %d: '%s'\n", strlen(name), name); }
@@ -595,11 +663,13 @@ void Tau_SOS_pack_double(const char * name, double value) {
     if (done) { return; }
     // first time?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     // if (strlen(name) > 256) { printf("long string, %d: '%s'\n", strlen(name), name); }
@@ -611,11 +681,13 @@ void Tau_SOS_pack_integer(const char * name, int value) {
     if (done) { return; }
     // first time?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     // if (strlen(name) > 256) { printf("long string, %d: '%s'\n", strlen(name), name); }
@@ -627,11 +699,13 @@ void Tau_SOS_pack_long(const char * name, long int value) {
     if (done) { return; }
     // first time?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     // if (strlen(name) > 256) { printf("long string, %d: '%s'\n", strlen(name), name); }
@@ -658,6 +732,14 @@ void TAU_SOS_pack_profile() {
     Tau_global_incr_insideTAU();
     // get the most up-to-date profile information
     TauProfiler_updateAllIntermediateStatistics();
+    TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
+    RtsLayer::LockDB();
+    /* Copy the function info database so we can release the lock */
+    std::vector<FunctionInfo*> tmpTimers(TheFunctionDB());
+    // use the normal copy constructor.
+    //tau::AtomicEventDB tmpCounters(tau::TheEventDB());
+    TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
+    RtsLayer::UnLockDB();
 
     // get the FunctionInfo database, and iterate over it
     std::vector<FunctionInfo*>::const_iterator it;
@@ -665,13 +747,12 @@ void TAU_SOS_pack_profile() {
     int numCounters;
     TauMetrics_getCounterList(&counterNames, &numCounters);
     //printf("Num Counters: %d, Counter[0]: %s\n", numCounters, counterNames[0]);
-    RtsLayer::LockDB();
 
     std::map<std::string, std::vector<double>* > low_res_timer_map;
     std::map<std::string, std::vector<double>* >::iterator timer_map_it;
 
     //foreach: TIMER
-    for (it = TheFunctionDB().begin(); it != TheFunctionDB().end(); it++) {
+    for (it = tmpTimers.begin(); it != tmpTimers.end(); it++) {
         FunctionInfo *fi = *it;
         /* First, check to see if we are including/excluding this timer */
         if (skip_timer(fi->GetName())) {
@@ -688,6 +769,9 @@ void TAU_SOS_pack_profile() {
         //foreach: THREAD
         for (tid = 0; tid < RtsLayer::getTotalThreads(); tid++) {
             calls = fi->GetCalls(tid);
+            // skip this timer if this thread didn't call it.
+            // for data-reduction reasons.
+            if (calls == 0) continue;
             std::stringstream calls_str;
             calls_str << "TAU_TIMER:" << tid << ":calls:" << fi->GetAllGroups() << ":" << fi->GetName();
             const std::string& tmpcalls = calls_str.str();
@@ -715,17 +799,20 @@ void TAU_SOS_pack_profile() {
         }
     }
 
+    tau::AtomicEventDB::const_iterator counterIterator;
     std::map<std::string, double> low_res_counter_map;
     std::map<std::string, double>::iterator counter_map_it;
 
     // do the same with counters.
-    //std::vector<tau::TauUserEvent*>::const_iterator it2;
-    tau::AtomicEventDB::iterator it2;
     int numEvents;
     double max, min, mean, sumsqr;
     std::stringstream tmp_str;
-    for (it2 = tau::TheEventDB().begin(); it2 != tau::TheEventDB().end(); it2++) {
-        tau::TauUserEvent *ue = (*it2);
+    TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
+    RtsLayer::LockDB();
+    for (counterIterator = tau::TheEventDB().begin();
+         counterIterator != tau::TheEventDB().end(); counterIterator++) {
+        tau::TauUserEvent *ue = (*counterIterator);
+        if (ue == NULL) continue;
         /* First, check to see if we are including/excluding this counter */
         if (skip_counter(ue->GetName().c_str())) {
             continue;
@@ -734,7 +821,7 @@ void TAU_SOS_pack_profile() {
 
         int tid = 0;
         for (tid = 0; tid < RtsLayer::getTotalThreads(); tid++) {
-            if (ue && ue->GetNumEvents(tid) == 0) continue;
+            if (ue->GetNumEvents(tid) == 0) continue;
             //if (ue && ue->GetWriteAsMetric()) continue;
             numEvents = ue->GetNumEvents(tid);
             mean = ue->GetMean(tid);
@@ -763,14 +850,18 @@ void TAU_SOS_pack_profile() {
             tmp_str.str(std::string());
         }
     }
+    TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
+    RtsLayer::UnLockDB();
     
     if ((ue_count + fi_count) > SOS_DEFAULT_ELEM_MAX) {
-        TAU_VERBOSE("DANGER, WILL ROBINSON! EXCEEDING MAX ELEMENTS IN SOS. Bad things might happen?\n");
+        TAU_VERBOSE("[TAU_SOS]DANGER, WILL ROBINSON! EXCEEDING MAX ELEMENTS IN SOS. Bad things might happen?\n");
     }
-    RtsLayer::UnLockDB();
 }
 
-void TAU_SOS_send_data(void) {
+extern "C" int Tau_open_status(void);
+extern "C" int Tau_read_status(int fd, long long * rss, long long * hwm);
+
+void TAU_SOS_send_data(bool finalizing) {
     // have we initialized?
     if (_runtime == NULL) { 
         fprintf(stderr, "ERROR! No SOS runtime found, did you initialize?\n");
@@ -778,26 +869,32 @@ void TAU_SOS_send_data(void) {
     }
     // Do we have a pub handle?
     if (tau_sos_pub == NULL) {
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::LockDB()\n");
         RtsLayer::LockDB();
         // protect against race conditions
         if (tau_sos_pub == NULL) {
             TAU_SOS_make_pub();
         }
+        TAU_VERBOSE("[TAU_SOS_send_data]:  RtsLayer::UnLockDB()\n");
         RtsLayer::UnLockDB();
     }
     // Make sure we have a pub handle */
     assert(tau_sos_pub);
-    // Update these now, WITHOUT a signal. Signals are TROUBLE.
-    TAU_TRACK_MEMORY_HERE();
-    TAU_TRACK_MEMORY_FOOTPRINT_HERE();
-    TAU_TRACK_LOAD_HERE();
+    /* records the heap, with no context, even though it says "here". */
+    Tau_track_memory_here();
+    /* records the rss/hwm, without context. */
+    Tau_track_memory_rss_and_hwm();
+    /* records the load, without context */
+    Tau_track_load();
     /* Only send a profile update if we aren't tracing */
-    if (!thePluginOptions().env_sos_tracing) {
+    if (finalizing || 
+        (!thePluginOptions().env_sos_tracing && 
+	     !thePluginOptions().env_sos_trace_adios)) {
         TAU_SOS_pack_profile();
     }
     static int frame = 0;
     TAU_VERBOSE("[TAU_SOS_send_data]: Publishing the values for frame %d...\n", frame++);
-    TAU_VERBOSE("MY RANK IS: %d/%d\n", _runtime->config.comm_rank, _runtime->config.comm_size);
+    TAU_VERBOSE("[TAU_SOS_send_data]MY RANK IS: %d/%d\n", _runtime->config.comm_rank, _runtime->config.comm_size);
     SOS_announce(tau_sos_pub);
     SOS_publish(tau_sos_pub);
     TAU_VERBOSE("[TAU_SOS_send_data]:   ...done.\n");
@@ -807,7 +904,11 @@ void TAU_SOS_send_data(void) {
 // C++ program to implement wildcard
 // pattern matching algorithm
 // from: https://www.geeksforgeeks.org/wildcard-pattern-matching/
+#if defined(__APPLE__) && defined(__clang__)
+// do nothing
+#else
 #include <bits/stdc++.h>
+#endif
 using namespace std;
  
 // Function that matches input str with
