@@ -14,9 +14,10 @@ e*			TAU Portable Profiling Package			   **
 ***************************************************************************/
 
 //////////////////////////////////////////////////////////////////////
-// Include Files 
+// Include Files
 //////////////////////////////////////////////////////////////////////
 
+//#define DEBUG_PROF 1
 #include "Profile/Profiler.h"
 #include <sstream>
 
@@ -27,7 +28,8 @@ using namespace std;
 #include <iostream.h>
 #endif /* TAU_DOT_H_LESS_HEADERS */
 
-#include <stdio.h> 
+
+#include <stdio.h>
 #include <fcntl.h>
 #include <time.h>
 #include <stdlib.h>
@@ -38,7 +40,7 @@ using namespace std;
   #include <vector>
 #endif //TAU_WINDOWS
 
-#ifdef TAU_VAMPIRTRACE 
+#ifdef TAU_VAMPIRTRACE
 #include <Profile/TauVampirTrace.h>
 #else /* TAU_VAMPIRTRACE */
 #ifdef TAU_EPILOG
@@ -55,7 +57,6 @@ using namespace std;
 #include <Profile/TauUtil.h>
 #include <Profile/TauPin.h>
 
-
 #include <Profile/TauPluginInternals.h>
 
 //////////////////////////////////////////////////////////////////////
@@ -64,7 +65,7 @@ using namespace std;
 // when running with fortran programs
 //////////////////////////////////////////////////////////////////////
 class FIvector : public vector<FunctionInfo*> {
-public: 
+public:
   ~FIvector() {
     Tau_destructor_trigger();
   }
@@ -92,11 +93,11 @@ vector<FunctionInfo*>& TheFunctionDB(void)
 }
 
 //////////////////////////////////////////////////////////////////////
-// It is not safe to call Profiler::StoreData() after 
+// It is not safe to call Profiler::StoreData() after
 // FunctionInfo::~FunctionInfo has been called as names are null
 //////////////////////////////////////////////////////////////////////
 int& TheSafeToDumpData()
-{ 
+{
   static int SafeToDumpData=1;
 
   return SafeToDumpData;
@@ -106,7 +107,7 @@ int& TheSafeToDumpData()
 // Set when uning Dyninst
 //////////////////////////////////////////////////////////////////////
 int& TheUsingDyninst()
-{ 
+{
   static int UsingDyninst=0;
   return UsingDyninst;
 }
@@ -115,7 +116,7 @@ int& TheUsingDyninst()
 // Set when uning Compiler Instrumentation
 //////////////////////////////////////////////////////////////////////
 int& TheUsingCompInst()
-{ 
+{
   static int UsingCompInst=0;
   return UsingCompInst;
 }
@@ -151,10 +152,11 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 {
   /* Make sure TAU is initialized */
   static bool flag = true;
+
   if (flag) {
     flag = false;
     Tau_init_initializeTAU();
-#ifdef __PIN__ 
+#ifdef __PIN__
 #if (!(defined (TAU_MPI) || defined(TAU_SHMEM)))
   if (RtsLayer::myNode() == -1) {
     TAU_PROFILE_SET_NODE(getpid());
@@ -163,9 +165,9 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
 #endif /* __PIN__ */
   }
 #ifdef TAU_SCOREP
-  if (Tau_global_getLightsOut()) { 
+  if (Tau_global_getLightsOut()) {
     TAU_VERBOSE("TAU<%d,%d>: FunctionInfoInit: Lights out... \n",RtsLayer::myNode(), tid);
-    return; 
+    return;
   }
 #endif /* TAU_SCOREP */
 
@@ -186,7 +188,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   static bool mm_initialized = Tau_MemMgr_initIfNecessary();
   TAU_UNUSED(mm_initialized);
 #endif /* _AIX */
-#endif  
+#endif
 
   GroupName = strdup(RtsLayer::PrimaryGroup(AllGroups).c_str());
 
@@ -213,7 +215,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   MyProfileGroup_ = ProfileGroup;
 
   // While accessing the global function database, lock it to ensure
-  // an atomic operation in the push_back and size() operations. 
+  // an atomic operation in the push_back and size() operations.
   // Important in the presence of concurrent threads.
   TheFunctionDB().push_back(this);
   FunctionId = RtsLayer::GenerateUniqueId();
@@ -227,7 +229,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   //         requires the use of an actual malloc
   //         while in the middle of some other malloc call.
 #ifndef TAU_WINDOWS
-#ifndef _AIX
+// #ifndef _AIX
   // create structure only if EBS is required.
   // Thread-safe, all (const char *) parameters. This check removes
   //   the need to create and allocate memory for EBS post-processed
@@ -253,7 +255,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
   //  callSiteKeyId = 0; // Any value works.
   firstSpecializedFunction = NULL;
 
-#endif // _AIX 
+// #endif // _AIX
 #endif // TAU_WINDOWS
 
 #if defined(TAU_VAMPIRTRACE)
@@ -292,13 +294,14 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
       << RtsLayer::myContext() << ", " << tid
       << " FunctionInfo::FunctionInfo(n,t) : Name : "<< GetName()
       << " Group :  " << GetProfileGroup()
+      << " Id :  " << GetFunctionId()
       << " Type : " << GetType() << endl;);
 
 #ifdef TAU_PROFILEMEMORY
   {
     char * buff = new char[strlen(Name)+strlen(Type)+100];
     sprintf(buff, "%s %s - Heap Memory Used (KB)", Name, Type);
-    MemoryEvent = new TauUserEvent(buff);
+    MemoryEvent = new tau::TauUserEvent(buff);
     delete buff;
   }
 #endif
@@ -320,7 +323,7 @@ void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, const char *Profile
     Tau_plugin_event_function_registration_data_t plugin_data;
     plugin_data.function_info_ptr = this;
     plugin_data.tid = tid;
-    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_REGISTRATION, &plugin_data);
+    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_REGISTRATION, GetName(), &plugin_data);
   }
 
   TauTraceSetFlushEvents(1);
@@ -332,9 +335,9 @@ FunctionInfo::FunctionInfo(const char *name, const char *type, TauGroup_t Profil
     const char *ProfileGroupName, bool InitData, int tid)
 {
   DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
-  Name = strdup(name); 
-  Type = strdup(type); 
-  FullName = NULL; 
+  Name = strdup(name);
+  Type = strdup(type);
+  FullName = NULL;
   DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
   FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
@@ -352,26 +355,26 @@ FunctionInfo::FunctionInfo(const char *name, const string& type, TauGroup_t Prof
 
 //////////////////////////////////////////////////////////////////////
 
-FunctionInfo::FunctionInfo(const string& name, const char * type, 
+FunctionInfo::FunctionInfo(const string& name, const char * type,
 	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
 	int tid) {
   Name = strdup(name.c_str());
   Type = strdup(type);
   FullName = NULL;
-  
+
   FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
 //////////////////////////////////////////////////////////////////////
 
-FunctionInfo::FunctionInfo(const string& name, const string& type, 
+FunctionInfo::FunctionInfo(const string& name, const string& type,
 	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
 	int tid) {
-  
+
   Name = strdup(name.c_str());
   Type = strdup(type.c_str());
   FullName = NULL;
-  
+
   FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
@@ -380,7 +383,7 @@ FunctionInfo::FunctionInfo(const string& name, const string& type,
 FunctionInfo::~FunctionInfo()
 {
 // Don't delete Name, Type - if dtor of static object dumps the data
-// after all these function objects are destroyed, it can't get the 
+// after all these function objects are destroyed, it can't get the
 // name, and type.
 //  delete [] Name;
 //  delete [] Type;
@@ -453,10 +456,10 @@ x_uint64 FunctionInfo::GetFunctionId(void) {
   }
   return FunctionId;
 }
-	    
+
 
 //////////////////////////////////////////////////////////////////////
-void FunctionInfo::ResetExclTimeIfNegative(int tid) { 
+void FunctionInfo::ResetExclTimeIfNegative(int tid) {
   /* if exclusive time is negative (at Stop) we set it to zero during
      compensation. This function is used to reset it to zero for single
      and multiple counters */
@@ -466,7 +469,7 @@ void FunctionInfo::ResetExclTimeIfNegative(int tid) {
       ExclTime[tid][i] = 0.0; /* set each negative counter to zero */
     }
   }
-  return; 
+  return;
 }
 
 
@@ -474,6 +477,9 @@ void FunctionInfo::ResetExclTimeIfNegative(int tid) {
 //////////////////////////////////////////////////////////////////////
 void tauCreateFI(void **ptr, const char *name, const char *type, TauGroup_t ProfileGroup, const char *ProfileGroupName)
 {
+  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
+   * make sure that TAU is ready to go before doing anything else! */
+  static int do_this_once = Tau_init_initializeTAU();
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
@@ -500,6 +506,9 @@ void tauCreateFI(void **ptr, const char *name, const char *type, TauGroup_t Prof
 void tauCreateFI(void **ptr, const char *name, const string& type, TauGroup_t ProfileGroup,
     const char *ProfileGroupName)
 {
+  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
+   * make sure that TAU is ready to go before doing anything else! */
+  static int do_this_once = Tau_init_initializeTAU();
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
@@ -526,6 +535,9 @@ void tauCreateFI(void **ptr, const char *name, const string& type, TauGroup_t Pr
 void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type, TauGroup_t ProfileGroup,
     const char *ProfileGroupName)
 {
+  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
+   * make sure that TAU is ready to go before doing anything else! */
+  static int do_this_once = Tau_init_initializeTAU();
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
@@ -544,7 +556,7 @@ void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type, Ta
     *ptr = Tau_MemMgr_malloc(RtsLayer::unsafeThreadId(), sizeof(FunctionInfo));
     /*  now, use the pacement new function to create a object in
      *  pre-allocated memory. NOTE - this memory needs to be explicitly
-     *  deallocated by explicitly calling the destructor. 
+     *  deallocated by explicitly calling the destructor.
      *  I think the best place for that is in the destructor for
      *  the hash table. */
     new(*ptr) FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
@@ -564,6 +576,9 @@ void tauCreateFI_signalSafe(void **ptr, const string& name, const char *type, Ta
 void tauCreateFI(void **ptr, const string& name, const char *type, TauGroup_t ProfileGroup,
     const char *ProfileGroupName)
 {
+  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
+   * make sure that TAU is ready to go before doing anything else! */
+  static int do_this_once = Tau_init_initializeTAU();
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
@@ -589,6 +604,9 @@ void tauCreateFI(void **ptr, const string& name, const char *type, TauGroup_t Pr
 void tauCreateFI(void **ptr, const string& name, const string& type, TauGroup_t ProfileGroup,
     const char *ProfileGroupName)
 {
+  /* This is the entry point into TAU from PDT-instrumented C++ codes, so
+   * make sure that TAU is ready to go before doing anything else! */
+  static int do_this_once = Tau_init_initializeTAU();
   if (*ptr == 0) {
     // Protect TAU from itself
     TauInternalFunctionGuard protects_this_function;
@@ -633,11 +651,12 @@ char const * FunctionInfo::GetFullName()
 /* EBS Sampling Profiles */
 
 #ifndef TAU_WINDOWS
-#ifndef _AIX
+//#ifndef _AIX
 void FunctionInfo::addPcSample(unsigned long *pcStack, int tid, double interval[TAU_MAX_COUNTERS])
 {
   // Add to the mmap-ed histogram. We start with a temporary conversion. This
   //   becomes unnecessary once we stop using the vector.
+  if (pathHistogram[tid] == NULL) return; 
   TauPathAccumulator * accumulator = pathHistogram[tid]->get(pcStack);
   if (accumulator == NULL) {
     /* KAH - Whoops!! We can't call "new" here, because malloc is not
@@ -663,11 +682,11 @@ void FunctionInfo::addPcSample(unsigned long *pcStack, int tid, double interval[
     }
   }
 }
-#endif // _AIX 
+//#endif // _AIX
 #endif // TAU_WINDOWS
 
 /***************************************************************************
  * $RCSfile: FunctionInfo.cpp,v $   $Author: amorris $
  * $Revision: 1.84 $   $Date: 2010/04/27 23:13:55 $
- * VERSION_ID: $Id: FunctionInfo.cpp,v 1.84 2010/04/27 23:13:55 amorris Exp $ 
+ * VERSION_ID: $Id: FunctionInfo.cpp,v 1.84 2010/04/27 23:13:55 amorris Exp $
  ***************************************************************************/
