@@ -23,7 +23,8 @@
 #include <Profile/TauMetaDataMerge.h>
 #include <Profile/TauPluginInternals.h>
 
-//#define DEBUG_PROF 1 
+// Define DEBUG_PROF if you want to debug timers
+// #define DEBUG_PROF 1
 
 #ifdef TAU_DOT_H_LESS_HEADERS
 #include <iostream>
@@ -37,7 +38,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 
-#include <stdio.h> 
+#include <stdio.h>
 #include <time.h>
 #include <stdlib.h>
 #include <limits.h>
@@ -88,6 +89,10 @@ double TauWindowsUsecD(void);
 #endif //__GNUC__
 #endif //CUPTI
 
+#ifdef TAU_ENABLE_ROCTRACER
+extern void Tau_roctracer_stop_tracing(void);
+#endif /* TAU_ENABLE_ROCTRACER */
+
 #ifdef TAU_SHMEM
 #include "shmem.h"
 extern "C" void  __real_shmem_finalize() ;
@@ -132,6 +137,10 @@ extern "C" void Tau_profile_exit_most_threads();
 extern "C" int TauCompensateInitialized(void);
 extern "C" void Tau_ompt_resolve_callsite(FunctionInfo &fi, char * resolved_address);
 
+#ifdef TAU_ENABLE_ROCM
+extern void TauFlushRocmEventsIfNecessary(int thread_id);
+#endif /* TAU_ENABLE_ROCM */
+
 x_uint64 Tau_get_firstTimeStamp();
 
 //////////////////////////////////////////////////////////////////////
@@ -219,7 +228,7 @@ static x_uint64 getTimeStamp()
 void Profiler::Start(int tid)
 {
 #ifdef DEBUG_PROF
-  TAU_VERBOSE( "[%d:%d-%d] Profiler::Start for %s (%p)\n", RtsLayer::getPid(), RtsLayer::getTid(), tid, ThisFunction->GetName(), ThisFunction);
+  TAU_VERBOSE( "[%d:%d-%d] Profiler::Start for %s (%p), node %d\n", RtsLayer::getPid(), RtsLayer::getTid(), tid, ThisFunction->GetName(), ThisFunction, RtsLayer::myNode());
 #endif
   ParentProfiler = TauInternal_ParentProfiler(tid);
 
@@ -263,7 +272,22 @@ void Profiler::Start(int tid)
   x_uint64 TimeStamp;
   // Record metrics in reverse order so wall clock metrics are recorded after PAPI, etc.
   RtsLayer::getUSecD(tid, StartTime, 1);
+
   TimeStamp = (x_uint64)StartTime[0];    // USE COUNTER1 for tracing
+  // This can happen when starting .TAU application on "virtual" GPU threads.
+  // The GPU timestamp isn't availble yet, so start is bogus.  Instead,
+  // get the timers read just after initialization.
+#ifndef TAU_SCOREP
+  if (TimeStamp == 0L) {
+    printf("Got a bogus start! %d %s\n", tid, ThisFunction->GetName());
+    TauMetrics_getDefaults(tid, StartTime, 1);
+    TimeStamp = (x_uint64)StartTime[0];    // USE COUNTER1 for tracing
+    if (TimeStamp == 0L) {
+      fprintf(stderr, "Still got a bogus start! %d %s\n", tid, ThisFunction->GetName());
+      abort();
+    }
+  }
+#endif /* !TAU_SCOREP */
 
   /********************************************************************************/
   /*** Extras ***/
@@ -276,7 +300,7 @@ void Profiler::Start(int tid)
 
   // An initialization of sorts. Call Paths (if any) will update this.
 #ifndef TAU_WINDOWS
-#ifndef _AIX 
+#ifndef _AIX
   if (TauEnv_get_callsite() == 1) {
     CallSiteAddPath(NULL, tid);
   }
@@ -346,7 +370,7 @@ void Profiler::Start(int tid)
   // If this function is not already on the call stack, put it
   if (ThisFunction->GetAlreadyOnStack(tid) == false) {
     AddInclFlag = true;
-    // We need to add Inclusive time when it gets over as 
+    // We need to add Inclusive time when it gets over as
     // it is not already on callstack.
 
     ThisFunction->SetAlreadyOnStack(true, tid);    // it is on callstack now
@@ -367,17 +391,18 @@ void Profiler::Start(int tid)
   if(Tau_plugins_enabled.function_entry) {
     Tau_plugin_event_function_entry_data_t plugin_data;
     plugin_data.timer_name = ThisFunction->GetName();
+    plugin_data.func_id = ThisFunction->GetFunctionId();
     plugin_data.timer_group = ThisFunction->GetAllGroups();
     plugin_data.tid = tid;
     plugin_data.timestamp = TimeStamp;
-    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_ENTRY, &plugin_data);
+    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_ENTRY, ThisFunction->GetName(), &plugin_data);
   }
 }
 
 void Profiler::Stop(int tid, bool useLastTimeStamp)
 {
 #ifdef DEBUG_PROF
-  TAU_VERBOSE( "[%d:%d-%d] Profiler::Stop  for %s (%p)\n", RtsLayer::getPid(), RtsLayer::getTid(), tid, ThisFunction->GetName(), ThisFunction); fflush(stderr);
+  TAU_VERBOSE( "[%d:%d-%d] Profiler::Stop  for %s (%p), node %d\n", RtsLayer::getPid(), RtsLayer::getTid(), tid, ThisFunction->GetName(), ThisFunction, RtsLayer::myNode());
 #endif
 
 /* It is possible that when the event stack gets deep, and has to be
@@ -453,6 +478,7 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
 #else
   RtsLayer::getUSecD(tid, CurrentTime);
 #endif /* TAU_TRACK_IDLE_THREADS */
+  //printf("In Stop: CurrentTime[0] = %f\n", CurrentTime[0]);
 
 #ifndef TAU_WINDOWS
 #ifndef _AIX
@@ -469,6 +495,37 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
   ThisKtauProfiler->Stop(this, AddInclFlag);
 #endif /* TAUKTAU */
 
+  // this happens during early initialization, before program load
+  //if (CurrentTime[0] == 0.0) { CurrentTime[0] = TauMetrics_getTimeOfDay(); }
+
+  // It's ok if CurrentTime is 0, because that means StartTime is too.
+  // However, if CurrentTime is not 0, we need to fix a timer that was read
+  // before we were done initializing metrics.
+//   // This code SHOULDN'T be needed any more.  but things slip through the cracks.
+//   if (CurrentTime[0] != 0.0 && StartTime[0] == 0.0) {
+//     abort();
+//     // get the CurrentTime again, but use the thread 0 context
+//     double CurrentTime_0[TAU_MAX_COUNTERS] = { 0 };
+//     RtsLayer::getUSecD(0, CurrentTime_0);
+//     // ...because the default values were captured by thread 0
+//     TauMetrics_getDefaults(tid, StartTime, 0);
+//     // ...and what we really care about is that the delta is correct.
+//     for (int k = 0; k < Tau_Global_numCounters; k++) {
+//       TotalTime[k] = CurrentTime_0[k] - StartTime[k];
+//     }
+//   } else {
+//     for (int k = 0; k < Tau_Global_numCounters; k++) {
+//       TotalTime[k] = CurrentTime[k] - StartTime[k];
+// #ifdef DEBUG_PROF
+//       printf("CurrentTime[%d] = %f\n", k, CurrentTime[k]);
+//       printf("StartTime[%d]   = %f\n", k, StartTime[k]);
+//       printf("TotalTime[%d]   = %f\n", k, TotalTime[k]);
+// #endif /* DEBUG_PROF */
+//     }
+//   }
+  if (CurrentTime[0] != 0.0 && StartTime[0] == 0.0) {
+    TauMetrics_getDefaults(tid, StartTime, 0);
+  }
   for (int k = 0; k < Tau_Global_numCounters; k++) {
     TotalTime[k] = CurrentTime[k] - StartTime[k];
   }
@@ -602,7 +659,7 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
     /* if the frequency of events is high, disable them */
     long numCalls = ThisFunction->GetCalls(tid);
     double inclusiveTime = ThisFunction->GetInclTimeForCounter(tid, 0);
-    /* here we get the array of double values representing the double 
+    /* here we get the array of double values representing the double
      metrics. We choose the first counter */
 
     /* Putting AddInclFlag means we can't throttle recursive calls */
@@ -623,7 +680,8 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
   /*** Throttling Code ***/
   /********************************************************************************/
 
-  if (!ParentProfiler) {
+  if (( TauEnv_get_recycle_threads() && (!ParentProfiler && tid == 0)) ||
+      (!TauEnv_get_recycle_threads() && !ParentProfiler)) {
     /*** Profile Compensation ***/
     // If I am still compensating, I do not expect a top level timer. Just pretend
     // this never happened.
@@ -671,7 +729,7 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
         }
 #endif /* TAU_DMAPP */
 
-#if defined(TAUKTAU) 
+#if defined(TAUKTAU)
         //AN Removed - New func inside
         //ThisKtauProfiler->KernProf.DumpKProfile();
         ThisKtauProfiler->KernProf.DumpKProfileOut();
@@ -693,6 +751,7 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
       }
     }
   }
+
   /********************************************************************************/
   /*** KTAU Code ***/
   /********************************************************************************/
@@ -704,10 +763,11 @@ void Profiler::Stop(int tid, bool useLastTimeStamp)
   if(Tau_plugins_enabled.function_exit) {
     Tau_plugin_event_function_exit_data_t plugin_data;
     plugin_data.timer_name = ThisFunction->GetName();
+    plugin_data.func_id = ThisFunction->GetFunctionId();
     plugin_data.timer_group = ThisFunction->GetAllGroups();
     plugin_data.tid = tid;
     plugin_data.timestamp = TimeStamp;
-    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_EXIT, &plugin_data);
+    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_FUNCTION_EXIT, ThisFunction->GetName(), &plugin_data);
   }
 }
 
@@ -735,6 +795,9 @@ void TauProfiler_theFunctionList(const char ***inPtr, int *numFuncs, bool addNam
 void TauProfiler_dumpFunctionNames()
 {
   TauInternalFunctionGuard protects_this_function;
+  if(!TheSafeToDumpData()) {
+    return;
+  }
 
   int numFuncs;
   const char ** functionList;
@@ -749,7 +812,7 @@ void TauProfiler_dumpFunctionNames()
 
   FILE* fp;
   if ((fp = fopen(filename, "w+")) == NULL) {
-    char errormsg[1024];
+    char errormsg[1048];
     sprintf(errormsg, "Error: Could not create %s", filename);
     perror(errormsg);
     return;
@@ -910,7 +973,7 @@ static void finalizeTrace(int tid)
     vt_close();
   }
 #else /* TAU_VAMPIRTRACE */
-#ifdef TAU_EPILOG 
+#ifdef TAU_EPILOG
   DEBUGPROFMSG("Calling elg_close()"<<endl;);
   if (RtsLayer::myThread() == 0) {
     esd_close();
@@ -982,8 +1045,8 @@ void TauProfiler_PurgeData(int tid)
 
 //////////////////////////////////////////////////////////////////////
 //  Profiler::GetNumChildren()
-//  Description: Returns the total number of child timers (including 
-//  children of children) executed under the given profiler. 
+//  Description: Returns the total number of child timers (including
+//  children of children) executed under the given profiler.
 //////////////////////////////////////////////////////////////////////
 long Profiler::GetNumChildren(void)
 {
@@ -1010,7 +1073,7 @@ void Profiler::AddNumChildren(long value)
 
 //////////////////////////////////////////////////////////////////////
 //  Profiler::GetPhase(void)
-//  Description: Returns if a profiler is a phase or not 
+//  Description: Returns if a profiler is a phase or not
 //////////////////////////////////////////////////////////////////////
 #ifdef TAU_PROFILEPHASE
 bool Profiler::GetPhase(void) {
@@ -1019,7 +1082,7 @@ bool Profiler::GetPhase(void) {
 
 //////////////////////////////////////////////////////////////////////
 //  Profiler::GetPhase(bool flag)
-//  Description: SetPhase sets a phase to be true or false based on the 
+//  Description: SetPhase sets a phase to be true or false based on the
 //               parameter flag
 //////////////////////////////////////////////////////////////////////
 void Profiler::SetPhase(bool flag) {
@@ -1027,6 +1090,13 @@ void Profiler::SetPhase(bool flag) {
 }
 #endif /* TAU_PROFILEPHASE */
 
+bool startsWith(const char *pre, const char *str)
+{
+    size_t lenpre = strlen(pre),
+           lenstr = strlen(str);
+    return lenstr < lenpre ? false : memcmp(pre, str, lenpre) == 0;
+}
+
 // writes user events to the file
 static int writeUserEvents(FILE *fp, int tid)
 {
@@ -1037,7 +1107,7 @@ static int writeUserEvents(FILE *fp, int tid)
   // Print UserEvent Data if any
   int numEvents = 0;
   for (it = TheEventDB().begin(); it != TheEventDB().end(); ++it) {
-    if ((*it) && (*it)->GetNumEvents(tid) == 0) {    // skip user events with no calls
+    if ((*it) && (*it)->GetNumEvents(tid) == 0 && !startsWith("CUDA", (*it)->GetName().c_str())) {    // skip user events with no calls
       continue;
     }
     if ((*it)->GetWriteAsMetric()) { //skip events that are written out as metrics.
@@ -1050,14 +1120,14 @@ static int writeUserEvents(FILE *fp, int tid)
   }
 
   if (numEvents > 0) {
-    // Data format 
+    // Data format
     // # % userevents
-    // # name numsamples max min mean sumsqr 
+    // # name numsamples max min mean sumsqr
     fprintf(fp, "%d userevents\n", numEvents);
     fprintf(fp, "# eventname numevents max min mean sumsqr\n");
 
     for (it = TheEventDB().begin(); it != TheEventDB().end(); ++it) {
-      if ((*it) && (*it)->GetNumEvents(tid) == 0) continue;
+      if ((*it) && (*it)->GetNumEvents(tid) == 0 && !startsWith("CUDA", (*it)->GetName().c_str())) continue;
       if ((*it) && (*it)->GetWriteAsMetric()) continue;
       fprintf(fp, "\"%s\" %ld %.16G %.16G %.16G %.16G\n", (*it)->GetName().c_str(), (*it)->GetNumEvents(tid),
           (*it)->GetMax(tid), (*it)->GetMin(tid), (*it)->GetMean(tid), (*it)->GetSumSqr(tid));
@@ -1068,7 +1138,7 @@ static int writeUserEvents(FILE *fp, int tid)
 
 static int writeHeader(FILE *fp, int numFunc, char *metricName)
 {
-  char header[256];
+  char header[2096];
   sprintf(header, "%d %s\n", numFunc, metricName);
   strcat(header, "# Name Calls Subrs Excl Incl ");
 
@@ -1131,7 +1201,7 @@ int TauProfiler_updateIntermediateStatistics(int tid)
       // *CWL* - This is stupid, but I do not currently have the time nor energy to
       //         attempt to refactor, especially since both forms are actively used.
       //         a) incltime and excltime are used for non-threaded programs
-      //            Note that getDump*Values(tid) grants pointer-access to the 
+      //            Note that getDump*Values(tid) grants pointer-access to the
       //            internal structures stored in the FunctionInfo object.
       //         b) InclTime and ExclTime are used for threaded programs.
       //            Note that InclTime and ExclTime allocates memory.
@@ -1169,7 +1239,7 @@ int TauProfiler_updateIntermediateStatistics(int tid)
 	  // done with this function, exit the stack loop
 	  break;
         } else {
-	  // get the start time for this "child" function. 
+	  // get the start time for this "child" function.
 	  // The goal is to get the start time for the immediate child of "fi".
           for (c = 0; c < Tau_Global_numCounters; c++) {
             prevStartTime[c] = currentTime[c] - current->getStartValues()[c];
@@ -1264,10 +1334,17 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
             int deviceCount;
             CUresult result = cuDeviceGetCount(&deviceCount);
             if (result != CUDA_SUCCESS) {
-                char const * err_str;
-                cuGetErrorString(result, &err_str);
-                fprintf(stderr, "cuDeviceGetCount failed: %s\n", err_str);
-                exit(result);
+                if (result == CUDA_ERROR_NOT_INITIALIZED) {
+                    cuInit(0);
+                    result = cuDeviceGetCount(&deviceCount);
+                }
+                if (result != CUDA_SUCCESS) {
+                    char const * err_str;
+                    cuGetErrorString(result, &err_str);
+                    fprintf(stderr, "cuDeviceGetCount failed: %s\n", err_str);
+                    // no device found.
+                    continue;
+                }
             }
             for(int dev=0; dev<deviceCount; ++dev) {
                 CUptiResult result;
@@ -1290,8 +1367,8 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
                 cudaGetDeviceProperties(&deviceProps, dev);
                 std::string device_name = deviceProps.name;
                 //std::replace(device_name.begin(), device_name.end(), ' ', '_');
-                // PGI compiler has some issues with c++11. 
-                Tau_util_replaceStringInPlace(device_name, " ", "_"); 
+                // PGI compiler has some issues with c++11.
+                Tau_util_replaceStringInPlace(device_name, " ", "_");
                 if (tau_cuda_device_name && strcmp(tau_cuda_device_name, device_name.c_str())) {
                     continue;
                 }
@@ -1359,10 +1436,10 @@ static int writeFunctionData(FILE *fp, int tid, int metric, const char **inFuncs
 
             /*Do not resolve addresses if they have already been resolved eagerly*/
             if(strcmp(fi.GetPrimaryGroup(), "TAU_OPENMP") == 0 && !TauEnv_get_ompt_resolve_address_eagerly()) {
-              char resolved_address[1024] = "";
+              char resolved_address[10240] = "";
               Tau_ompt_resolve_callsite(fi, resolved_address);
               fprintf(fp, "\"%s", resolved_address);
-            } else { 
+            } else {
               fprintf(fp, "\"%s", fi.GetName());
             }
             if (strlen(fi.GetType()) > 0)
@@ -1426,7 +1503,7 @@ static int getTrueFunctionCount(int count, int tid, const char **inFuncs, int nu
             trueCount += (tempCount - 1)*2; //account for noncontext event and two functions per event (callpath/noncallpath).
           }*/
         }
-  
+
     }
   }
   return trueCount;
@@ -1465,10 +1542,10 @@ extern "C" int Tau_print_metadata_for_traces(int tid) {
 
   MetaDataRepo *localRepo = NULL;
     localRepo = &(Tau_metadata_getMetaData(tid));
-  
+
    for (MetaDataRepo::iterator it = (*localRepo).begin(); it != (*localRepo).end(); it++) {
-      string metadata_str(it->first.name + string(" | ") + string(it->second->data.cval)); 
-      TAU_TRIGGER_EVENT(metadata_str.c_str(), 1.0); 
+      string metadata_str(it->first.name + string(" | ") + string(it->second->data.cval));
+      TAU_TRIGGER_EVENT(metadata_str.c_str(), 1.0);
   }
   return 0;
 }
@@ -1477,6 +1554,12 @@ extern "C" void finalizeCallSites_if_necessary();
 int TauProfiler_StoreData(int tid)
 {
   TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData\n", RtsLayer::myNode(), tid);
+  if(!TheSafeToDumpData()) {
+    return -1;
+  }
+#ifdef TAU_ENABLE_ROCM
+  TauFlushRocmEventsIfNecessary(tid);
+#endif /* TAU_ENABLE_ROCM */
   TauMetrics_finalize();
 
 #ifndef TAU_MPI
@@ -1485,11 +1568,11 @@ int TauProfiler_StoreData(int tid)
   if(Tau_plugins_enabled.pre_end_of_execution) {
     Tau_plugin_event_pre_end_of_execution_data_t plugin_data;
     plugin_data.tid = tid;
-    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_PRE_END_OF_EXECUTION, &plugin_data);
+    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_PRE_END_OF_EXECUTION, "*", &plugin_data);
   }
 #endif
 
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 1\n", RtsLayer::myNode(), tid);
+  //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 1\n", RtsLayer::myNode(), tid);
   if (TauEnv_get_tracing() && (tid == 0) && (TauEnv_get_trace_format() != TAU_TRACE_FORMAT_OTF2)) {
     Tau_print_metadata_for_traces(tid);
   }
@@ -1498,9 +1581,15 @@ int TauProfiler_StoreData(int tid)
   Tau_write_metadata_records_in_scorep(tid);
 #endif /* TAU_SCOREP */
   profileWriteCount[tid]++;
-  if ((tid != 0) && (profileWriteCount[tid] > 1)) return 0;
-
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 2\n", RtsLayer::myNode(), tid);
+  // if ((tid != 0) && (profileWriteCount[tid] > 1)) return 0;
+#if !defined(PTHREADS)
+  // Rob:  Needed to evaluate for kernels to show in profiles (ignore dreaded #2 thread)!
+  if ((tid != 0) && (profileWriteCount[tid] > 1)) {
+    TAU_VERBOSE("[Profiler]: TauProfiler_StoreData: returning, tid: %i, profileWriteCount[%i]: %i\n", tid, tid, profileWriteCount[tid]);
+    return 0;
+  }
+#endif
+  //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 2\n", RtsLayer::myNode(), tid);
   if (profileWriteCount[tid] == 10) {
     RtsLayer::LockDB();
     if (profileWriteWarningPrinted == 0) {
@@ -1524,13 +1613,14 @@ int TauProfiler_StoreData(int tid)
   }
 #endif
 
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 3\n", RtsLayer::myNode(), tid);
+  //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 3\n", RtsLayer::myNode(), tid);
 
   Tau_MemMgr_finalizeIfNecessary();
 
 #ifndef TAU_WINDOWS
-#ifndef _AIX
+// #ifndef _AIX
   if (TauEnv_get_callsite()) {
+    TAU_VERBOSE("finalizeCallSites_if_necessary: Total threads = %d\n", RtsLayer::getTotalThreads());
     finalizeCallSites_if_necessary();
   }
 
@@ -1538,19 +1628,27 @@ int TauProfiler_StoreData(int tid)
     // Tau_sampling_finalize(tid);
     Tau_sampling_finalize_if_necessary(tid);
   }
-#endif /* _AIX */
+// #endif /* _AIX */
 #endif
   if (TauEnv_get_profiling()) {
-    Tau_snapshot_writeFinal("final");
+    if (TauEnv_get_profile_format() == TAU_FORMAT_SNAPSHOT) {
+      Tau_snapshot_writeFinal("final");
+	}
     if (TauEnv_get_profile_format() == TAU_FORMAT_PROFILE) {
       TauProfiler_DumpData(false, tid, "profile");
 	}
   }
-#if defined(PTHREADS) || defined(TAU_OPENMP)
-  if (RtsLayer::myThread() == 0 && tid == 0) {
+  /* If we have thread recycling enabled, threads won't write
+   * their profiles when they exit.  So thread 0 has to do it,
+   * even in cases where CUDA is used without pthread or openmp
+   * support.  For some reason, thread 0 is getting its myThread()
+   * value changed from 0, still need to investigate that. */
+    if (RtsLayer::myThread() == 0 && tid == 0) {
     /* clean up other threads? */
-    for (int i = 1; i < TAU_MAX_THREADS; i++) {
-      if (TauInternal_ParentProfiler(i) != (Profiler *)NULL) {
+    for (int i = 1; i < RtsLayer::getTotalThreads(); i++) {
+      TAU_VERBOSE("Thread 0 checking other threads... i = %d\n", i);
+      if (TauInternal_CurrentProfiler(i)) {
+        TAU_VERBOSE("Thread 0 writing data for thread %d\n", i);
         TauProfiler_StoreData(i);
       }
     }
@@ -1567,8 +1665,7 @@ int TauProfiler_StoreData(int tid)
 #endif
 #endif
   }
-#endif /* PTHREADS */
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 4\n", RtsLayer::myNode(), tid);
+  //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 4\n", RtsLayer::myNode(), tid);
 
 #if defined(TAU_SHMEM) && !defined(TAU_MPI)
   if (TauEnv_get_profile_format() == TAU_FORMAT_MERGED) {
@@ -1581,20 +1678,26 @@ int TauProfiler_StoreData(int tid)
   }
 #endif /* TAU_SHMEM */
 
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 5\n", RtsLayer::myNode(), tid);
+  //TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 5\n", RtsLayer::myNode(), tid);
   /*Invoke plugins only if both plugin path and plugins are specified
    *Do this first, because the plugin can write TAU_METADATA as recommendations to the user*/
-  if(Tau_plugins_enabled.end_of_execution) {
+  if(RtsLayer::myThread() == 0 && tid == 0 && Tau_plugins_enabled.end_of_execution) {
     Tau_plugin_event_end_of_execution_data_t plugin_data;
     plugin_data.tid = tid;
-    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_END_OF_EXECUTION, &plugin_data);
+    Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_END_OF_EXECUTION, "*", &plugin_data);
   }
-  TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 6\n", RtsLayer::myNode(), tid);
+  ////TAU_VERBOSE("TAU<%d,%d>: TauProfiler_StoreData 6\n", RtsLayer::myNode(), tid);
 /* static dtors cause a crash. This could fix it */
 #ifdef TAU_SCOREP
   TAU_VERBOSE("TAU<%d,%d>: Turning off the lights... \n", RtsLayer::myNode(), tid);
   Tau_global_setLightsOut();
-#endif /* TAU_SCOREP */  
+#endif /* TAU_SCOREP */
+
+
+#ifdef TAU_ENABLE_ROCTRACER
+  Tau_roctracer_stop_tracing();
+#endif /* TAU_ENABLE_ROCTRACER */
+
   return 1;
 }
 
@@ -1612,7 +1715,7 @@ static int getProfileLocation(int metric, char *str)
     profiledir = TauEnv_get_profiledir();
 #endif
 
-    if (Tau_Global_numCounters <= 1) {
+    if (Tau_Global_numCounters - Tau_Global_numGPUCounters <= 1) {
         sprintf(str, "%s", profiledir);
     } else {
 #ifdef DEBUGPROF
@@ -1638,6 +1741,9 @@ static int getProfileLocation(int metric, char *str)
 int TauProfiler_DumpData(bool increment, int tid, const char *prefix)
 {
   TAU_VERBOSE("TAU<%d,%d>: TauProfiler_DumpData\n", RtsLayer::myNode(), tid);
+  if(!TheSafeToDumpData()) {
+    return -1;
+  }
 
   int rc = TauProfiler_writeData(tid, prefix, increment);
 
@@ -1652,16 +1758,24 @@ void getMetricHeader(int i, char *header)
 // Stores profile data
 int TauProfiler_writeData(int tid, const char *prefix, bool increment, const char **inFuncs, int numFuncs)
 {
+  if(!TheSafeToDumpData()) {
+    return -1;
+  }
 
   TauProfiler_updateIntermediateStatistics(tid);
 
   RtsLayer::LockDB();
 
-  static bool createFlag = TauProfiler_createDirectories();
-  if (createFlag) {
-    TAU_VERBOSE ("Profile directories created\n");
-  }
+  //If we haven't created any directories yet go ahead and keep checking until we have. Otherwise we may give up before initializing metrics
+  static bool createdDirectories=false;
+  bool createFlag=false;
 
+  if(!createdDirectories){
+    createFlag = TauProfiler_createDirectories();
+      if (createFlag) {
+        createdDirectories=true;
+      }
+   }
 //#ifdef CUPTI
 //  CUdevice device;
 //  int retval;
@@ -1698,6 +1812,9 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
       FILE* fp;
 
       getMetricHeader(i, metricHeader);
+      if (TauMetrics_getIsCuptiMetric(i) == TAU_METRIC_CUPTI_METRIC) continue;
+      if (TauMetrics_getIsCuptiMetric(i) == TAU_METRIC_CUPTI_EVENT) continue;
+
       //cout << "metric name: " << metricHeader << endl;
 #ifdef CUPTI
       // Is a Cupti event, do not record
@@ -1724,7 +1841,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
       //}
 #endif //CUPTI
       getProfileLocation(i, profileLocation);
-//       sprintf(filename, "%s/temp.%d.%d.%d", profileLocation, 
+//       sprintf(filename, "%s/temp.%d.%d.%d", profileLocation,
 //         RtsLayer::myNode(), RtsLayer::myContext(), tid);
 
       const char *selectivePrefix = "";
@@ -1732,7 +1849,7 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
         selectivePrefix = "sel_";
       }
 
-      char dumpfile[1024];
+      char dumpfile[1128];
       if (increment) {
         // place date and time in the filename
         time_t theTime = time(NULL);
@@ -1745,14 +1862,14 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
         char *year = strtok(NULL, " ");
         // remove newline
         year[4] = '\0';
-        char newStringTime[1024];
+        char newStringTime[64];
         sprintf(newStringTime, "%s-%s-%s-%s-%s", day, month, dayInt, time, year);
 
         sprintf(dumpfile, "%s/%s%s__%s__.%d.%d.%d", profileLocation, selectivePrefix, prefix, newStringTime,
             RtsLayer::myNode(), RtsLayer::myContext(), tid);
 
         if ((fp = fopen(dumpfile, "w+")) == NULL) {
-          char errormsg[1024];
+          char errormsg[1256];
           sprintf(errormsg, "Error: Could not create %s", dumpfile);
           perror(errormsg);
           return 0;
@@ -1793,21 +1910,21 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
             test = open(dumpfile, flags, mode);
           }
           if ((fp = fdopen(test, "w")) == NULL) {
-            char errormsg[1024];
+            char errormsg[1256];
             sprintf(errormsg, "Error: Could not create %s", dumpfile);
             perror(errormsg);
             return 0;
           }
 
         } else {
-#ifdef TAU_MPI 
-#ifndef TAU_SHMEM 
-        if (Tau_get_usesMPI()) 
+#ifdef TAU_MPI
+#ifndef TAU_SHMEM
+        if (Tau_get_usesMPI())
 #endif /* TAU_SHMEM */
         {
 #endif /* TAU_MPI */
           if ((fp = fopen(dumpfile, "w+")) == NULL) {
-            char errormsg[1024];
+            char errormsg[1256];
             sprintf(errormsg, "Error: Could not create %s", dumpfile);
             perror(errormsg);
             return 0;
@@ -1826,21 +1943,8 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
           TAU_VERBOSE("[pid=%d], TAU: Writing B profile %s, cwd = %s\n", RtsLayer::getPid(), dumpfile, cwd);
         }
       }
-#ifdef TAU_MPI
-      if (Tau_get_usesMPI()) {
-#endif /* TAU_MPI */
-        TAU_VERBOSE("[pid=%d], TAU: Uses MPI Rank=%d\n", RtsLayer::getPid(), RtsLayer::myNode());
-        writeProfile(fp, metricHeader, tid, i, inFuncs, numFuncs);
-#ifdef TAU_MPI
-      } else {
-#ifdef TAU_SHMEM
-        writeProfile(fp, metricHeader, tid, i, inFuncs, numFuncs);
-#else /* TAU_SHMEM */
-        printf("TAU: WARNING! An MPI configuration was used in TAU, but MPI_Init was not called. No data will be written for pid=%d.\n", getpid()); 
-	//printf("Node = %d\n", RtsLayer::myNode());
-#endif /* TAU_SHMEM */
-      }
-#endif /* TAU_MPI */
+      TAU_VERBOSE("[pid=%d], TAU: Uses MPI Rank=%d\n", RtsLayer::getPid(), RtsLayer::myNode());
+      writeProfile(fp, metricHeader, tid, i, inFuncs, numFuncs);
     }
   }
 
@@ -1852,6 +1956,9 @@ int TauProfiler_writeData(int tid, const char *prefix, bool increment, const cha
 int TauProfiler_dumpFunctionValues(const char **inFuncs, int numFuncs, bool increment, int tid, const char *prefix)
 {
   TauInternalFunctionGuard protects_this_function;
+  if(!TheSafeToDumpData()) {
+    return -1;
+  }
 
   TAU_PROFILE("TAU_DUMP_FUNC_VALS()", " ", TAU_IO);
 
@@ -1863,6 +1970,8 @@ int TauProfiler_dumpFunctionValues(const char **inFuncs, int numFuncs, bool incr
 bool TauProfiler_createDirectories()
 {
     char newdirname[1024];
+    int countDirs=0;
+    TAU_VERBOSE("Creating Directories\n");
 #ifdef KTAU_NG
     getProfileLocation(0, newdirname);
     mkdir(newdirname, S_IRWXU | S_IRGRP | S_IXGRP);
@@ -1870,6 +1979,7 @@ bool TauProfiler_createDirectories()
     for (int i = 0; i < Tau_Global_numCounters; i++) {
         if (TauMetrics_getMetricUsed(i)) {
             getProfileLocation(i, newdirname);
+	    countDirs++;
 #ifdef TAU_WINDOWS
             mkdir(newdirname);
 #else
@@ -1878,11 +1988,15 @@ bool TauProfiler_createDirectories()
         }
     }
 #endif
+    if(countDirs==0)
+    {
+	return false;
+    }
     return true;
 }
 
 /***************************************************************************
  * $RCSfile: Profiler.cpp,v $   $Author: sameer $
  * $Revision: 1.271 $   $Date: 2010/05/25 23:06:19 $
- * VERSION_ID: $Id: Profiler.cpp,v 1.271 2010/05/25 23:06:19 sameer Exp $ 
+ * VERSION_ID: $Id: Profiler.cpp,v 1.271 2010/05/25 23:06:19 sameer Exp $
  ***************************************************************************/
