@@ -30,11 +30,11 @@
 #include <dirent.h>
 #include <stdint.h>
 
-#if defined(HAVE_GNU_DEMANGLE) && HAVE_GNU_DEMANGLE
+#if defined(HAVE_GNU_DEMANGLE)
 #define HAVE_DECL_BASENAME 1
 #include <demangle.h>
 #define DEFAULT_DEMANGLE_FLAGS DMGL_PARAMS | DMGL_ANSI | DMGL_VERBOSE | DMGL_TYPES
-#ifdef __PGI
+#if defined(__PGI) && defined(DMGL_ARM)
 #define DEMANGLE_FLAGS DEFAULT_DEMANGLE_FLAGS | DMGL_ARM
 #else
 #define DEMANGLE_FLAGS DEFAULT_DEMANGLE_FLAGS
@@ -61,7 +61,28 @@
 #include <elf-bfd.h>
 #endif
 
+#ifdef TAU_DWARF
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <errno.h>
+#include <dwarf.h>
+#include <libdwarf.h>
+#endif
 
+/* When BFD 2.34 was released, some API calls were replaced. */
+#if !defined(bfd_get_section)
+#define TAU_BFD_GET_SECTION_FLAGS(_abfd, _section) bfd_section_flags(_section)
+#define TAU_BFD_GET_SECTION_VMA(_abfd, _section) bfd_section_vma(_section)
+#define TAU_BFD_GET_SECTION_SIZE(_section) bfd_section_size(_section)
+#define TAU_BFD_GET_SECTION(_symbol) bfd_asymbol_section(_symbol)
+#else
+#define TAU_BFD_GET_SECTION_FLAGS(_abfd, _section) bfd_get_section_flags(_abfd, _section)
+#define TAU_BFD_GET_SECTION_VMA(_abfd, _section) bfd_get_section_vma(_abfd, _section)
+#define TAU_BFD_GET_SECTION_SIZE(_section) bfd_get_section_size(_section)
+#define TAU_BFD_GET_SECTION(_symbol) bfd_get_section(_symbol)
+#endif
 
 using namespace std;
 
@@ -69,9 +90,9 @@ static char const * Tau_bfd_internal_getExecutablePath();
 
 struct TauBfdModule
 {
-  TauBfdModule() :
+  TauBfdModule(const std::string & n = "") :
       bfdImage(NULL), syms(NULL), nr_all_syms(0), dynamic(false), bfdOpen(false),
-      lastResolveFailed(false), processCode(TAU_BFD_SYMTAB_NOT_LOADED), textOffset(0)
+      lastResolveFailed(false), processCode(TAU_BFD_SYMTAB_NOT_LOADED), textOffset(0), name(n)
   { }
 
   ~TauBfdModule() {
@@ -146,7 +167,7 @@ struct TauBfdModule
       if (!size) {
         TAU_VERBOSE("loadSymbolTable: Cannot get symbol table size [%s]\n", path);
         return (bfdOpen = false);
-      } 
+      }
     }
 
     syms = (asymbol **)malloc(size);
@@ -164,7 +185,7 @@ struct TauBfdModule
     if (bfd_get_flavour(bfdImage) == bfd_target_elf_flavour) {
       Elf_Internal_Phdr * elf_pheader = elf_tdata(bfdImage)->phdr;
       if(elf_pheader != NULL) {
-        unsigned int num_segments = elf_elfheader(bfdImage)->e_phnum;  
+        unsigned int num_segments = elf_elfheader(bfdImage)->e_phnum;
         for(unsigned int i = 0; i < num_segments; i++, elf_pheader++) {
           if(elf_pheader->p_type != PT_LOAD) {
             // Only care about LOAD segments
@@ -202,6 +223,8 @@ struct TauBfdModule
 
   // The virtual offset at which the text segment is loaded
   bfd_vma textOffset;
+
+  std::string name;
 };
 
 struct TauBfdUnit
@@ -209,6 +232,7 @@ struct TauBfdUnit
   TauBfdUnit() : objopen_counter(-1) {
     executablePath = Tau_bfd_internal_getExecutablePath();
     executableModule = new TauBfdModule;
+    executableModule->name = std::string(Tau_bfd_internal_getExecutablePath());
   }
 
   void ClearMaps() {
@@ -245,6 +269,26 @@ struct LocateAddressData
   TauBfdInfo & info;
 };
 
+struct SymbolTableLineNumMap : public map<string, int>
+{
+  SymbolTableLineNumMap() {}
+  virtual ~SymbolTableLineNumMap() {
+    //Wait! We might not be done! Unbelieveable as it may seem, this map
+    //could (and does sometimes) get destroyed BEFORE we have resolved the addresses. Bummer.
+    Tau_destructor_trigger();
+  }
+};
+
+static SymbolTableLineNumMap & TheSymbolTableLineNumMap() {
+  static SymbolTableLineNumMap map;
+  return map;
+}
+
+static SymbolTableLineNumMap & TheCachedSymbolTableLineNumMap() {
+  static SymbolTableLineNumMap map;
+  return map;
+}
+
 // Internal function prototypes
 static bool Tau_bfd_internal_loadSymTab(TauBfdUnit *unit, int moduleIndex);
 static bool Tau_bfd_internal_loadExecSymTab(TauBfdUnit *unit);
@@ -262,24 +306,13 @@ static void Tau_bfd_internal_updateBGPMaps(TauBfdUnit *unit);
 // ensure that non-local static variables are initialised before being
 // used (Ref: Scott Meyers, Item 47 Eff. C++).
 //////////////////////////////////////////////////////////////////////
-extern "C" void Tau_sampling_finalize_if_necessary(int tid);
-extern "C" void finalizeCallSites_if_necessary(void);
 struct bfd_unit_vector_t : public std::vector<TauBfdUnit*>
 {
   bfd_unit_vector_t() {}
   virtual ~bfd_unit_vector_t() {
   //Wait! We might not be done! Unbelieveable as it may seem, this object
   //could (and does sometimes) get destroyed BEFORE we have resolved the addresses. Bummer.
-#ifndef TAU_WINDOWS
-    if (TauEnv_get_callsite()) {
-      finalizeCallSites_if_necessary();
-    }
-
-    if (TauEnv_get_ebs_enabled()) {
-      // Tau_sampling_finalize(tid);
-      Tau_sampling_finalize_if_necessary(Tau_get_local_tid());
-    }
-#endif
+  Tau_destructor_trigger();
   }
 };
 
@@ -385,7 +418,7 @@ static void Tau_bfd_internal_updateProcSelfMaps(TauBfdUnit *unit)
 #if (defined (TAU_BGP) || defined(TAU_BGQ) || (TAU_WINDOWS))
   /* do nothing */
   // *JCL* - Windows has no /proc filesystem, so don't try to use it
-#else 
+#else
 
   // Note: Linux systems only.
   FILE * mapsfile = fopen("/proc/self/maps", "r");
@@ -412,7 +445,7 @@ static void Tau_bfd_internal_updateProcSelfMaps(TauBfdUnit *unit)
       TAU_VERBOSE("[%d] Module: %s, %p-%p (%d)\n",
           count++, module, start, end, offset);
       unit->addressMaps.push_back(new TauBfdAddrMap(start, end, offset, module));
-      unit->modules.push_back(new TauBfdModule);
+      unit->modules.push_back(new TauBfdModule(std::string(module)));
     }
   }
   fclose(mapsfile);
@@ -444,7 +477,7 @@ static int Tau_bfd_internal_BGP_dl_iter_callback(struct dl_phdr_info * info, siz
   TauBfdAddrMap * map = new TauBfdAddrMap(start, start + max_addr, 0, info->dlpi_name);
   TAU_VERBOSE("BG Module: %s, %p-%p (%d)\n", map->name, map->start, map->end, map->offset);
   unit->addressMaps.push_back(map);
-  unit->modules.push_back(new TauBfdModule);
+  unit->modules.push_back(new TauBfdModule(std::string(map->name)));
   return 0;
 }
 
@@ -520,7 +553,7 @@ static void Tau_bfd_internal_updateWindowsMaps(TauBfdUnit *unit)
     TAU_VERBOSE("[%d] Module: %s, %p-%p (%d)\n", count++, map->name, map->start, map->end, map->offset);
 
     unit->addressMaps.push_back(map);
-    unit->modules.push_back(new TauBfdModule);
+    unit->modules.push_back(new TauBfdModule(std::string(map->name)));
   }
 
   // Release the process handle
@@ -588,7 +621,7 @@ TauBfdAddrMap const * Tau_bfd_getAddressMap(tau_bfd_handle_t handle, unsigned lo
 static char const * Tau_bfd_internal_tryDemangle(bfd * bfdImage, char const * funcname)
 {
   char const * demangled = NULL;
-#if defined(HAVE_GNU_DEMANGLE) && HAVE_GNU_DEMANGLE
+#if defined(HAVE_GNU_DEMANGLE)
   if (funcname && bfdImage) {
     // Some compilers prepend .text. to the symbol name
     if (strncmp(funcname, ".text.", 6) == 0) {
@@ -726,9 +759,9 @@ bool Tau_bfd_resolveBfdInfo(tau_bfd_handle_t handle, unsigned long probeAddr, Ta
 #endif /* TAU_INTEL12 */
     info.funcname = Tau_bfd_internal_tryDemangle(module->bfdImage, info.funcname);
     return true;
-  } 
+  }
 
-  // Data wasn't found, so check every symbol's address directly to see if it matches.  
+  // Data wasn't found, so check every symbol's address directly to see if it matches.
   // If so, try to get the function name from the symbol name.
   for (asymbol ** s = module->syms; *s; s++) {
     asymbol const & asym = **s;
@@ -762,7 +795,7 @@ bool Tau_bfd_resolveBfdInfo(tau_bfd_handle_t handle, unsigned long probeAddr, Ta
 
   TAU_VERBOSE("result: %p, %s, %s, %d\n", probeAddr, info.funcname, info.filename, info.lineno);
 
-  // we might have partial information, like filename and line number. 
+  // we might have partial information, like filename and line number.
   // MOST LIKELY this is an outlined region or some other code that the compiler
   // generated.
   if ((info.funcname == NULL) && (info.filename != NULL) && (info.lineno > 0)) {
@@ -898,6 +931,7 @@ static bool Tau_bfd_internal_loadSymTab(TauBfdUnit *unit, int moduleIndex)
   char const * name = unit->addressMaps[moduleIndex]->name;
   TauBfdModule * module = Tau_bfd_internal_getModuleFromIdx(unit, moduleIndex);
 
+  TAU_VERBOSE("TAU_BFD: Tau_bfd_internal_loadSymTab: name=%s, moduleIndex=%d\n", name, moduleIndex);
   return module->loadSymbolTable(name);
 }
 
@@ -1003,19 +1037,19 @@ static void Tau_bfd_internal_locateAddress(bfd * bfdptr, asection * section, voi
   }
 
   // Skip this section if it isn't a debug info section
-  if ((bfd_get_section_flags(bfdptr, section) & SEC_ALLOC) == 0) {
+  if ((TAU_BFD_GET_SECTION_FLAGS(bfdptr, section) & SEC_ALLOC) == 0) {
     return;
   }
 
 
   // Skip this section if the address is before the section start
-  bfd_vma vma = bfd_get_section_vma(bfdptr, section);
+  bfd_vma vma = TAU_BFD_GET_SECTION_VMA(bfdptr, section);
   if (data.info.probeAddr < vma) {
       return;
   }
 
   // Skip this section if the address is after the section end
-  bfd_size_type size = bfd_get_section_size(section);
+  bfd_size_type size = TAU_BFD_GET_SECTION_SIZE(section);
   if (data.info.probeAddr >= vma + size) {
     return;
   }
@@ -1039,4 +1073,428 @@ static void Tau_bfd_internal_locateAddress(bfd * bfdptr, asection * section, voi
   return;
 }
 
+#ifdef TAU_DWARF
+
+static void
+simple_error_handler(Dwarf_Error error, Dwarf_Ptr errarg)
+{
+    (void)errarg; // intentionally unused
+    printf("\nTAU: libdwarf error detected: 0x%" DW_PR_DUx " %s\n",
+        dwarf_errno(error),dwarf_errmsg(error));
+    return;
+}
+
+
+static void Tau_get_dwarf_line_number(tau_bfd_handle_t bfd_handle, Dwarf_Debug dbg, Dwarf_Die die, map<string, int> & sym_map, bfd * bfdImage) {
+    // This retrieves the line numbers for each DIE that represents a subprogram (function/routine).
+    char * name = NULL;
+    Dwarf_Half tag = 0;
+    const char * tagname = NULL;
+    int noname = 0;
+    int res = 0;
+    Dwarf_Error * errp = NULL;
+
+    res = dwarf_tag(die, &tag, errp);
+    if(res != DW_DLV_OK) {
+        printf("TAU: Error in dwarf_tag\n");
+        return;
+    }
+    if(tag != DW_TAG_subprogram) {
+        return; // Only care about subprograms
+    }
+
+    // There are two ways that the name can be stored. One is directly as the name of the DIE,
+    // which is retrieved using a dedicated function dwarf_diename
+    // This should be a "human readable" name
+    res = dwarf_diename(die, &name,errp);
+    if(res == DW_DLV_ERROR) {
+        printf("TAU: Error in dwarf_diename\n");
+        return;
+    }
+    if(res == DW_DLV_NO_ENTRY) {
+        name = NULL;
+        noname = 1;
+    }
+    res = dwarf_get_TAG_name(tag, &tagname);
+    if(res != DW_DLV_OK) {
+        printf("TAU: Error in dwarf_get_TAG_name\n");
+        return;
+    }
+
+    // The other is the linkage name. This should be the mangled name used by the linker.
+    Dwarf_Attribute linkage_name;
+    res = dwarf_attr(die, DW_AT_linkage_name, &linkage_name, errp);
+    char * linkage_name_str = NULL;
+    if(res == DW_DLV_OK) {
+        res = dwarf_formstring(linkage_name, &linkage_name_str, errp);
+        if( res != DW_DLV_OK) {
+            fprintf(stderr, "TAU: Error getting linkage name string\n");
+        }
+    }
+
+    // The line number corresponding to the function declaration is stored in the DW_AT_decl_line field
+    Dwarf_Attribute line_number;
+    res = dwarf_attr(die, DW_AT_decl_line, &line_number, errp);
+    Dwarf_Unsigned line_number_u = 0;
+    if(res == DW_DLV_OK) {
+        res = dwarf_formudata(line_number, &line_number_u, errp);
+        if(res != DW_DLV_OK) {
+            line_number_u = 0;
+        }
+    }
+
+    // If there isn't a DW_AT_decl_line field, try looking up the function line number by address.
+    // This will help get a source line for compiler-generated ("artificial") functions.
+    if((linkage_name_str || !noname) && (line_number_u == 0)) {
+        Dwarf_Addr low_pc_addr = 0;
+        res = dwarf_lowpc(die, &low_pc_addr, errp);
+        if(res == DW_DLV_OK) {
+            TauBfdInfo bfd_info;
+            const bool resolved = Tau_bfd_resolveBfdInfo(bfd_handle, low_pc_addr, bfd_info);
+            if(resolved) {
+                if(bfd_info.lineno > 0) {
+                    line_number_u = bfd_info.lineno;
+                }
+            }
+        }
+    }
+
+#ifdef DEBUG_PROF
+    fprintf(stderr, "Name: %s, Linkage name: %s, line = %" DW_PR_DUu "\n", name, linkage_name_str, line_number_u);
+#endif
+
+    // Add linkage name to the map, if it exists.
+    if(linkage_name_str) {
+        sym_map[std::string(linkage_name_str)] = line_number_u;
+        const char * demangled_name_str = Tau_bfd_internal_tryDemangle(bfdImage, linkage_name_str);
+        if(demangled_name_str != NULL) {
+            sym_map[std::string(demangled_name_str)] = line_number_u;
+        }
+    }
+
+    // Add regular name to the map, if it exists.
+    if(!noname) {
+        sym_map[std::string(name)] = line_number_u;
+    }
+
+    if(!noname) {
+        dwarf_dealloc(dbg, name, DW_DLA_STRING);
+    }
+}
+
+static void Tau_process_debug_symbols(tau_bfd_handle_t bfd_handle, Dwarf_Debug dbg, Dwarf_Die in_die, int is_info, map<string, int> & sym_map, bfd * bfdImage) {
+    int res = DW_DLV_ERROR;
+    Dwarf_Die cur_die=in_die;
+    Dwarf_Die child = 0;
+    Dwarf_Error *errp = 0;
+
+    // Get any line numbers from the current DIE
+    Tau_get_dwarf_line_number(bfd_handle, dbg, in_die, sym_map, bfdImage);
+
+    // This loop recursively processes all the children of this DIE, then the next sibling of this DIE
+    for(;;) {
+        Dwarf_Die sib_die = 0;
+        res = dwarf_child(cur_die, &child, errp);
+        if(res == DW_DLV_ERROR) {
+            printf("Error in dwarf_child\n");
+            return;
+        }
+        if(res == DW_DLV_OK) {
+            // Get line numbers from the child and its siblings
+            Tau_process_debug_symbols(bfd_handle, dbg, child, is_info, sym_map, bfdImage);
+            dwarf_dealloc(dbg, child, DW_DLA_DIE);
+            child = 0;
+        }
+        res = dwarf_siblingof_b(dbg, cur_die, is_info, &sib_die, errp);
+        if(res == DW_DLV_ERROR) {
+            printf("Error in dwarf_siblingof_b\n");
+            return;
+        }
+        if(res == DW_DLV_NO_ENTRY) {
+            break;
+        }
+        if(cur_die != in_die) {
+            dwarf_dealloc(dbg, cur_die, DW_DLA_DIE);
+            cur_die = 0;
+        }
+        cur_die = sib_die;
+        // Get line numbers from the sibling DIE
+        Tau_get_dwarf_line_number(bfd_handle, dbg, cur_die, sym_map, bfdImage);
+    }
+    return;
+}
+
+static void Tau_get_dwarf_symbols(tau_bfd_handle_t bfd_handle, const char * filename, map<string, int> & sym_map, bfd * bfdImage) {
+
+    Dwarf_Debug dbg = 0;
+    int fd = -1;
+    const char * filepath = filename;
+    int res = DW_DLV_ERROR;
+
+    Dwarf_Handler errhand = simple_error_handler;
+    Dwarf_Ptr errarg = (Dwarf_Ptr)1;
+    Dwarf_Error * errp = 0;
+
+    fd = open(filepath, O_RDONLY);
+    if(fd < 0) {
+        TAU_VERBOSE("TAU_BFD: Unable to open file %s\n", filepath);
+        return;
+    }
+
+    res = dwarf_init(fd, DW_DLC_READ, errhand, errarg, &dbg, errp);
+
+    if(res != DW_DLV_OK) {
+        fprintf(stderr, "Error opening DWARF session: %d\n", res);
+        return;
+    }
+
+    Dwarf_Unsigned cu_header_length = 0;
+    Dwarf_Unsigned abbrev_offset = 0;
+    Dwarf_Half     address_size = 0;
+    Dwarf_Half     version_stamp = 0;
+    Dwarf_Half     offset_size = 0;
+    Dwarf_Half     extension_size = 0;
+    Dwarf_Sig8     signature;
+    Dwarf_Unsigned typeoffset = 0;
+    Dwarf_Unsigned next_cu_header = 0;
+    Dwarf_Half     header_cu_type = DW_UT_compile;
+    Dwarf_Bool     is_info = 1;
+    int cu_number = 0;
+
+    // This loop iterates over all the Compilation Units
+    for(;;++cu_number) {
+        Dwarf_Die no_die = 0;
+        Dwarf_Die cu_die = 0;
+        memset(&signature,0, sizeof(signature));
+
+        res = dwarf_next_cu_header_d(dbg, is_info, &cu_header_length, &version_stamp, &abbrev_offset,
+                &address_size, &offset_size, &extension_size, &signature, &typeoffset, &next_cu_header,
+                &header_cu_type, errp);
+        if(res == DW_DLV_ERROR) {
+            fprintf(stderr, "TAU: Error in dwarf_next_cu_header: %d\n", res);
+            return;
+        }
+        if(res == DW_DLV_NO_ENTRY) {
+            /* Done. */
+            break;
+        }
+        // Each CU has one sibling, a CU_DIE (Debugging Information Entity)
+        res = dwarf_siblingof_b(dbg, no_die, is_info, &cu_die, errp);
+        if(res == DW_DLV_ERROR) {
+            fprintf(stderr, "TAU: Error in dwarf_siblingof_b on CU die: %d\n", res);
+            return;
+        }
+
+        // Process the DIE of the CU
+        Tau_process_debug_symbols(bfd_handle, dbg, cu_die, is_info, sym_map, bfdImage);
+
+        dwarf_dealloc(dbg, cu_die, DW_DLA_DIE);
+    }
+
+
+    res = dwarf_finish(dbg, errp);
+    if(res != DW_DLV_OK) {
+        fprintf(stderr, "TAU: Error closing DWARF session: %d\n", res);
+        return;
+    }
+
+    res = close(fd);
+    if(res != 0) {
+        fprintf(stderr, "TAU: Error closing dwarf file\n");
+    }
+}
+
+#endif // TAU_DWARF
+
+// Given a function name, this routine iterates through the list of symbols and
+// returns the line number associated with the function name. It needs to
+// cache this information - it is not efficient.
+static int Tau_internal_get_lineno_for_function(tau_bfd_handle_t bfd_handle, char const * funcname) {
+  if (TauEnv_get_lite_enabled()) return 0;
+
+  static bool first_time = true;
+  map<string, int>::iterator cit, fit;
+  int lno = 0;
+
+  if (!first_time) {
+    // See if the funcname has appeared before
+    cit = TheCachedSymbolTableLineNumMap().find(funcname);
+    if (cit == TheCachedSymbolTableLineNumMap().end()) {
+      TAU_VERBOSE("TAU_BFD: Didn't find %s in the cached_symtab\n", funcname);
+      // Let us search for it in the full_symtab.
+      fit = TheSymbolTableLineNumMap().find(funcname);
+      if (fit == TheSymbolTableLineNumMap().end()) {
+        TAU_VERBOSE("TAU_BFD: Didn't find %s in the full_symtab either!\n", funcname);
+        return 0; /* didn't find it */
+      } else { // found it in the full_symtab!
+        // add it to the cached entry first!
+        lno = fit->second;
+        TheCachedSymbolTableLineNumMap()[funcname] = lno;
+        TAU_VERBOSE("TAU_BFD: Adding: cached_symtab[%s] = %d\n", funcname, lno);
+	return lno; // line number
+      }
+    } else {
+        // Found the function name in the cached_symtab!
+        lno = cit->second;
+        TAU_VERBOSE("TAU_BFD: Found: cached_symtab[%s] = %d\n", funcname, lno);
+        return lno;
+    }
+  }
+  // This is the first time
+  // reset the flag. Acquire lock?
+  first_time = false;
+
+  TauBfdUnit * unit = ThebfdUnits()[bfd_handle];
+  int result_line = 0;
+#ifdef TAU_DWARF
+  // If we have libdwarf, use it; it is much faster than libbfd
+
+  if(unit != NULL) {
+    for(vector<TauBfdModule*>::iterator it = unit->modules.begin(); it != unit->modules.end(); ++it) {
+      TauBfdModule * module = *it;
+      if(module != NULL) {
+        bfd * bfdImage = module->bfdImage;
+        if(bfdImage == NULL) {
+          TAU_VERBOSE("TAU_BFD: Forcing load of symbol table for %s\n", module->name.c_str());
+          module->loadSymbolTable(module->name.c_str());
+          bfdImage = module->bfdImage;
+          if(bfdImage == NULL) {
+            TAU_VERBOSE("TAU_BFD: Skipping %s because its symbol table couldn't be loaded.\n", module->name.c_str());
+            continue;
+          }
+        }
+        TAU_VERBOSE("TAU_BFD: Will process symbols in %s using libdwarf\n", module->name.c_str());
+        Tau_get_dwarf_symbols(bfd_handle, module->name.c_str(), TheSymbolTableLineNumMap(), bfdImage);
+      }
+    }
+  }
+  fit = TheSymbolTableLineNumMap().find(funcname);
+  if(fit == TheSymbolTableLineNumMap().end()) {
+    TAU_VERBOSE("TAU_BFD: Didn't find %s in the full_symtab during first attempt!\n", funcname);
+    return 0;
+  } else {
+    lno = fit->second;
+    result_line = lno;
+    TAU_VERBOSE("TAU_BFD: Adding: cached_symtab[%s] = %d\n", funcname, lno);
+    TheCachedSymbolTableLineNumMap()[funcname] = lno;
+  }
+
+#else // we don't have TAU_DWARF; do it the slow way with libbfd
+  if(unit != NULL) {
+    for(vector<TauBfdModule*>::iterator it = unit->modules.begin(); it != unit->modules.end(); ++it) {
+      TauBfdModule *module = *it;
+      bfd * bfdImage;
+      if(module != NULL) {
+          bfdImage = module->bfdImage;
+          if(bfdImage == NULL) {
+            TAU_VERBOSE("TAU_BFD: Forcing load of symbol table for %s\n", module->name.c_str());
+            module->loadSymbolTable(module->name.c_str());
+            bfdImage = module->bfdImage;
+            if(bfdImage == NULL) {
+              TAU_VERBOSE("TAU_BFD: Skipping %s because its symbol table couldn't be loaded.\n", module->name.c_str());
+              continue;
+            }
+          }
+      } else {
+          continue;
+      }
+      /*
+      char const * module_name = unit->addressMaps[bfd_handle]->name;
+      printf("TAU_BFD ---> NAME of Module =%s \n", module_name);
+      module = Tau_bfd_internal_getModuleFromIdx(unit, bfd_handle);
+      bfdImage = module->bfdImage;
+      */
+
+      /* we have a valid bfdImage pointer. Examine the symbol table. */
+      size_t sz = bfd_get_symtab_upper_bound(bfdImage);
+      asymbol **syms;
+      bool dynamic = false;
+      int nr_all_syms, i;
+      if (!sz) {
+          TAU_VERBOSE("loadSymbolTable: Retrying with dynamic\n");
+          sz = bfd_get_dynamic_symtab_upper_bound(bfdImage);
+          //dynamic = true;
+          if (!sz) {
+          TAU_VERBOSE("loadSymbolTable: Cannot get symbol table size \n" );
+          continue;
+          }
+      }
+
+      // allocate the symbol table.
+      syms = (asymbol **)malloc(sz);
+      //long addr;
+      const char *filename = NULL;
+      const char *func;
+      unsigned int lineno = 0;
+
+      if (dynamic) {
+          nr_all_syms = bfd_canonicalize_dynamic_symtab(bfdImage, syms);
+      } else {
+          nr_all_syms = bfd_canonicalize_symtab(bfdImage, syms);
+      }
+
+      if (nr_all_syms < 1) {
+          TAU_VERBOSE("TAU_BFD: Skipping %s because it has no symbols\n", module->name.c_str(), nr_all_syms);
+          continue;
+      }
+
+      // iterate through all the symbols and see if we get a match. If we do,
+      // return the line number associated with it. Previous invocations should
+      // be cached.
+      for (i = 0; i < nr_all_syms; i++) {
+          //addr = syms[i]->section->vma + syms[i]->value;
+          bfd_find_nearest_line(bfdImage, TAU_BFD_GET_SECTION(syms[i]), syms,
+          syms[i]->value, &filename, &func, &lineno);
+          func = syms[i]->name;
+          if (lineno > 0) { // We only store non-zero entries now
+            TheSymbolTableLineNumMap()[func] = lineno; // Add this entry to the full symbol table.
+          }
+      }
+      fit = TheSymbolTableLineNumMap().find(funcname);
+      if (fit == TheSymbolTableLineNumMap().end()) { // We didn't find it - return 0;
+          TAU_VERBOSE("TAU_BFD: Didn't find line number for %s\n", funcname);
+          continue;
+      } else { // found it!
+          lineno = fit->second;
+          TAU_VERBOSE("TAU_BFD: Found it - first time! %s line no = %d\n", funcname, lineno);
+          TheCachedSymbolTableLineNumMap()[funcname] = lineno;
+          result_line = lineno;
+      }
+    }
+  }
+#endif // TAU_DWARF else clause
+
+  return result_line;
+
+}
+
+int Tau_get_lineno_for_function(tau_bfd_handle_t bfd_handle, char const * funcname) {
+    int line_number = Tau_internal_get_lineno_for_function(bfd_handle, funcname);
+    // To fix a mismatch between the regular symbol table and the debug symbol table, if we didn't find
+    // the name from the regular symbol table, and it ends with an underscore, also try the lookup
+    // without the underscore. Intel compilers on Cray seem to use e.g. "foo_" in the regular table but
+    // "foo" in the debug table.
+    if(line_number == 0) {
+        std::string underscore_name_str = std::string(funcname);
+        if((*underscore_name_str.rbegin()) == '_') {
+            underscore_name_str.erase(underscore_name_str.end() - 1);
+            line_number = Tau_internal_get_lineno_for_function(bfd_handle, underscore_name_str.c_str());
+        }
+    }
+    return line_number;
+}
+
+/* If we have the demangler support from demangle.h, use it */
+#if defined(HAVE_GNU_DEMANGLE)
+char * Tau_demangle_name(const char * name) {
+    char * dem_name = cplus_demangle(name, DMGL_PARAMS | DMGL_ANSI | DMGL_VERBOSE | DMGL_TYPES);
+    if (dem_name == NULL) {
+        dem_name = strdup(name);
+    }
+    TAU_VERBOSE("Demangled: '%s'\n", dem_name);
+    return dem_name;
+}
+#endif // #if defined(HAVE_GNU_DEMANGLE)
 #endif /* TAU_BFD */
+
+
