@@ -41,6 +41,7 @@
 #include <limits.h>
 #include <time.h>
 #include <Profile/TauEnv.h>
+#include <Profile/TauHandler.h>
 #include <TAU.h>
 #include <tauroot.h>
 #include <tauarch.h>
@@ -48,11 +49,8 @@
 #include <string>
 #include <set>
 
-#if TAU_OPENMP // for querying OpenMP settings
-#include "omp.h"
-#endif
-
 #include <iostream>
+#include <sstream>
 using namespace std;
 
 #ifndef TAU_BGP
@@ -77,10 +75,12 @@ using namespace std;
 # define TAU_CALLPATH_DEFAULT 0
 #endif
 
+#define TAU_ENABLE_THREAD_CONTEXT_DEFAULT 0
+
 #define TAU_CALLSITE_DEFAULT 0
 #define TAU_CALLSITE_DEPTH_DEFAULT 1 /* default to be local */
 
-#ifdef __PGI 
+#ifdef __PGI
 #define TAU_CALLSITE_OFFSET_DEFAULT 6 /* PGI needs 6 */
 #else /* __PGI */
 #define TAU_CALLSITE_OFFSET_DEFAULT 2 /* otherwise 2 */
@@ -99,12 +99,6 @@ using namespace std;
 #define TAU_OMPT_SUPPORT_LEVEL_LOWOVERHEAD "lowoverhead"
 #define TAU_OMPT_SUPPORT_LEVEL_FULL "full"
 
-#define TAU_SOS_DEFAULT 0
-#define TAU_SOS_TRACE_EVENTS_DEFAULT 0
-#define TAU_SOS_PERIODIC_DEFAULT 0
-#define TAU_SOS_PERIOD_DEFAULT 2000000 // microseconds
-#define TAU_SOS_HIGH_RESOLUTION_DEFAULT 0 // group, timer
-
 /* if we are doing EBS sampling, set the default sampling period */
 #define TAU_EBS_DEFAULT 0
 #define TAU_EBS_DEFAULT_TAU 0
@@ -114,7 +108,7 @@ using namespace std;
 #else
 #if (defined (TAU_CRAYCNL) || defined(TAU_BGQ))
 #define TAU_EBS_PERIOD_DEFAULT 50000 // Sameer made this bigger,
-#else 
+#else
 #define TAU_EBS_PERIOD_DEFAULT 10000 // Kevin made this bigger,
 
 #ifdef TAU_PYTHON
@@ -131,9 +125,13 @@ using namespace std;
 
 #define TAU_EBS_SOURCE_DEFAULT "itimer"
 #define TAU_EBS_UNWIND_DEFAULT 0
+#ifdef TAU_USE_BACKTRACE
+#define TAU_EBS_UNWIND_DEPTH_DEFAULT 0
+#else
 #define TAU_EBS_UNWIND_DEPTH_DEFAULT 10
+#endif
 
-#define TAU_EBS_RESOLUTION_STR_DEFAULT "line"
+#define TAU_EBS_RESOLUTION_STR_LINE "line"
 #define TAU_EBS_RESOLUTION_STR_FILE "file"
 #define TAU_EBS_RESOLUTION_STR_FUNCTION "function"
 
@@ -200,6 +198,7 @@ using namespace std;
 #define TAU_TRACK_CUDA_SASS_DEFAULT 0
 #define TAU_SASS_TYPE_DEFAULT "kernel"
 #define TAU_OUTPUT_CUDA_CSV_DEFAULT 0
+#define TAU_TRACK_CUDA_ENV_DEFAULT 0
 
 #define TAU_MIC_OFFLOAD_DEFAULT 0
 
@@ -230,27 +229,42 @@ using namespace std;
 #define TAU_MEM_CALLPATH_DEFAULT 0
 #define TAU_REGION_ADDRESSES_DEFAULT 0
 
+/* Thread recycling */
+#define TAU_RECYCLE_THREADS_DEFAULT 0
+
 // forward declartion of cuserid. need for c++ compilers on Cray.
 extern "C" char *cuserid(char *);
 
+#ifdef TAU_MPI
+extern "C" void Tau_set_usesMPI(int value);
+#endif /* TAU_MPI */
+
+#ifdef TAU_ENABLE_ROCTRACER
+extern void Tau_roctracer_start_tracing(void);
+#endif /* TAU_ENABLE_ROCTRACER */
+
 /************************** tau.conf stuff, adapted from Scalasca ***********/
 
 extern "C" {
 
 static int env_synchronize_clocks = 0;
 static int env_verbose = 0;
+static int env_verbose_file = 0;
+static int env_verbose_rank = -1;
 static int env_throttle = 0;
 static double env_evt_threshold = 0.0;
 static int env_interval = 0;
 static int env_disable_instrumentation = 0;
-static double env_max_records = 0;
+static double env_max_records = 64*1024;
 static int env_callpath = 0;
+static int env_thread_context = 0;
 static int env_callsite = 0;
 static int env_callsite_depth = 0;
 static int env_callsite_offset = TAU_CALLSITE_OFFSET_DEFAULT;
 static int env_compensate = 0;
 static int env_profiling = 0;
-static int env_tracing = 0; 
+static int env_tracing = 0;
+static int env_thread_per_gpu_stream = 0;
 static int env_trace_format = TAU_TRACE_FORMAT_DEFAULT;
 static int env_callpath_depth = 0;
 static int env_depth_limit = 0;
@@ -276,7 +290,7 @@ static int env_ibm_bg_hwp_counters = 0;
 static int env_ebs_keep_unresolved_addr = 0;
 static int env_ebs_period = 0;
 static int env_ebs_inclusive = 0;
-static int env_ompt_resolve_address_eagerly = 0;
+static int env_ompt_resolve_address_eagerly = 1;
 static int env_ompt_support_level = 0;
 static int env_openmp_runtime_enabled = 1;
 static int env_openmp_runtime_states_enabled = 0;
@@ -291,6 +305,7 @@ static int env_ebs_resolution = TAU_EBS_RESOLUTION_LINE;
 
 static int env_stat_precompute = 0;
 static int env_child_forkdirs = 0;
+static int env_l0_api_tracing = 0;
 
 static int env_profile_format = TAU_FORMAT_PROFILE;
 static const char *env_profile_prefix = NULL;
@@ -300,22 +315,31 @@ static const char *env_profiledir = NULL;
 static const char *env_tracedir = NULL;
 static const char *env_metrics = NULL;
 static const char *env_cvar_metrics = NULL;
+static const char *env_mpi_t_comm_metric_values = NULL;
 static const char *env_cvar_values = NULL;
 static const char *env_plugins_path = NULL;
 static const char *env_plugins = NULL;
 static int env_plugins_enabled = 0;
+static int env_track_mpi_t_comm_metric_values = 0;
 static const char *env_select_file = NULL;
 static const char *env_cupti_api = TAU_CUPTI_API_DEFAULT;
 static const char * env_cuda_device_name = TAU_CUDA_DEVICE_NAME_DEFAULT;
 static int env_sigusr1_action = TAU_ACTION_DUMP_PROFILES;
 static const char *env_track_cuda_instructions = TAU_TRACK_CUDA_INSTRUCTIONS_DEFAULT;
 static int env_track_cuda_cdp = TAU_TRACK_CUDA_CDP_DEFAULT;
-static int env_track_unified_memory = TAU_TRACK_UNIFIED_MEMORY_DEFAULT; 
+static int env_track_unified_memory = TAU_TRACK_UNIFIED_MEMORY_DEFAULT;
 static int env_track_cuda_sass = TAU_TRACK_CUDA_SASS_DEFAULT;
 static const char* env_sass_type = TAU_SASS_TYPE_DEFAULT;
 static int env_output_cuda_csv = TAU_OUTPUT_CUDA_CSV_DEFAULT;
 static const char *env_binaryexe = NULL;
+static int env_track_cuda_env = TAU_TRACK_CUDA_ENV_DEFAULT;
+static int env_current_timer_exit_params = 0;
 
+static int env_node_set = -1;
+
+static int env_cudatotalthreads = 0;
+static int env_taucuptiavail = 0;
+static int env_nodenegoneseen = 0;
 static int env_mic_offload = 0;
 static int env_bfd_lookup = 0;
 
@@ -353,6 +377,7 @@ static int env_mem_all = 0;
 static const char *env_mem_classes = NULL;
 static std::set<std::string> * env_mem_classes_set = NULL;
 static int env_region_addresses = TAU_REGION_ADDRESSES_DEFAULT;
+static int env_recycle_threads = TAU_RECYCLE_THREADS_DEFAULT;
 
 static const char *env_tau_exec_args = NULL;
 static const char *env_tau_exec_path = NULL;
@@ -476,7 +501,7 @@ static int TauConf_parse(FILE *cfgFile, const char *fname) {
   return 0;
 }
 
-extern int Tau_util_readFullLine(char *line, FILE *fp); 
+extern int Tau_util_readFullLine(char *line, FILE *fp);
 /*********************************************************************
  * Get executable directory name: /usr/local/foo will return /usr/local
  ********************************************************************/
@@ -501,7 +526,7 @@ static char * Tau_get_cwd_of_exe()
       }
       free((void*)line);
     }
-    fclose(f); // close the file if it is not null 
+    fclose(f); // close the file if it is not null
   }
   return retval;
 }
@@ -511,12 +536,25 @@ static char * Tau_get_cwd_of_exe()
  ********************************************************************/
 void Tau_util_replaceStringInPlace(std::string& subject, const std::string& search,
                           const std::string& replace) {
-    size_t pos = 0; 
+    size_t pos = 0;
     while ((pos = subject.find(search, pos)) != std::string::npos) {
          subject.replace(pos, search.length(), replace);
          pos += replace.length();
-    }           
-}               
+    }
+}
+
+/* A C implementation, for crappy compilers. */
+void Tau_util_replaceStringInPlaceC(char * subject, const char search,
+                          const char replace) {
+    size_t pos = 0;
+    size_t len = strlen(subject);
+    while (pos < len) {
+         if (subject[pos] == search) {
+            subject[pos] = replace;
+         }
+         pos++;
+    }
+}
 
 /*********************************************************************
  * Parse a boolean value
@@ -550,8 +588,8 @@ static int parse_int(const char *str, int default_value = 0) {
     return default_value;
   }
   int tmp = atoi(str);
-  if (tmp < 0) { 
-    return default_value; 
+  if (tmp < 0) {
+    return default_value;
   }
   return tmp;
 }
@@ -564,9 +602,30 @@ static int TauConf_read()
   const char *tmp;
   char conf_file_name[1024];
 
+/* Eagerly get some settings needed for configuring verbose output */
+
   tmp = getenv("TAU_VERBOSE");
   if (parse_bool(tmp)) {
     env_verbose = 1;
+    tmp = getenv("TAU_VERBOSE_FILE");
+    if (parse_bool(tmp,env_verbose_file)) {
+        env_verbose_file = 1;
+    }
+    tmp = getenv("TAU_VERBOSE_RANK");
+    if (parse_int(tmp,env_verbose_rank)) {
+        env_verbose_rank = Tau_get_node();
+    }
+    if ((env_profiledir = getenv("PROFILEDIR")) == NULL) {
+      env_profiledir = ".";   /* current directory */
+#ifdef TAU_GPI
+      // if exe is /usr/local/foo, this will return /usr/local where profiles
+      // may be stored if PROFILEDIR is not specified
+      char const * cwd = Tau_get_cwd_of_exe();
+      if (cwd) {
+        env_profiledir = strdup(cwd);
+      }
+#endif /* TAU_GPI */
+    }
   } else {
     env_verbose = 0;
   }
@@ -607,9 +666,9 @@ static int TauConf_read()
 }
 
 /*********************************************************************
- * Local getconf routine
+ * TAU's getconf routine
  ********************************************************************/
-static const char *getconf(const char *key) {
+const char *getconf(const char *key) {
   const char *val = TauConf_getval(key);
   //TAU_VERBOSE("%s=%s\n", key, val);
   if (val) {
@@ -689,7 +748,7 @@ char * Tau_check_dirname(const char * dir)
       mkdir(logfiledir, S_IRWXU | S_IRGRP | S_IXGRP | S_IXGRP | S_IRWXO);
       TAU_VERBOSE("mkdir %s\n", logfiledir);
       umask(oldmode);
-#endif 
+#endif
     }
     return strdup(logfiledir);
   }
@@ -701,9 +760,29 @@ char * Tau_check_dirname(const char * dir)
 
 /****************************************************************************/
 
+class Tau_logfile_t {
+public:
+    FILE * pfile;
+    Tau_logfile_t() : pfile(stderr) {
+        if (env_verbose_file == 1 &&
+            env_verbose_rank == Tau_get_node()) {
+            std::stringstream ss;
+            ss << env_profiledir << "/tau." << Tau_get_node() << ".log";
+            std::string tmp(ss.str());
+            pfile = fopen(tmp.c_str(),"w");
+        }
+    }
+    ~Tau_logfile_t() {
+        if (env_verbose_file == 1 &&
+            env_verbose_rank == Tau_get_node()) {
+            fclose(pfile);
+        }
+    }
+};
+
 extern "C" { /* C linkage */
 
-#ifdef TAU_GPI 
+#ifdef TAU_GPI
 #include <GPI.h>
 #include <GpiLogger.h>
 #endif /* TAU_GPI */
@@ -713,6 +792,7 @@ extern "C" { /* C linkage */
 void TAU_VERBOSE(const char *format, ...)
 {
   if (env_verbose == 1) {
+    static Tau_logfile_t foo;
     TauInternalFunctionGuard protects_this_function;
     va_list args;
 
@@ -733,10 +813,10 @@ void TAU_VERBOSE(const char *format, ...)
 #ifdef TAU_GPI
     gpi_vprintf(format, args);
 #else
-    vfprintf(stderr, format, args);
+    vfprintf(foo.pfile, format, args);
 #endif
     va_end(args);
-    fflush(stderr);
+    fflush(foo.pfile);
 
 #endif
   } // END inside TAU
@@ -750,6 +830,10 @@ extern "C" const char *TauEnv_get_cvar_metrics() {
   return env_cvar_metrics;
 }
 
+extern "C" const char *TauEnv_get_mpi_t_comm_metric_values() {
+  return env_mpi_t_comm_metric_values;
+}
+
 extern "C" const char *TauEnv_get_plugins_path() {
   return env_plugins_path;
 }
@@ -762,6 +846,14 @@ extern "C" int TauEnv_get_plugins_enabled() {
   return env_plugins_enabled;
 }
 
+extern "C" int TauEnv_get_track_mpi_t_comm_metric_values() {
+  return env_track_mpi_t_comm_metric_values;
+}
+
+extern "C" int TauEnv_get_set_node() {
+  return env_node_set;
+}
+
 extern "C" const char *TauEnv_get_cvar_values() {
   return env_cvar_values;
 }
@@ -794,6 +886,10 @@ int TauEnv_get_throttle() {
   return env_throttle;
 }
 
+void TauEnv_set_throttle(int throttle) {
+  env_throttle = throttle;
+}
+
 int TauEnv_get_disable_instrumentation() {
   return env_disable_instrumentation;
 }
@@ -814,6 +910,10 @@ int TauEnv_get_callpath() {
   return env_callpath;
 }
 
+int TauEnv_get_threadContext() {
+  return env_thread_context;
+}
+
 int TauEnv_get_callsite() {
   return env_callsite;
 }
@@ -830,10 +930,18 @@ int TauEnv_get_compensate() {
   return env_compensate;
 }
 
+int TauEnv_get_level_zero_enable_api_tracing() {
+  return env_l0_api_tracing;
+}
+
 int TauEnv_get_comm_matrix() {
   return env_comm_matrix;
 }
 
+int TauEnv_get_current_timer_exit_params() {
+  return env_current_timer_exit_params;
+}
+
 int TauEnv_get_ompt_resolve_address_eagerly() {
   return env_ompt_resolve_address_eagerly;
 }
@@ -852,17 +960,17 @@ int TauEnv_get_mpi_t_enable_user_tuning_policy() {
 
 int TauEnv_set_track_mpi_t_pvars(int value) {
   env_track_mpi_t_pvars = value;
-  return env_track_mpi_t_pvars; 
+  return env_track_mpi_t_pvars;
 }
 
 int TauEnv_set_ompt_resolve_address_eagerly(int value) {
   env_ompt_resolve_address_eagerly = value;
-  return env_ompt_resolve_address_eagerly; 
+  return env_ompt_resolve_address_eagerly;
 }
 
 int TauEnv_set_ompt_support_level(int value) {
   env_ompt_support_level = value;
-  return env_ompt_support_level; 
+  return env_ompt_support_level;
 }
 
 int TauEnv_get_track_signals() {
@@ -917,6 +1025,10 @@ int TauEnv_get_region_addresses() {
   return env_region_addresses;
 }
 
+int TauEnv_get_recycle_threads() {
+  return env_recycle_threads;
+}
+
 int TauEnv_get_track_io_params() {
   return env_track_io_params;
 }
@@ -937,6 +1049,10 @@ int TauEnv_get_tracing() {
   return env_tracing;
 }
 
+int TauEnv_get_thread_per_gpu_stream() {
+  return env_thread_per_gpu_stream;
+}
+
 int TauEnv_get_trace_format() {
   return env_trace_format;
 }
@@ -1022,7 +1138,7 @@ int TauEnv_get_openmp_runtime_events_enabled() {
     env_openmp_runtime_events_enabled = 1;
   } else {
     env_openmp_runtime_events_enabled = 0;
-  } 
+  }
 
   return env_openmp_runtime_events_enabled;
 }
@@ -1045,7 +1161,7 @@ const char *TauEnv_get_ebs_source() {
 
 void TauEnv_override_ebs_source(const char *newName) {
   env_ebs_source = newName;
-  TAU_METADATA("TAU_EBS_PERIOD (Override)", newName);
+  TAU_METADATA("TAU_EBS_SOURCE (Override)", newName);
 }
 
 int TauEnv_get_stat_precompute() {
@@ -1092,6 +1208,31 @@ const char* TauEnv_get_cuda_binary_exe(){
   return env_binaryexe;
 }
 
+int TauEnv_get_cuda_track_env(){
+  return env_track_cuda_env;
+}
+
+void TauEnv_set_cudaTotalThreads(int nthreads) {
+    env_cudatotalthreads = nthreads;
+}
+int TauEnv_get_cudaTotalThreads() {
+    return env_cudatotalthreads;
+}
+
+void TauEnv_set_tauCuptiAvail(int off) {
+    env_taucuptiavail = off;
+}
+int TauEnv_get_tauCuptiAvail() {
+    return env_taucuptiavail;
+}
+
+void TauEnv_set_nodeNegOneSeen(int nthreads) {
+    env_nodenegoneseen = nthreads;
+}
+int TauEnv_get_nodeNegOneSeen() {
+    return env_nodenegoneseen;
+}
+
 int TauEnv_get_mic_offload(){
   return env_mic_offload;
 }
@@ -1113,8 +1254,8 @@ int TauEnv_get_memdbg_protect_above() {
 }
 void TauEnv_set_memdbg_protect_above(int value) {
   env_memdbg_protect_above = value;
-  env_memdbg = (env_memdbg_protect_above || 
-                env_memdbg_protect_below || 
+  env_memdbg = (env_memdbg_protect_above ||
+                env_memdbg_protect_below ||
                 env_memdbg_protect_free);
 }
 
@@ -1123,8 +1264,8 @@ int TauEnv_get_memdbg_protect_below() {
 }
 void TauEnv_set_memdbg_protect_below(int value) {
   env_memdbg_protect_below = value;
-  env_memdbg = (env_memdbg_protect_above || 
-                env_memdbg_protect_below || 
+  env_memdbg = (env_memdbg_protect_above ||
+                env_memdbg_protect_below ||
                 env_memdbg_protect_free);
 }
 
@@ -1133,8 +1274,8 @@ int TauEnv_get_memdbg_protect_free() {
 }
 void TauEnv_set_memdbg_protect_free(int value) {
   env_memdbg_protect_free = value;
-  env_memdbg = (env_memdbg_protect_above || 
-                env_memdbg_protect_below || 
+  env_memdbg = (env_memdbg_protect_above ||
+                env_memdbg_protect_below ||
                 env_memdbg_protect_free);
 }
 
@@ -1230,7 +1371,7 @@ const char * TauEnv_get_tau_exec_path() {
 /*********************************************************************
  * Initialize the TauEnv module, get configuration values
  ********************************************************************/
-void TauEnv_initialize() 
+void TauEnv_initialize()
 {
   char tmpstr[512];
 
@@ -1242,7 +1383,7 @@ void TauEnv_initialize()
   if (!initialized) {
     const char *tmp;
     char *saveptr;
-    const char *key; 
+    const char *key;
     const char *val;
 
     /* Read the configuration file */
@@ -1255,6 +1396,21 @@ void TauEnv_initialize()
       env_verbose = 1;
     }
 
+    tmp = getconf("TAU_VERBOSE_FILE");
+    if (parse_bool(tmp,env_verbose_file)) {
+      TAU_VERBOSE("TAU: VERBOSE to file enabled\n");
+      TAU_METADATA("TAU_VERBOSE_FILE", "on");
+      env_verbose_file = 1;
+    }
+
+    tmp = getconf("TAU_VERBOSE_RANK");
+    if (parse_int(tmp,env_verbose_rank)) {
+      TAU_VERBOSE("TAU: VERBOSE RANK enabled\n");
+      sprintf(tmpstr, "%d", env_verbose_rank);
+      TAU_METADATA("TAU_VERBOSE_RANK", tmpstr);
+      env_verbose_rank = Tau_get_node();
+    }
+
     sprintf(tmpstr, "%d", TAU_MAX_THREADS);
     TAU_VERBOSE("TAU: Supporting %d threads\n", TAU_MAX_THREADS);
     TAU_METADATA("TAU_MAX_THREADS", tmpstr);
@@ -1274,7 +1430,7 @@ void TauEnv_initialize()
       sscanf(interval,"%d",&interval_value);
       env_interval = interval_value;
       sprintf(tmpstr, "%d", env_interval);
-      TAU_SET_INTERRUPT_INTERVAL(interval_value); 
+      TAU_SET_INTERRUPT_INTERVAL(interval_value);
       TAU_METADATA("TAU_INTERRUPT_INTERVAL", tmpstr);
     }
 
@@ -1286,9 +1442,9 @@ void TauEnv_initialize()
 #else
       TAU_VERBOSE("TAU: Power tracking Enabled\n");
       TAU_METADATA("TAU_TRACK_POWER", "on");
-      TAU_TRACK_POWER();
+      TauEnableTrackingPower();
 #endif
-    } 
+    }
 
     tmp = getconf("TAU_TRACK_LOAD");
     if (parse_bool(tmp, env_track_load)) {
@@ -1298,12 +1454,20 @@ void TauEnv_initialize()
 #else
       TAU_VERBOSE("TAU: system load tracking Enabled\n");
       TAU_METADATA("TAU_TRACK_LOAD", "on");
-      // Don't do this now! It will cause infinite recurison...
-      //TAU_TRACK_LOAD();
+      TauEnableTrackingLoad();
 #endif
-    } 
+    }
 
 #ifdef TAU_MPI_T
+    if ((env_mpi_t_comm_metric_values = getconf("TAU_MPI_T_COMM_METRIC_VALUES")) == NULL) {
+      env_mpi_t_comm_metric_values = "";   /* Not set */
+      env_track_mpi_t_comm_metric_values=0;
+    } else {
+      env_track_mpi_t_comm_metric_values=1;
+      TAU_VERBOSE("TAU: TAU_MPI_T_COMM_METRIC_VALUES is \"%s\"\n", env_mpi_t_comm_metric_values);
+      TAU_METADATA("TAU_MPI_T_COMM_METRIC_VALUES", env_mpi_t_comm_metric_values);
+    }
+
     tmp = getconf("TAU_TRACK_MPI_T_PVARS");
     if (parse_bool(tmp, env_track_mpi_t_pvars)) {
 #ifdef TAU_DISABLE_MEM_MANAGER
@@ -1331,7 +1495,7 @@ void TauEnv_initialize()
 
 #endif /* TAU_MPI_T */
 
-#ifdef TAU_OMPT_TR6
+#if defined (TAU_USE_OMPT_TR6) || defined (TAU_USE_OMPT_TR7) || defined (TAU_USE_OMPT_5_0)
     tmp = getconf("TAU_OMPT_RESOLVE_ADDRESS_EAGERLY");
     if (parse_bool(tmp, env_ompt_resolve_address_eagerly)) {
 #ifdef TAU_DISABLE_MEM_MANAGER
@@ -1342,11 +1506,12 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: OMPT resolving addresses eagerly Enabled\n");
       TAU_METADATA("TAU_OMPT_RESOLVE_ADDRESS_EAGERLY", "on");
       TAU_VERBOSE("TAU: Resolving OMPT addresses eagerly\n");
-#endif
+#endif /*  TAU_DISABLE_MEM_MANAGER */
     } else {
+      env_ompt_resolve_address_eagerly = 0;
       TAU_METADATA("TAU_OMPT_RESOLVE_ADDRESS_EAGERLY", "off");
-    } 
-    
+    }
+
     env_ompt_support_level = 0; // Basic OMPT support is the default
     const char *omptSupportLevel = getconf("TAU_OMPT_SUPPORT_LEVEL");
     if (omptSupportLevel != NULL && 0 == strcasecmp(omptSupportLevel, TAU_OMPT_SUPPORT_LEVEL_BASIC)) {
@@ -1362,7 +1527,7 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: OMPT support will be full - all events will be supported\n");
       TAU_METADATA("TAU_OMPT_SUPPORT_LEVEL", "full");
     }
-#endif/* TAU_OMPT_TR6 */
+#endif /* defined (TAU_USE_OMPT_TR6) || defined (TAU_USE_OMPT_TR7) || defined (TAU_USE_OMPT_5_0) */
 
     tmp = getconf("TAU_TRACK_HEAP");
     if (parse_bool(tmp, env_track_memory_heap)) {
@@ -1398,13 +1563,8 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: TAU_TRACK_MEMORY_FOOTPRINT VmRSS and VmHWM tracking Enabled\n");
       TAU_METADATA("TAU_TRACK_MEMORY_FOOTPRINT", "on");
 
-      /*Adding a check to prevent signal handler from being invoked over and over.
- 	Set env_track_memory_footprint BEFORE invoking the signal handler. If not, a segfault is coming your way.*/
-      if(env_track_memory_footprint == 0) {
-
-	env_track_memory_footprint = 1;
-        TAU_TRACK_MEMORY_FOOTPRINT();
-      }
+	  env_track_memory_footprint = 1;
+      TAU_TRACK_MEMORY_FOOTPRINT();
 #endif
     } else {
       TAU_METADATA("TAU_TRACK_MEMORY_FOOTPRINT", "off");
@@ -1459,7 +1619,17 @@ void TauEnv_initialize()
     } else {
       TAU_METADATA("TAU_REGION_ADDRESSES", "off");
       env_region_addresses = 0;
-    }                                     
+    }
+
+    tmp = getconf("TAU_RECYCLE_THREADS");
+    if (parse_bool(tmp, TAU_RECYCLE_THREADS_DEFAULT)) {
+      TAU_VERBOSE("TAU: Region addresses Enabled\n");
+      TAU_METADATA("TAU_RECYCLE_THREADS", "on");
+      env_recycle_threads = 1;
+    } else {
+      TAU_METADATA("TAU_RECYCLE_THREADS", "off");
+      env_recycle_threads = 0;
+    }
 
     // Setting TAU_MEMDBG_PROTECT_{ABOVE,BELOW,FREE} enables memory debugging.
 
@@ -1654,7 +1824,7 @@ void TauEnv_initialize()
 
 #ifdef TAU_SCOREP
     TAU_VERBOSE("[%d] TAU: SCOREP active! (TAU measurement disabled)\n", RtsLayer::getPid());
-    //return; 
+    //return;
     //if we return here, the other TAU variables such as TAU_SELECT_FILE are not read!
 #endif
 
@@ -1695,6 +1865,7 @@ void TauEnv_initialize()
     tmp = getconf("TAU_TRACE");
     if (parse_bool(tmp, TAU_TRACING_DEFAULT)) {
       env_tracing = 1;
+      env_thread_per_gpu_stream = 1;
       env_track_message = 1;
       profiling_default = 0;
       TAU_VERBOSE("TAU: Tracing Enabled\n");
@@ -1704,6 +1875,7 @@ void TauEnv_initialize()
       }
     } else {
       env_tracing = 0;
+      env_thread_per_gpu_stream = 0;
       env_track_message = TAU_TRACK_MESSAGE_DEFAULT;
       TAU_VERBOSE("TAU: Tracing Disabled\n");
       TAU_METADATA("TAU_TRACE", "off");
@@ -1743,7 +1915,23 @@ void TauEnv_initialize()
       TAU_METADATA("TAU_PROFILE", "off");
     }
 
-    if (env_profiling) {
+    tmp = getconf("TAU_CURRENT_TIMER_EXIT_PARAMS");
+    if (parse_bool(tmp, profiling_default)) {
+      env_current_timer_exit_params = 1;
+      TAU_VERBOSE("TAU: Profiling Enabled\n");
+      TAU_METADATA("TAU_CURRENT_TIMER_EXIT_PARAMS", "on");
+    } else {
+      env_current_timer_exit_params = 0;
+      TAU_VERBOSE("TAU: Current Timer Exit Disabled\n");
+      TAU_METADATA("TAU_CURRENT_TIMER_EXIT_PARAMS", "off");
+    }
+
+    /* Switched this from env_profiling to !env_tracing.
+     * If we are using alternative outputs (ADIOS2, SQLITE, SOS)
+     * we want to disable profile wrting at the end of execution
+     * but we don't want to disable callpaths.
+     */
+    if (!env_tracing) {
       /* callpath */
       tmp = getconf("TAU_CALLPATH");
       if (parse_bool(tmp, TAU_CALLPATH_DEFAULT)) {
@@ -1756,6 +1944,18 @@ void TauEnv_initialize()
         TAU_METADATA("TAU_CALLPATH", "off");
       }
 
+      /* thread context */
+      tmp = getconf("TAU_ENABLE_THREAD_CONTEXT");
+      if (parse_bool(tmp, TAU_ENABLE_THREAD_CONTEXT_DEFAULT)) {
+        env_thread_context = 1;
+        TAU_VERBOSE("TAU: Thread Context Enabled\n");
+        TAU_METADATA("TAU_ENABLE_THREAD_CONTEXT", "on");
+      } else {
+        env_thread_context = 0;
+        TAU_VERBOSE("TAU: Thread Context Disabled\n");
+        TAU_METADATA("TAU_ENABLE_THREAD_CONTEXT", "off");
+      }
+
       /* compensate */
       tmp = getconf("TAU_COMPENSATE");
       if (parse_bool(tmp, TAU_COMPENSATE_DEFAULT)) {
@@ -1767,6 +1967,13 @@ void TauEnv_initialize()
         TAU_VERBOSE("TAU: Overhead Compensation Disabled\n");
         TAU_METADATA("TAU_COMPENSATE", "off");
       }
+
+      tmp = getconf("TAU_THREAD_PER_GPU_STREAM");
+      if (parse_bool(tmp, 0)) {
+        env_thread_per_gpu_stream = 1;
+        TAU_VERBOSE("TAU: Enabling new thread for every GPU stream\n");
+        TAU_METADATA("TAU_THREAD_PER_GPU_STREAM", "on");
+      }
     }
 
     tmp = getconf("TAU_CALLSITE");
@@ -1774,7 +1981,7 @@ void TauEnv_initialize()
       env_callsite = 1;
       TAU_VERBOSE("TAU: Callsite Discovery via Unwinding Enabled\n");
       TAU_METADATA("TAU_CALLSITE", "on");
-    } 
+    }
 
     const char *callsiteDepth = getconf("TAU_CALLSITE_DEPTH");
     env_callsite_depth = TAU_CALLSITE_DEPTH_DEFAULT;
@@ -1830,6 +2037,31 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: Message Tracking Disabled\n");
       TAU_METADATA("TAU_TRACK_MESSAGE", "off");
     }
+
+
+    const char *max_records = getconf("TAU_MAX_RECORDS");
+    env_max_records = TAU_MAX_RECORDS;
+    if (max_records) {
+      env_max_records = strtod(max_records, 0);
+      TAU_VERBOSE("TAU: TAU_MAX_RECORDS = %g\n", env_max_records);
+    }
+
+
+
+#ifdef TAU_MPI
+    tmp = getconf("TAU_SET_NODE");
+    if (tmp) {
+      int node_id = 0;
+      sscanf(tmp,"%d",&node_id);
+      env_node_set=node_id;
+      TAU_VERBOSE("TAU: Setting node value forcibly to (TAU_SET_NODE): %d\n", node_id);
+      TAU_PROFILE_SET_NODE(node_id);
+      Tau_set_usesMPI(1);
+      TAU_METADATA("TAU_SET_NODE", tmp);
+    }
+#endif /* TAU_MPI */
+
+
 #endif /* TAU_MPI || TAU_SHMEM || TAU_DMAPP || TAU_UPC || TAU_GPI */
 
     /* clock synchronization */
@@ -1898,7 +2130,7 @@ void TauEnv_initialize()
     tmp = getconf("TAU_DISABLE_INSTRUMENTATION");
     if (parse_bool(tmp, TAU_DISABLE_INSTRUMENTATION_DEFAULT)) {
       env_disable_instrumentation = 1;
-      TAU_DISABLE_INSTRUMENTATION(); 
+      TAU_DISABLE_INSTRUMENTATION();
       TAU_VERBOSE("TAU: Instrumentation Disabled\n");
       TAU_METADATA("TAU_DISABLE_INSTRUMENTATION", "on");
     } else { /* default: instrumentation is enabled */
@@ -1911,6 +2143,11 @@ void TauEnv_initialize()
       env_throttle_percall = strtod(percall, 0);
     }
 
+    const char *l0_api_tracing = getconf("ZE_ENABLE_API_TRACING");
+    if (l0_api_tracing) {
+      env_l0_api_tracing =1 ; /* Intel OneAPI Level Zero API TRACING */
+    }
+
     const char *evt_threshold = getconf("TAU_EVENT_THRESHOLD");
     env_evt_threshold = TAU_EVENT_THRESHOLD_DEFAULT;
     if (evt_threshold) {
@@ -1926,12 +2163,6 @@ void TauEnv_initialize()
     if (numcalls) {
       env_throttle_numcalls = strtod(numcalls, 0);
     }
-    const char *max_records = getconf("TAU_MAX_RECORDS");
-    env_max_records = TAU_MAX_RECORDS;
-    if (max_records) {
-      env_max_records = strtod(max_records, 0);
-      TAU_VERBOSE("TAU: TAU_MAX_RECORDS = %g\n", env_max_records);
-    }
 
     if (env_throttle) {
       TAU_VERBOSE("TAU: Throttle PerCall = %g\n", env_throttle_percall);
@@ -1942,7 +2173,7 @@ void TauEnv_initialize()
       sprintf(tmpstr, "%g", env_throttle_numcalls);
       TAU_METADATA("TAU_THROTTLE_NUMCALLS", tmpstr);
     }
-		
+
     const char *sigusr1Action = getconf("TAU_SIGUSR1_ACTION");
     if (sigusr1Action != NULL && 0 == strcasecmp(sigusr1Action, "backtraces")) {
       env_sigusr1_action = TAU_ACTION_DUMP_BACKTRACES;
@@ -2020,45 +2251,45 @@ void TauEnv_initialize()
     }
 
     if ((env_plugins_path = getconf("TAU_PLUGINS_PATH")) == NULL) {
-      env_plugins_path = NULL;  
+      env_plugins_path = NULL;
       TAU_VERBOSE("TAU: TAU_PLUGINS_PATH is not set\n", env_plugins_path);
     } else {
       TAU_VERBOSE("TAU: TAU_PLUGINS_PATH is \"%s\"\n", env_plugins_path);
     }
 
     if ((env_plugins = getconf("TAU_PLUGINS")) == NULL) {
-      env_plugins = NULL;   
+      env_plugins = NULL;
       TAU_VERBOSE("TAU: TAU_PLUGINS is not set\n", env_plugins);
     } else {
       TAU_VERBOSE("TAU: TAU_PLUGINS is \"%s\"\n", env_plugins);
     }
-    if (env_plugins_path != NULL && env_plugins != NULL) {
-        env_plugins_enabled = 1;
-    }
 
     if((env_select_file = getconf("TAU_SELECT_FILE")) == NULL) {
-      env_select_file = NULL; 
+      env_select_file = NULL;
     } else {
       if ((env_plugins == NULL) && (env_plugins_path == NULL)) {
         TAU_VERBOSE("TAU: TAU_SELECT_FILE is set to %s when TAU plugins are not initialized\n", env_select_file);
-          //env_plugins_path=strdup(TAU_LIB_DIR); 
-          env_plugins_path=strdup("."); 
-          TAU_VERBOSE("TAU: TAU_PLUGINS_PATH is now %s\n", env_plugins_path);
-          //sprintf(env_plugins,"libTAU-filter-plugin.so(%s)", env_select_file); 
-          char *plugins = (char *) malloc(1024); 
+          env_plugins_path=strdup(TAU_LIB_DIR);
+          TAU_VERBOSE("TAU: TAU_PLUGINS_PATH is now %s, TAU_LIB_DIR=%s\n", env_plugins_path, TAU_LIB_DIR);
+          //sprintf(env_plugins,"libTAU-filter-plugin.so(%s)", env_select_file);
+          char *plugins = (char *) malloc(1024);
 	  char *filename = strdup(env_select_file);
-          sprintf(plugins, "libTAU-filter-plugin.so(%s)", filename); 
-          env_plugins = plugins; 
+          sprintf(plugins, "libTAU-filter-plugin.so(%s)", filename);
+          env_plugins = plugins;
           TAU_VERBOSE("TAU: TAU plugin is now %s\n", env_plugins);
 	  TAU_METADATA("TAU_SELECT_FILE", filename);
       } else {
-        TAU_VERBOSE("TAU: Ignoring TAU_SELECT_FILE because TAU_PLUGINS and/or TAU_PLUGINS_PATH is set.\nPlease use export TAU_PLUGINS_PATH=%s and export TAU_PLUGINS=\"libTAU-filter-plugin.so(%s)\"\n", 
-			//strdup(TAU_LIB_DIR), 
-			strdup("."), 
-			strdup(env_select_file)); 
+        TAU_VERBOSE("TAU: Ignoring TAU_SELECT_FILE because TAU_PLUGINS and/or TAU_PLUGINS_PATH is set.\nPlease use export TAU_PLUGINS_PATH=%s and export TAU_PLUGINS=\"libTAU-filter-plugin.so(%s)\"\n",
+			strdup(TAU_LIB_DIR),
+			strdup(env_select_file));
       }
     }
 
+    // Check for these paths after TAU_SELECT_FILE sets them
+    if (env_plugins_path != NULL && env_plugins != NULL) {
+        env_plugins_enabled = 1;
+    }
+
     tmp = getconf("TAU_OPENMP_RUNTIME");
     if (parse_bool(tmp, TAU_OPENMP_RUNTIME_DEFAULT)) {
       env_openmp_runtime_enabled = 1;
@@ -2107,64 +2338,6 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: OpenMP Runtime Support Context none\n");
       TAU_METADATA("TAU_OPENMP_RUNTIME_CONTEXT", "none");
     }
-    
-// MPC wht OpenMP isn't initialized before TAU is, so these function calls will hang.
-#if TAU_OPENMP && !defined(TAU_MPC) 
-    const char* schedule;
-    int modifier;
-#if defined(omp_sched_t)
-    omp_sched_t kind;
-    omp_get_schedule(&kind, &modifier);
-    if (kind == omp_sched_static) {
-        schedule = "STATIC";
-    } else if (kind == omp_sched_dynamic) {
-        schedule = "DYNAMIC";
-    } else if ( kind == omp_sched_guided) {
-        schedule = "GUIDED";
-    } else if ( kind == omp_sched_auto) {
-        schedule = "AUTO";
-    } else {
-        schedule = "UNKNOWN";
-    }
-    TAU_METADATA("OMP_SCHEDULE", schedule);
-    sprintf(tmpstr,"%d",modifier);
-    TAU_METADATA("OMP_CHUNK_SIZE", tmpstr);
-#else
-    schedule = "UNKNOWN";
-    modifier = 1;
-    TAU_METADATA("OMP_SCHEDULE", schedule);
-    sprintf(tmpstr,"%d",modifier);
-    TAU_METADATA("OMP_CHUNK_SIZE", tmpstr);
-#endif
-
-    int value = omp_get_max_threads();
-    sprintf(tmpstr,"%d",value);
-    TAU_METADATA("OMP_MAX_THREADS", tmpstr);
-
-    value = omp_get_num_procs();
-    sprintf(tmpstr,"%d",value);
-    TAU_METADATA("OMP_NUM_PROCS", tmpstr);
-
-    value = omp_get_dynamic();
-    sprintf(tmpstr,"%s",value?"on":"off");
-    TAU_METADATA("OMP_DYNAMIC", tmpstr);
-
-    value = omp_get_nested();
-    sprintf(tmpstr,"%s",value?"on":"off");
-    TAU_METADATA("OMP_NESTED", tmpstr);
-
-#if defined(omp_get_thread_limit)
-    value = omp_get_thread_limit();
-    sprintf(tmpstr,"%d",value);
-    TAU_METADATA("OMP_THREAD_LIMIT", tmpstr);
-#endif
-
-#if defined(omp_get_max_active_levels)
-    value = omp_get_max_active_levels();
-    sprintf(tmpstr,"%d",value);
-    TAU_METADATA("OMP_MAX_ACTIVE_LEVELS", tmpstr);
-#endif
-#endif
 
     tmp = getconf("TAU_MEASURE_TAU");
     if (parse_bool(tmp, TAU_EBS_DEFAULT_TAU)) {
@@ -2180,7 +2353,8 @@ void TauEnv_initialize()
     }
 
     tmp = getconf("TAU_SAMPLING");
-    if (parse_bool(tmp, TAU_EBS_DEFAULT)) {
+    // We should disable sampling if tracing has been enabled!
+    if (parse_bool(tmp, TAU_EBS_DEFAULT) && (env_tracing == 0)) {
 #ifdef TAU_DISABLE_MEM_MANAGER
       env_ebs_enabled = 0;
       TAU_VERBOSE("TAU: Sampling Disabled - memory management was disabled at configuration!\n");
@@ -2210,7 +2384,7 @@ void TauEnv_initialize()
       // *CWL* Acquire the sampling source. This has to be done first
       //       because the default EBS_PERIOD will depend on whether
       //       the specified source relies on timer interrupts or
-      //       PAPI overflow interrupts or some other future 
+      //       PAPI overflow interrupts or some other future
       //       mechanisms for triggering samples. The key problem with
       //       EBS_PERIOD defaults are that they are source-semantic
       //       sensitive (ie. 1000 microseconds is fine for timer
@@ -2233,7 +2407,7 @@ void TauEnv_initialize()
       //         number for now. The reason for a prime number? So we
       //         do not get into cyclical sampling problems on sources
       //         like L1 cache misses.
-      // 
+      //
       //         The check for PAPI sources will be extremely naive for
       //         now.
       if (strncmp(env_ebs_source, "PAPI", 4) == 0) {
@@ -2320,8 +2494,9 @@ void TauEnv_initialize()
           } else if (strcmp(ebs_resolution, TAU_EBS_RESOLUTION_STR_FUNCTION) == 0) {
               env_ebs_resolution = TAU_EBS_RESOLUTION_FUNCTION;
               TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_FUNCTION);
-          } else { // otherwise, it's the default - line.
-              TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_DEFAULT);
+          } else if (strcmp(ebs_resolution, TAU_EBS_RESOLUTION_STR_LINE) == 0) { // otherwise, it's the default - line.
+              env_ebs_resolution = TAU_EBS_RESOLUTION_LINE;
+              TAU_METADATA("TAU_EBS_RESOLUTION", TAU_EBS_RESOLUTION_STR_LINE);
           }
       }
 
@@ -2412,7 +2587,7 @@ void TauEnv_initialize()
       // get arg of sass type
       const char *sass_type = getconf("TAU_SASS_TYPE");
       if (sass_type) {
-	env_sass_type = sass_type; 
+	env_sass_type = sass_type;
       }
       TAU_VERBOSE("TAU: SASS type = %s \n", env_sass_type);
       sprintf(tmpstr, "%s", env_sass_type);
@@ -2441,7 +2616,15 @@ void TauEnv_initialize()
       TAU_VERBOSE("TAU: CUDA binary exe: %s\n", env_binaryexe);
       TAU_METADATA("TAU_CUDA_BINARY_EXE", env_binaryexe);
     }
-
+    tmp = getconf("TAU_TRACK_CUDA_ENV");
+    if (parse_bool(tmp, TAU_TRACK_CUDA_ENV_DEFAULT)) {
+      env_track_cuda_env = 1;
+      TAU_VERBOSE("TAU: tracking CUDA Environment Enabled\n");
+      TAU_METADATA("TAU_TRACK_CUDA_ENV", "on");
+    } else {
+      TAU_VERBOSE("TAU: tracking CUDA Environment Disabled\n");
+      TAU_METADATA("TAU_TRACK_CUDA_ENV", "off");
+    }
     tmp = getconf("TAU_MIC_OFFLOAD");
     if (parse_bool(tmp, TAU_MIC_OFFLOAD_DEFAULT)) {
       env_mic_offload = 1;
@@ -2498,7 +2681,7 @@ void TauEnv_initialize()
     } else {
       TAU_VERBOSE("TAU: MEM_CLASSES is \"%s\"\n", env_mem_classes);
       if(strcmp(env_mem_classes, "all") == 0) {
-        env_mem_all = 1; 
+        env_mem_all = 1;
         TAU_VERBOSE("TAU: Tracking All Class Allocations\n");
       }
       env_mem_classes_set = new std::set<std::string>();
@@ -2529,20 +2712,28 @@ void TauEnv_initialize()
 /* Add metadata in the form of "<key1=val1:key2=val2:key3=val3>" */
     char *metadata = (char *) getconf("TAU_METADATA");
 
-#ifndef TAU_WINDOWS 
-    // export TAU_METADATA="<key1=val1:key2=val2:key3=val3>" 
+#ifndef TAU_WINDOWS
+    // export TAU_METADATA="<key1=val1:key2=val2:key3=val3>"
     if (metadata) {
       key = strtok_r(metadata, "<=,>", &saveptr);
       while (key != (char *) NULL) {
         val = strtok_r(NULL, ":>", &saveptr);
         TAU_VERBOSE("TAU_METADATA %s = %s \n", key, val);
-        TAU_METADATA(key, val); 
+        TAU_METADATA(key, val);
         key = strtok_r(NULL, "=", &saveptr); // get the next pair
       }
     }
 #endif /* TAU_WINDOWS - use strtok under Windows */
+    /* Now that we have set all the options, start the one signal
+     * handler that will handle load, power, memory, headroom, etc.
+     */
+    //TauSetupHandler();
   }
 
+  TAU_VERBOSE("Calling TAU_ROCTRACER...\n");
+#ifdef TAU_ENABLE_ROCTRACER
+  Tau_roctracer_start_tracing();
+#endif /* TAU_ENABLE_ROCTRACER */
 }
 
 } /* C linkage */
