@@ -560,7 +560,7 @@ void tau_profile_init_()
 {
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
-  Tau_set_node(0); 
+  if (Tau_get_node() == -1) { Tau_set_node(0); }
 #endif /* TAU_SHMEM */
 #endif /* TAU_MPI */
 }
@@ -944,7 +944,7 @@ void TAU_PROFILE_INIT()
   // tau_profile_init_(argc, argv);
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
-  Tau_set_node(0); 
+  if (Tau_get_node() == -1) { Tau_set_node(0); }
 #endif /* TAU_SHMEM */
 #endif /* TAU_MPI */
 }
@@ -953,7 +953,7 @@ void TAU_PROFILE_INIT_()
 {
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
-  Tau_set_node(0); 
+  if (Tau_get_node() == -1) { Tau_set_node(0); }
 #endif /* TAU_SHMEM */
 #endif /* TAU_MPI */
 }
@@ -1393,7 +1393,7 @@ void tau_profile_init(void)
 
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
-  Tau_set_node(0); 
+  if (Tau_get_node() == -1) { Tau_set_node(0); }
 #endif /* TAU_SHMEM */
 #endif /* TAU_MPI */
 }
@@ -1518,7 +1518,7 @@ void tau_profile_init__()
   // tau_profile_init_(argc, argv);
 #ifndef TAU_MPI
 #ifndef TAU_SHMEM
-  Tau_set_node(0); 
+  if (Tau_get_node() == -1) { Tau_set_node(0); }
 #endif /* TAU_SHMEM */
 #endif /* TAU_MPI */
 }
@@ -1895,7 +1895,7 @@ void tau_alloc_(void ** ptr, int* line, int *size, char *name, int slen)
     char const * localname;
     int locallen;
     getFortranName(&localname, &locallen, name, slen);
-    if (!Tau_memory_wrapper_is_registered()) {
+    if (Tau_memory_wrapper_is_registered()) {
       Tau_track_memory_allocation((void*)ptr, *size, localname, *line);
     }
     free((void*)localname);
@@ -1923,7 +1923,7 @@ void tau_dealloc_(void ** ptr, int* line, char *name, int slen)
     char const * localname;
     int locallen;
     getFortranName(&localname, &locallen, name, slen);
-    if (!Tau_memory_wrapper_is_registered()) {
+    if (Tau_memory_wrapper_is_registered()) {
       Tau_track_memory_deallocation((void*)ptr, localname, *line);
     }
     free((void*)localname);
