@@ -793,7 +793,7 @@ void TauStartOpenMPRegionTimer(struct ompregdescr *r, int index)
 #else 
   FunctionInfo *f = (FunctionInfo *)r->data;
 #endif
-  Tau_start_timer(f, 0, Tau_get_tid());
+  Tau_start_timer(f, 0, Tau_get_thread());
   
   omp_unset_lock(&tau_ompregdescr_lock);
 }
@@ -815,7 +815,7 @@ void TauStopOpenMPRegionTimer(struct ompregdescr *r, int index)
     int tid = RtsLayer::myThread(); 
     Profiler *p =TauInternal_CurrentProfiler(tid); 
     if (p != NULL && p->ThisFunction == f) {
-      Tau_stop_timer(f, Tau_get_tid());
+      Tau_stop_timer(f, Tau_get_thread());
     } else {
       // nothing, it must have been disabled/throttled
     }
@@ -832,7 +832,7 @@ void POMP_Finalize() {
     omp_fin_called = 1;
   }
 #ifdef DEBUG_PROF
-  fprintf(stderr, "  0: finalize\n");
+  TAU_VERBOSE( "  0: finalize\n");
 #endif /* DEBUG_PROF */
 }
 
@@ -842,7 +842,7 @@ void POMP_Init() {
   atexit(POMP_Finalize);
 
 #ifdef DEBUG_PROF
-  fprintf(stderr, "  0: init\n");
+  TAU_VERBOSE( "  0: init\n");
 #endif /* DEBUG_PROF */
 
   for(i=0; i<POMP_MAX_ID; ++i) {
@@ -874,7 +874,7 @@ void POMP_Atomic_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter atomic\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: enter atomic\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -891,7 +891,7 @@ void POMP_Atomic_exit(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  atomic\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: exit  atomic\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -909,9 +909,9 @@ void POMP_Barrier_enter(struct ompregdescr* r) {
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
     if ( r->name[0] == 'b' )
-      fprintf(stderr, "%3d: enter barrier\n", omp_get_thread_num());
+      TAU_VERBOSE( "%3d: enter barrier\n", omp_get_thread_num());
     else
-      fprintf(stderr, "%3d: enter implicit barrier of %s\n",
+      TAU_VERBOSE( "%3d: enter implicit barrier of %s\n",
 	      omp_get_thread_num(), r->name);
   }
 #endif /* DEBUG_PROF */
@@ -931,9 +931,9 @@ void POMP_Barrier_exit(struct ompregdescr* r) {
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
     if ( r->name[0] == 'b' )
-      fprintf(stderr, "%3d: exit  barrier\n", omp_get_thread_num());
+      TAU_VERBOSE( "%3d: exit  barrier\n", omp_get_thread_num());
     else
-      fprintf(stderr, "%3d: exit  implicit barrier of %s\n",
+      TAU_VERBOSE( "%3d: exit  implicit barrier of %s\n",
 	      omp_get_thread_num(), r->name);
   }
 #endif /* DEBUG_PROF */
@@ -951,7 +951,7 @@ void POMP_Critical_begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin critical %s\n",
+    TAU_VERBOSE( "%3d: begin critical %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -970,7 +970,7 @@ void POMP_Critical_end(struct ompregdescr* r) {
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   critical %s\n",
+    TAU_VERBOSE( "%3d: end   critical %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -988,7 +988,7 @@ void POMP_Critical_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter critical %s\n",
+    TAU_VERBOSE( "%3d: enter critical %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1007,7 +1007,7 @@ void POMP_Critical_exit(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  critical %s\n",
+    TAU_VERBOSE( "%3d: exit  critical %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1026,7 +1026,7 @@ void POMP_For_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter for\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: enter for\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1045,7 +1045,7 @@ void POMP_For_exit(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  for\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: exit  for\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1062,7 +1062,7 @@ void POMP_Master_begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin master\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: begin master\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1080,7 +1080,7 @@ void POMP_Master_end(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   master\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: end   master\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1099,7 +1099,7 @@ void POMP_Parallel_begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin parallel\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: begin parallel\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1117,7 +1117,7 @@ void POMP_Parallel_end(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   parallel\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: end   parallel\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1135,7 +1135,7 @@ void POMP_Parallel_fork(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: fork  parallel\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: fork  parallel\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1153,7 +1153,7 @@ void POMP_Parallel_join(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: join  parallel\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: join  parallel\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1170,7 +1170,7 @@ void POMP_Section_begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin section\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: begin section\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1188,7 +1188,7 @@ void POMP_Section_end(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   section\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: end   section\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1205,7 +1205,7 @@ void POMP_Sections_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter sections\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: enter sections\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1223,7 +1223,7 @@ void POMP_Sections_exit(struct ompregdescr* r) {
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  sections\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: exit  sections\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1240,7 +1240,7 @@ void POMP_Single_begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin single\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: begin single\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1258,7 +1258,7 @@ void POMP_Single_end(struct ompregdescr* r) {
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   single\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: end   single\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1275,7 +1275,7 @@ void POMP_Single_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter single\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: enter single\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1292,7 +1292,7 @@ void POMP_Single_exit(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  single\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: exit  single\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1309,7 +1309,7 @@ void POMP_Workshare_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: enter workshare\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: enter workshare\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1327,7 +1327,7 @@ void POMP_Workshare_exit(struct ompregdescr* r) {
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: exit  workshare\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: exit  workshare\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
 }
@@ -1344,7 +1344,7 @@ void POMP_Begin(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: begin region %s\n",
+    TAU_VERBOSE( "%3d: begin region %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1363,7 +1363,7 @@ void POMP_End(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: end   region %s\n",
+    TAU_VERBOSE( "%3d: end   region %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1382,7 +1382,7 @@ void POMP_Flush_enter(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: flush enter region %s\n",
+    TAU_VERBOSE( "%3d: flush enter region %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1401,7 +1401,7 @@ void POMP_Flush_exit(struct ompregdescr* r) {
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: flush exit   region %s\n",
+    TAU_VERBOSE( "%3d: flush exit   region %s\n",
             omp_get_thread_num(), r->sub_name);
   }
 #endif /* DEBUG_PROF */
@@ -1412,7 +1412,7 @@ void POMP_Init_lock(omp_lock_t *s) {
   TAU_PROFILE("omp_init_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: init lock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: init lock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_init_lock(s);
@@ -1422,7 +1422,7 @@ void POMP_Destroy_lock(omp_lock_t *s) {
   TAU_PROFILE("omp_destroy_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: destroy lock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: destroy lock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_destroy_lock(s);
@@ -1432,7 +1432,7 @@ void POMP_Set_lock(omp_lock_t *s) {
   TAU_PROFILE("omp_set_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: set lock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: set lock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_set_lock(s);
@@ -1442,7 +1442,7 @@ void POMP_Unset_lock(omp_lock_t *s) {
   TAU_PROFILE("omp_unset_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: unset lock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: unset lock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_unset_lock(s);
@@ -1452,7 +1452,7 @@ int  POMP_Test_lock(omp_lock_t *s) {
   TAU_PROFILE("omp_test_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: test lock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: test lock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   return omp_test_lock(s);
@@ -1462,7 +1462,7 @@ void POMP_Init_nest_lock(omp_nest_lock_t *s) {
   TAU_PROFILE("omp_init_nest_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: init nestlock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: init nestlock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_init_nest_lock(s);
@@ -1472,7 +1472,7 @@ void POMP_Destroy_nest_lock(omp_nest_lock_t *s) {
   TAU_PROFILE("omp_destroy_nest_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: destroy nestlock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: destroy nestlock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_destroy_nest_lock(s);
@@ -1482,7 +1482,7 @@ void POMP_Set_nest_lock(omp_nest_lock_t *s) {
   TAU_PROFILE("omp_set_nest_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: set nestlock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: set nestlock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_set_nest_lock(s);
@@ -1492,7 +1492,7 @@ void POMP_Unset_nest_lock(omp_nest_lock_t *s) {
   TAU_PROFILE("omp_unset_nest_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: unset nestlock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: unset nestlock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   omp_unset_nest_lock(s);
@@ -1502,7 +1502,7 @@ int  POMP_Test_nest_lock(omp_nest_lock_t *s) {
   TAU_PROFILE("omp_test_nest_lock", "[OpenMP]", OpenMP);
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
-    fprintf(stderr, "%3d: test nestlock\n", omp_get_thread_num());
+    TAU_VERBOSE( "%3d: test nestlock\n", omp_get_thread_num());
   }
 #endif /* DEBUG_PROF */
   return omp_test_nest_lock(s);
