@@ -38,6 +38,7 @@ static int *tau_pvar_count;
 static int *tau_cvar_num_vals;
 static int tau_initial_pvar_count = 0;
 static int tau_mpi_t_is_initialized = 0;
+static MPI_Comm default_comm = MPI_COMM_WORLD;
 int num_cvars = 0; //For now, we don't support case where number of CVARS changes dynamically at runtime
 
 //////////////////////////////////////////////////////////////////////
@@ -47,6 +48,10 @@ extern void Tau_track_pvar_event(int current_pvar_index, int current_pvar_subind
 extern void Tau_allocate_pvar_event(int num_pvars, const int *tau_pvar_count);
 extern void *Tau_MemMgr_malloc(int tid, size_t size);
 extern void Tau_MemMgr_free(int tid, void *addr, size_t size);
+extern char * Tau_get_pvar_name(int i, int j);
+int Tau_msg_ini();
+
+int Tau_mpi_t_initialize();
 
 #define dprintf TAU_VERBOSE
 
@@ -156,7 +161,12 @@ int Tau_mpi_t_initialize(void) {
     tau_mpi_datatype[i] = datatype;
 
   /* allocate a pvar handle that will be used later */
-  return_val = MPI_T_pvar_handle_alloc(tau_pvar_session, i, NULL, &tau_pvar_handles[i], &tau_pvar_count[i]);
+  if(bind == MPI_T_BIND_MPI_COMM) {
+    return_val = MPI_T_pvar_handle_alloc(tau_pvar_session, i, &default_comm, &tau_pvar_handles[i], &tau_pvar_count[i]);
+  } else {
+    return_val = MPI_T_pvar_handle_alloc(tau_pvar_session, i, NULL, &tau_pvar_handles[i], &tau_pvar_count[i]);
+  }
+  
   if (return_val != MPI_SUCCESS) {
     perror("MPI_T_pvar_handle_alloc ERROR:");
     return return_val;
@@ -306,7 +316,7 @@ void Tau_mpi_t_add_vector_element(VectorControlVariable *vectors, char *vector_n
   ListStringPair *current_string_pair;
 
   for(iter = 0; iter < *current_vector; iter++) {
-    //Found a vector with the same name that already exits
+    //Found a vector with the same name that already exists
     if(strcmp(vectors[iter].name, vector_name) == 0) {
       counter = 0;
       current_string_pair = vectors[iter].list;
@@ -760,6 +770,7 @@ int Tau_mpi_t_cvar_initialize(void) {
     }
   }
 
+  Tau_msg_init(); 
   return_val = Tau_mpi_t_print_all_cvar_desc(num_cvars);
   if(return_val != MPI_SUCCESS) {
     printf("TAU: Rank %d: Can't read the MPI_T control variables in this MPI implementation\n", rank);
@@ -807,8 +818,6 @@ int Tau_track_mpi_t_here(void) {
   if(return_val != MPI_SUCCESS) 
     return return_val;
 
-  fprintf(stdout, "Track MPIT PVARs\n");
-
   /* get number of pvars from MPI_T */
   return_val = MPI_T_pvar_get_num(&num_pvars);
   if (return_val != MPI_SUCCESS) {
@@ -875,21 +884,17 @@ int Tau_track_mpi_t_here(void) {
           Tau_track_pvar_event(i, j, tau_pvar_count, tau_initial_pvar_count, mydata);
      
           //TauInternalFunctionGuard protects_this_function; 
-          fprintf(stdout, "TAU track MPI-T: pvar initial count=%d, pvar count=%d, pvar index=%d, pvar sub index=%d, pvar value=%llu\n", tau_initial_pvar_count, tau_pvar_count[i], i, j, mydata);
+          dprintf("MPI_T: pvar initial count=%d, pvar count=%d, pvar index=%d, pvar sub index=%d, pvar value=%llu\n", tau_initial_pvar_count, tau_pvar_count[i], i, j, mydata);
           /*Invoke plugins only if both plugin path and plugins are specified*/
           if(TauEnv_get_plugins_enabled()) {
             Tau_plugin_event_mpit_data_t plugin_data;
  
-            //fprintf(stdout, " Track MPI-T PVARs - get pvar name: %s\n", Tau_get_pvar_name(i,j));
-            //std::cout << "Track MPI-T PVARs - get pvar name: " << Tau_get_pvar_name(i,j) << std::endl;
             char * pvar_name_char = Tau_get_pvar_name(i,j);
-            fprintf(stdout, " Track MPI-T PVARs - get pvar name: %s\n", pvar_name_char);
-            //dprintf("MPI-T invoke callback: pvar_name char: %s\n", pvar_name_str);
             plugin_data.pvar_name = Tau_get_pvar_name(i,j);
             plugin_data.pvar_index = j;
             plugin_data.pvar_value = mydata;
             dprintf("MPI-T invoke callback\n"); 
-            Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_MPIT, &plugin_data);
+            Tau_util_invoke_callbacks(TAU_PLUGIN_EVENT_MPIT, plugin_data.pvar_name, &plugin_data);
           }
  
         }
@@ -901,6 +906,338 @@ int Tau_track_mpi_t_here(void) {
   dprintf("Finished!!\n");
 }
 
+////////////////////////////////////////////////////////////////////////////////////
+int Tau_set_mpi_t_cvar_value_for_communicator(char* cname, int value, void* comm, const char *comm_name){
+  MPI_T_cvar_handle handle;
+  int err = MPI_SUCCESS;
+  int read_value, count, cvar_index = 0;
+ 
+#if (defined MVAPICH2_NUMVERSION && MVAPICH2_NUMVERSION > 20200300) || (!defined MVAPICH2_NUMVERSION)
+  printf("TAU: rank [%d] Tau_set_mpi_t_cvar_value_for_communicator: CVAR name=%s, value = %d, comm_name=%s, comm = %p\n", Tau_get_node(), cname, value, comm_name, comm);
+  err = MPI_T_cvar_get_index( cname , &cvar_index);
+  if(err!=MPI_SUCCESS) {
+    fprintf(stderr,"\nTAU: Error: cvar_get_index err=%d\n",err);
+  }
+  else {
+    dprintf("Comm_name = %s, cvar = %s, value = %d, cvar index = %d, comm = %p\n", 
+      comm_name, cname, value, cvar_index, comm); 
+  } 
+  err = MPI_T_cvar_handle_alloc(cvar_index, comm, &handle, &count);
+  if(err!=MPI_SUCCESS) {
+    fprintf(stderr, "\nTAU: Error: cvar_handle_alloc err=%d\n", err);
+  }
+  
+  err = MPI_T_cvar_read(handle, &read_value);
+  if(err!=MPI_SUCCESS) {
+    fprintf(stderr, "\nTAU: Error cvar_read error err=%d\n", err);
+  }
+
+  err = MPI_T_cvar_write(handle, &value);
+  if(err!=MPI_SUCCESS) {
+    fprintf(stderr,"\nTAU: Error: cvar_write err=%d\n",err);
+  }
+
+  /* check if the cvar can be read properly - it is the same as what we wrote above. */
+  err = MPI_T_cvar_read(handle, &read_value);
+  if(err!=MPI_SUCCESS) {
+    fprintf(stderr, "\nTAU: Error: cvar_read err=%d\n", err);
+  }
+
+
+  /* free the handle. */
+  MPI_T_cvar_handle_free(&handle);
+#else
+  fprintf(stderr, "\nTAU: Warning: Communicator-based CVAR support not available for MVAPICH2.2 and older. Please use a newer MVAPICH2 version.\n");
+#endif
+  return 0;
+}
+
+void Tau_show_mpi_t_usage() {
+  if (Tau_get_node() == 0) {
+    printf("TAU: Error: Please set TAU_MPI_T_COMM_METRIC_VALUES environment variable to <comm_name,cvar_name,cvar_value>,<...>\n");
+    return;  
+  }
+}
+//////////////////////////////////////////////////////////////////////
+void Tau_mpi_t_check_communicator(void *comm, const char *comm_name) {
+  const char *communicator;
+  const char *cvar_metric;
+  const char *cvar_value;
+  const char *comm_metric_values = TauEnv_get_mpi_t_comm_metric_values(); 
+  
+  dprintf("TAU_MPI_T_Check_communicator: comm: %p, name = %s\n", comm, comm_name);
+  if (TauEnv_get_track_mpi_t_comm_metric_values) {
+    //comm_metric_values = TauEnv_get_mpi_t_comm_metric_values();
+    char *cvar_copy = Tau_MemMgr_malloc(Tau_get_thread(), (strlen(comm_metric_values)+1)*sizeof(char));
+    char *save_ptr, *token, *start;
+    char *comm_extracted, *metric;
+    char *value;
+
+    strcpy(cvar_copy, comm_metric_values);
+    dprintf("TAU: cvar_copy = %s\n comm=%s\n", cvar_copy, comm_name); 
+    
+    while(cvar_copy) {
+      dprintf("TAU:Tau_mpi_t_check_communicator: In loop: rank=%d: cvar_copy=%s\n", Tau_get_node(), cvar_copy);
+      start = strstr(cvar_copy, comm_name);
+      save_ptr = NULL;
+      if(start) {
+        dprintf("cvar_copy = %s, start=%s\n", cvar_copy, start); 
+        comm_extracted = strtok_r(start, ",", &save_ptr);
+        if(comm_extracted) { 
+          dprintf("CVAR Communicator Name = %s\n", comm_extracted);
+        } else {
+          Tau_show_mpi_t_usage();  
+          return;
+        } 
+        metric = strtok_r(NULL, ",", &save_ptr);
+        if(metric) {
+          dprintf("CVAR METRIC = %s\n", metric);
+        } else {
+          Tau_show_mpi_t_usage();  
+          return;
+        } 
+        value = strtok_r(NULL, ",", &save_ptr);
+        if(value) {
+          dprintf("CVAR VALUE = %s\n", value);
+          TAU_METADATA(metric, value);
+        } else {
+          Tau_show_mpi_t_usage();
+          return;
+        }
+        int intvalue;
+        sscanf(value, "%d", &intvalue); 
+        // We now have the metric, integer value, and the communicator address:
+        Tau_set_mpi_t_cvar_value_for_communicator(metric, intvalue, comm, comm_name);
+        //printf("save_ptr = %s\n", save_ptr);
+      }
+      cvar_copy = save_ptr; 
+    }
+  }
+}
+
+/* Here we define all the profile path based tracking calls */
+typedef struct {
+    /* pvar metadata variables */
+    char        *name;
+    char        *desc;
+    int         name_len;
+    int         desc_len;
+    int         verbosity;
+    int         var_class;
+    int         bind;
+    int         readonly;
+    int         continuous;
+    int         atomic;
+    int         count;
+    MPI_Datatype    datatype;
+    MPI_T_enum      enumtype;
+} pvar_data_t;
+
+static MPI_T_pvar_handle tau_handle_send;
+static MPI_T_pvar_handle tau_handle_recv;
+static MPI_T_pvar_session tau_session; 
+static unsigned long long * tau_send_value;
+static unsigned long long * tau_recv_value;
+static int tau_send_count = 0;
+static int tau_recv_count = 0;
+
+
+//////////////////////////////////////////////////////////////////////
+int Tau_msg_init(void){
+  int err, i=0; 
+  char *send_name = malloc(sizeof(char) * TAU_NAME_LENGTH);
+  char *send_description = malloc(sizeof(char) * TAU_NAME_LENGTH);
+  char *recv_name = malloc(sizeof(char) * TAU_NAME_LENGTH);
+  char *recv_description = malloc(sizeof(char) * TAU_NAME_LENGTH);
+  int pvar_send_index = 0;
+  int pvar_recv_index = 0;
+  pvar_data_t *pvar_send = malloc(sizeof(pvar_data_t));
+  pvar_data_t *pvar_recv = malloc(sizeof(pvar_data_t));
+  pvar_send->name_len = TAU_NAME_LENGTH;
+  pvar_send->desc_len = TAU_NAME_LENGTH;
+  pvar_recv->name_len = TAU_NAME_LENGTH;
+  pvar_recv->desc_len = TAU_NAME_LENGTH;
+
+  MPI_T_pvar_session_create(&tau_session);
+  /* For MVAPICH2: check if the link utilization for send is there */
+  PMPI_T_pvar_get_index("mv2_hardware_link_utilization_send", MPI_T_PVAR_CLASS_COUNTER, &pvar_send_index);
+  PMPI_T_pvar_get_info(pvar_send_index,
+            send_name,
+            &pvar_send->name_len,
+            &pvar_send->verbosity,
+            &pvar_send->var_class,
+            &pvar_send->datatype,
+            &pvar_send->enumtype,
+            send_description,
+            &pvar_send->desc_len,
+            &pvar_send->bind,
+            &pvar_send->readonly,
+            &pvar_send->continuous,
+            &pvar_send->atomic);
+  pvar_send->name = malloc(pvar_send->name_len);
+  pvar_send->desc = malloc(pvar_send->desc_len);
+  strcpy(pvar_send->name, send_name);
+  strcpy(pvar_send->desc, send_description);
+
+  /* For MVAPICH2: check if the link utilization for recv is there */
+  PMPI_T_pvar_get_index("mv2_hardware_link_utilization_recv", MPI_T_PVAR_CLASS_COUNTER, &pvar_recv_index);
+  PMPI_T_pvar_get_info(pvar_recv_index,
+            recv_name,
+            &pvar_recv->name_len,
+            &pvar_recv->verbosity,
+            &pvar_recv->var_class,
+            &pvar_recv->datatype,
+            &pvar_recv->enumtype,
+            recv_description,
+            &pvar_recv->desc_len,
+            &pvar_recv->bind,
+            &pvar_recv->readonly,
+            &pvar_recv->continuous,
+            &pvar_recv->atomic);
+  pvar_recv->name = malloc(pvar_recv->name_len);
+  pvar_recv->desc = malloc(pvar_recv->desc_len);
+  strcpy(pvar_recv->name, recv_name);
+  strcpy(pvar_recv->desc, recv_description);
+
+  /* Next, we allocate handles: */
+  MPI_Comm comm = MPI_COMM_WORLD; 
+  err = MPI_T_pvar_handle_alloc(tau_session,pvar_send_index, &comm, &tau_handle_send, &tau_send_count);
+  if (err != MPI_SUCCESS) {
+    fprintf(stderr, "\nTAU: ERROR pvar_handle_alloc failed for NO_OBJ (%d)\n", err);
+  }
+
+  err = MPI_T_pvar_handle_alloc(tau_session,pvar_recv_index, &comm, &tau_handle_recv, &tau_recv_count);
+  if (err != MPI_SUCCESS) {
+    fprintf(stderr, "\nTAU: ERROR pvar_handle_alloc failed for NO_OBJ (%d)\n", err);
+  }
+    
+  /* Parse the description of PVARs for all paths between CPUs and GPUs */
+  char **send_paths = malloc(tau_send_count * sizeof(char*));
+  TAU_VERBOSE("send_description=%s, tau_send_count=%d\n", send_description, tau_send_count);
+  TAU_VERBOSE("recv_description=%s, tau_recv_count=%d\n", recv_description, tau_recv_count);
+  int nl_found; // does it find a new line in the string?
+  for(i = 0; i < tau_send_count; i++)
+  {
+    send_paths[i] = malloc(sizeof(char) * TAU_NAME_LENGTH);
+    nl_found = 0; 
+    char *nextLine = strchr(send_description, '\n');
+    if (nextLine) {
+      nl_found = 1; // new line is found! 
+      *nextLine = '\0'; // null terminate it
+    } 
+    strcpy(send_paths[i], send_description); // chop off and store the path
+    if (nl_found) {
+      *nextLine = '\n';  
+      send_description = nextLine + 1; // if new line is found
+    }
+    TAU_METADATA_ITERATION("TAU_SEND_PATH_ID", i, send_paths[i]);
+  }
+
+  char **recv_paths = malloc(tau_recv_count * sizeof(char*));
+  for(i = 0; i < tau_recv_count; i++)
+  {
+    recv_paths[i] = malloc(sizeof(char) * TAU_NAME_LENGTH);
+    nl_found = 0; 
+    char *nextLine = strchr(recv_description, '\n');
+    if (nextLine) {
+      nl_found = 1; // new line is found! 
+      *nextLine = '\0'; // null terminate it
+    }
+    strcpy(recv_paths[i], recv_description);
+    if (nl_found) {
+      *nextLine = '\n';  
+      recv_description = nextLine + 1; // if new line is found
+    }
+    TAU_METADATA_ITERATION("TAU_RECV_PATH_ID", i, recv_paths[i]);
+  }
+    
+  /* Initialize value arrays to get link utilization for within the node and inter-node links */
+  tau_send_value = malloc(sizeof(unsigned long long) * tau_send_count);
+  tau_recv_value = malloc(sizeof(unsigned long long) * tau_recv_count);
+
+}
+
+//////////////////////////////////////////////////////////////////////
+int Tau_msg_send_prolog(void){
+  TAU_VERBOSE("Inside message send prolog\n");
+  int err; 
+  err = PMPI_T_pvar_start(tau_session, tau_handle_send);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar start in send prolog (%d)", err);
+  TAU_VERBOSE("Exiting from message send prolog\n");
+  fflush(stdout);
+  return 0;
+} 
+
+//////////////////////////////////////////////////////////////////////
+int Tau_msg_recv_prolog(void){
+  TAU_VERBOSE("Inside message recv prolog\n");
+  int err; 
+  err = PMPI_T_pvar_start(tau_session, tau_handle_recv);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar start in recv prolog (%d)", err);
+  return 0;
+} 
+
+//////////////////////////////////////////////////////////////////////
+long Tau_get_message_send_path(void) {
+  // Query MPI_T and return the path associated with the pvars. 
+  int err;
+  int i;  
+  long id = 0; 
+  int found = 0;
+  
+  err = PMPI_T_pvar_stop(tau_session, tau_handle_send);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar stop in send path (%d)", err);
+
+  /* Read send value from pvar */
+  err = PMPI_T_pvar_read(tau_session, tau_handle_send, tau_send_value);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar read in send path (%d)", err);
+
+  for(i = 0; i < tau_send_count; i++)
+  { 
+    /* If delta is > 0, send_paths[i] was used */
+    if (tau_send_value[i] > 0)
+    {
+      id = id+(pow(100,found)*i);
+      found++;
+    }
+  }
+  return id; /* default path id */
+}
+
+//////////////////////////////////////////////////////////////////////
+long Tau_get_message_recv_path(void) {
+  // Query MPI_T and return the path associated with the pvars. 
+  int err; 
+  int i, found;
+  long id = 0;
+
+  err = PMPI_T_pvar_stop(tau_session, tau_handle_recv);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar stop in recv path (%d)", err);
+
+
+   
+  /* Read recv value from pvar */
+  err = PMPI_T_pvar_read(tau_session, tau_handle_recv, tau_recv_value);
+  if (err != MPI_SUCCESS)
+    fprintf(stderr,"TAU: ERROR in MPI_T pvar read in recv path (%d)", err);
+
+  found = 0;
+  for(i = 0; i < tau_recv_count; i++)
+  { 
+    /* If delta is > 0, recv_paths[i] was used */
+    if (tau_recv_value[i] > 0) {
+      id = id+(pow(100,found)*i);
+      found++;
+    }
+  }
+  return id; /* default path id */
+
+}
 //////////////////////////////////////////////////////////////////////
 // EOF : TauMpiT.c
 //////////////////////////////////////////////////////////////////////
