@@ -82,6 +82,8 @@
 #include <signal.h>
 #include <stdlib.h>
 #include <strings.h>
+#include <stdint.h>
+#include <mutex>
 
 /* Android didn't provide <ucontext.h> so we make our own */
 #ifdef TAU_ANDROID
@@ -98,7 +100,9 @@
 #include <omp.h>
 #endif
 
+#ifndef TAU_AIX
 #include <sys/syscall.h>
+#endif /* TAU_AIX */
 #include <time.h>
 #include <unistd.h>
 #ifdef SIGEV_THREAD_ID
@@ -123,7 +127,7 @@ extern FunctionInfo * Tau_create_thread_state_if_necessary_string(const string &
 extern "C" void Tau_ompt_resolve_callsite(FunctionInfo &fi, char * resolved_address);
 extern "C" int Tau_get_usesMPI();
 
-#if defined(TAU_OPENMP) && !defined (TAU_USE_OMPT_TR6) && (defined(TAU_USE_OMPT) || defined (TAU_IBM_OMPT))
+#if defined(TAU_OPENMP) && !defined (TAU_USE_OMPT_TR6) && !defined (TAU_USE_OMPT_TR7) && !defined (TAU_USE_OMPT_5_0) && (defined(TAU_USE_OMPT) || defined (TAU_IBM_OMPT))
 extern "C" int Tau_get_thread_omp_state(int tid);
 #endif
 
@@ -195,7 +199,7 @@ extern void Tau_sampling_outputTraceCallstack(int tid, void *pc,
 //               calls made by TAU (eg. dependance on compilers) between the user
 //               code representing that context to the point in TAU where we begin to
 //               unwind the event context. All we know is we can safely drop
-//               exactly 1 call layer, which explains the "skip" variable 
+//               exactly 1 call layer, which explains the "skip" variable
 //               (see TauSampling_libunwind). This layer is invariably
 //               "Tau_sampling_event_start"
 //
@@ -205,10 +209,13 @@ extern void Tau_sampling_unwindTauContext(int tid, void **address);
 extern void Tau_sampling_unwind(int tid, Profiler *profiler,
     void *pc, void *context, unsigned long stack[]);
 
+#ifdef TAU_USE_LIBBACKTRACE
+extern void Tau_sampling_unwind_init();
+#endif
+
 extern "C" bool unwind_cutoff(void **addresses, void *address) {
   // if the unwind depth is not "auto", then return
-  if (TauEnv_get_ebs_unwind_depth() > 0) 
-    return false;
+  if (TauEnv_get_ebs_unwind_depth() > 0) return false;
   bool found = false;
   for (int i=0; i<TAU_SAMP_NUM_ADDRESSES; i++) {
     if ((unsigned long)(addresses[i]) == (unsigned long)address) {
@@ -283,16 +290,50 @@ struct CallSiteCacheNode {
 };
 
 //typedef TAU_HASH_MAP<unsigned long, CallSiteCacheNode*> CallSiteCacheMap;
-struct CallSiteCacheMap : public TAU_HASH_MAP<unsigned long, CallSiteCacheNode*> 
+struct CallSiteCacheMap : public TAU_HASH_MAP<unsigned long, CallSiteCacheNode*>
 {
   CallSiteCacheMap() {}
   virtual ~CallSiteCacheMap() {
     //Wait! We might not be done! Unbelieveable as it may seem, this map
 	//could (and does sometimes) get destroyed BEFORE we have resolved the addresses. Bummer.
-    Tau_sampling_finalize_if_necessary(Tau_get_local_tid());
+    Tau_destructor_trigger();
   }
 };
 
+#if defined(SIGEV_THREAD_ID) && !defined(TAU_BGQ) && !defined(TAU_FUJITSU)
+struct ThreadTimerMap : public TAU_HASH_MAP<int, timer_t> {
+  ThreadTimerMap() {};
+  virtual ~ThreadTimerMap() {
+    Tau_destructor_trigger();
+  };
+};
+
+static ThreadTimerMap & TheThreadTimerMap() {
+    static ThreadTimerMap threadTimerMap;
+    return threadTimerMap;
+}
+
+static std::mutex & TheThreadTimerMapMutex() {
+    static std::mutex thread_timer_map_mutex;
+    return thread_timer_map_mutex;
+}
+#endif
+
+struct DeferredInit {
+  int tid;
+  pid_t pid;
+
+  public:
+  DeferredInit(int tid, pid_t pid) : tid(tid), pid(pid) {};
+};
+
+typedef vector<DeferredInit> DeferredInitVector;
+
+static DeferredInitVector & TheDeferredInitVector() {
+  static DeferredInitVector vector;
+  return vector;
+}
+
 
 static CallSiteCacheMap & TheCallSiteCache() {
   static CallSiteCacheMap map;
@@ -333,9 +374,10 @@ struct tau_sampling_flags {
 
 #ifdef TAU_USE_TLS
 // thread local storage
-__thread struct tau_sampling_flags tau_sampling_tls_flags;
-static inline struct tau_sampling_flags *tau_sampling_flags(void)
-{ return &tau_sampling_tls_flags; }
+struct tau_sampling_flags *tau_sampling_flags(void) {
+    static thread_local struct tau_sampling_flags tau_sampling_tls_flags;
+    return &tau_sampling_tls_flags;
+}
 #elif defined(TAU_USE_DTLS)
 // thread local storage
 __declspec(thread) struct tau_sampling_flags tau_sampling_tls_flags;
@@ -353,6 +395,7 @@ static inline struct tau_sampling_flags *tau_sampling_flags(void)
 { return &tau_sampling_tls_flags[Tau_get_local_tid()]; }
 #endif
 
+static bool samplingThrInitialized[TAU_MAX_THREADS] = { false };
 
 /* The trace for this node, mulithreaded execution currently not supported */
 //FILE *ebsTrace[TAU_MAX_THREADS] = { NULL };
@@ -364,6 +407,7 @@ int collectingSamples = 0;
 // so that we can call it when we are done.
 static struct sigaction application_sa;
 
+
 /*********************************************************************
  * Get the architecture specific PC
  ********************************************************************/
@@ -376,10 +420,9 @@ static struct sigaction application_sa;
 
 #define PPC_REG_PC 32
 
-#if (defined(sun) || defined(__APPLE__) || defined(_AIX)) || \
-    (!defined(TAU_BGP) && !defined(TAU_BGQ) && !defined(__x86_64__) && \
+#if (defined(sun) || defined(__APPLE__) ||  (!defined(TAU_BGP) && !defined(TAU_BGQ) && !defined(__x86_64__) && \
     !defined(i386) && !defined(__ia64__) && !defined(__powerpc64__) && \
-	!defined(__powerpc__) && !defined(__arm__) && !defined(__aarch64__))
+	!defined(__powerpc__) && !defined(__arm__) && !defined(__aarch64__)))
 static void issueUnavailableWarning(const char *text)
 {
   static bool warningIssued = false;
@@ -413,6 +456,8 @@ unsigned long get_pc(void *p)
   pc = uct->uc_mcontext->__ss.__rip;
 #elif defined (__i386__)
   pc = uct->uc_mcontext->__ss.__eip;
+#elif defined (__arm64__)
+  pc = uct->uc_mcontext->__ss.__pc;
 #else
   pc = uct->uc_mcontext->__ss.__srr0;
 #endif /* defined(_STRUCT_X86_THREAD_STATE64) && !defined(__i386__) */
@@ -421,8 +466,20 @@ unsigned long get_pc(void *p)
 /* AIX SUPPORT */
 
 #elif _AIX
+/*
   issueUnavailableWarning("Warning, TAU Sampling does not work on AIX\n");
   return 0;
+*/
+  ucontext_t *uct = (ucontext_t *)p;
+
+  //pc = uct->uc_mcontext->jmp_context.iar;
+  if (uct != NULL)
+    pc = uct->uc_mcontext.jmp_context.iar;
+  else
+    pc = 0; 
+  // printf("pc = %p\n", pc); 
+  //pc = (((os_ucontext*)(uct))->uc_mcontext.jmp_context.iar); 
+  return pc; 
 
 /* EVERYTHING ELSE SUPPORT */
 
@@ -452,8 +509,10 @@ unsigned long get_pc(void *p)
   pc = (unsigned long)sc->arm_pc;
 # elif __aarch64__
   pc = (unsigned long)sc->pc;
+# elif __NEC__
+  pc = (unsigned long)sc->IC;
 #elif defined(TAU_FUJITSU)
-  pc = ((struct sigcontext *)p)->sigc_regs.tpc; 
+  pc = ((struct sigcontext *)p)->sigc_regs.tpc;
 # else
   issueUnavailableWarning("Warning, TAU Sampling does not work on unknown platform.\n");
   return 0;
@@ -477,6 +536,41 @@ extern "C" void Tau_sampling_resume(int tid)
   tau_sampling_flags()->suspendSampling = 0;
 }
 
+extern "C" void Tau_sampling_timer_pause() {
+#if defined(SIGEV_THREAD_ID) && !defined(TAU_BGQ) && !defined(TAU_FUJITSU)
+  std::lock_guard<std::mutex> guard(TheThreadTimerMapMutex());
+  auto it = TheThreadTimerMap().find(RtsLayer::getTid());
+  if(it != TheThreadTimerMap().end()) {
+    struct itimerspec ts;
+    ts.it_interval.tv_nsec = ts.it_value.tv_nsec = 0;
+    ts.it_interval.tv_sec  = ts.it_value.tv_sec  = 0;
+    TAU_VERBOSE("Pausing timer on thread %d\n", RtsLayer::getTid());
+    int ret = timer_settime(it->second, 0, &ts, NULL);
+    if(ret != 0) {
+      fprintf(stderr, "TAU: Failed to pause timer\n");
+    }
+  }
+#endif
+}
+
+extern "C" void Tau_sampling_timer_resume() {
+#if defined(SIGEV_THREAD_ID) && !defined(TAU_BGQ) && !defined(TAU_FUJITSU)
+  std::lock_guard<std::mutex> guard(TheThreadTimerMapMutex());
+  auto it = TheThreadTimerMap().find(RtsLayer::getTid());
+  if(it != TheThreadTimerMap().end()) {
+    int threshold = TauEnv_get_ebs_period();
+    struct itimerspec ts;
+    ts.it_interval.tv_nsec = ts.it_value.tv_nsec = (threshold % TAU_MILLION) * TAU_THOUSAND;
+    ts.it_interval.tv_sec  = ts.it_value.tv_sec  = threshold / TAU_MILLION;
+    TAU_VERBOSE("Resuming timer on thread %d\n", RtsLayer::getTid());
+    int ret = timer_settime(it->second, 0, &ts, NULL);
+    if(ret != 0) {
+      fprintf(stderr, "TAU: Failed to resume timer\n");
+    }
+  }
+#endif
+}
+
 // TODO: Why is this here?  For HPC Toolkit?
 extern "C" void Tau_sampling_dlopen()
 {
@@ -727,6 +821,14 @@ char *Tau_sampling_getShortSampleName(const char *sampleName)
   return NULL;
 }
 
+#ifdef TAU_BFD
+int Tau_get_lineno_for_function(tau_bfd_handle_t bfd_handle, char const * funcname);
+#else /* TAU_BFD */
+int Tau_get_lineno_for_function(tau_bfd_handle_t bfd_handle, char const * funcname) {
+  return 10;
+}
+#endif /* TAU_BFD */
+
 extern "C"
 CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag,
     char const * childName, char **newShortName, bool addAddress)
@@ -747,9 +849,9 @@ CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag
     node = callSiteCache[addr];
     if (!node) {
       node = new CallSiteCacheNode;
-#if defined(__APPLE__) 
+#if defined(__APPLE__)
 #if defined(TAU_HAVE_CORESYMBOLICATION)
-      static CSSymbolicatorRef symbolicator = CSSymbolicatorCreateWithPid(getpid()); 
+      static CSSymbolicatorRef symbolicator = CSSymbolicatorCreateWithPid(getpid());
       CSSourceInfoRef source_info = CSSymbolicatorGetSourceInfoWithAddressAtTime(symbolicator, (vm_address_t)addr, kCSNow);
       if(CSIsNull(source_info)) {
           node->resolved = false;
@@ -805,55 +907,55 @@ CallSiteInfo * Tau_sampling_resolveCallSite(unsigned long addr, char const * tag
     // make sure we allocate enough space for the buffer!!!!
     if (childName) {
         if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FILE) {
-            buff = (char*)malloc(strlen(tag) + strlen(childName) + 
+            buff = (char*)malloc(strlen(tag) + strlen(childName) +
                     strlen(resolvedInfo.filename) + 32);
             sprintf(buff, "[%s] %s [@] [{%s} {0}]",
                 tag, childName, resolvedInfo.filename);
         } else if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FUNCTION) {
-            buff = (char*)malloc(strlen(tag) + strlen(childName) + 
-                    strlen(resolvedInfo.funcname) + 
+            buff = (char*)malloc(strlen(tag) + strlen(childName) +
+                    strlen(resolvedInfo.funcname) + strlen(resolvedInfo.filename) +
                     strlen(lineno) + 32);
             sprintf(buff, "[%s] %s [@] %s [{%s} {0}]",
-                tag, childName, resolvedInfo.funcname, 
+                tag, childName, resolvedInfo.funcname,
                 resolvedInfo.filename);
         } else { // Line resolution
-            buff = (char*)malloc(strlen(tag) + strlen(childName) + 
-                    strlen(resolvedInfo.funcname) + 
-                    strlen(resolvedInfo.filename) + 
+            buff = (char*)malloc(strlen(tag) + strlen(childName) +
+                    strlen(resolvedInfo.funcname) +
+                    strlen(resolvedInfo.filename) +
                     strlen(lineno) + 32);
             sprintf(buff, "[%s] %s [@] %s [{%s} {%d}]",
-                tag, childName, resolvedInfo.funcname, 
+                tag, childName, resolvedInfo.funcname,
                 resolvedInfo.filename, resolvedInfo.lineno);
         }
     } else {
         if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FILE) {
-            buff = (char*)malloc(strlen(tag) + 
+            buff = (char*)malloc(strlen(tag) +
                     strlen(resolvedInfo.filename) + 32);
             sprintf(buff, "[%s] [{%s} {0}]",
                 tag, resolvedInfo.filename);
+            *newShortName = (char*)malloc(strlen(resolvedInfo.filename) + 2);
+            sprintf(*newShortName, "%s", resolvedInfo.filename);
         } else if (TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_FUNCTION) {
-            buff = (char*)malloc(strlen(tag) + 
-                    strlen(resolvedInfo.funcname) + 
+            buff = (char*)malloc(strlen(tag) +
+                    strlen(resolvedInfo.funcname) +
                     strlen(resolvedInfo.filename) + 32);
             sprintf(buff, "[%s] %s [{%s} {0}]",
-                tag, resolvedInfo.funcname, 
+                tag, resolvedInfo.funcname,
                 resolvedInfo.filename);
+            *newShortName = (char*)malloc(strlen(resolvedInfo.funcname) + 2);
+            sprintf(*newShortName, "%s", resolvedInfo.funcname);
         } else { // Line resolution
-            buff = (char*)malloc(strlen(tag) + 
-                    strlen(resolvedInfo.funcname) + 
-                    strlen(resolvedInfo.filename) + 
+            buff = (char*)malloc(strlen(tag) +
+                    strlen(resolvedInfo.funcname) +
+                    strlen(resolvedInfo.filename) +
                     strlen(lineno) + 32);
             sprintf(buff, "[%s] %s [{%s} {%d}]",
-                tag, resolvedInfo.funcname, 
+                tag, resolvedInfo.funcname,
                 resolvedInfo.filename, resolvedInfo.lineno);
+            *newShortName = (char*)malloc(strlen(resolvedInfo.filename) + strlen(lineno) + 2);
+            sprintf(*newShortName, "%s.%d", resolvedInfo.filename, resolvedInfo.lineno);
         }
     }
-    *newShortName = (char*)malloc(strlen(resolvedInfo.filename) + strlen(lineno) + 2);
-    sprintf(*newShortName, "%s.%d", resolvedInfo.filename, resolvedInfo.lineno);
-    //newName = (char*)malloc(strlen(resolvedInfo.funcname) + strlen(lineno) + 2);
-    //sprintf(newName, "%s.%d", resolvedInfo.funcname, resolvedInfo.lineno);
-    //*newShortName = newName;
-    //TAU_VERBOSE("resolved function name (newName in TauSampling.cpp) = %s\n", newName);
   } else {
     char const * mapName = "UNKNOWN";
     if (TauEnv_get_bfd_lookup()) {
@@ -915,7 +1017,7 @@ string *Tau_sampling_getPathName(unsigned int index, CallStackInfo *callStack) {
     fprintf(stderr, "ERROR: EBS attempted to access index %d of vector of length %ld\n", index, sites.size());
     exit(-1);
   }
-  
+
   startIdx = (int)(sites.size()) - 1;
   stringstream buffer;
   buffer << (sites[startIdx])->name;
@@ -954,7 +1056,10 @@ CallStackInfo * Tau_sampling_resolveCallSites(const unsigned long * addresses)
       for (int i = 2; i < length; ++i) {
         unsigned long address = addresses[i];
         callStack->callSites.push_back(Tau_sampling_resolveCallSite(
-            address, "UNWIND", prevShortName, &newShortName, addAddress));
+            address, "UNWIND",
+            ((TauEnv_get_ebs_resolution() == TAU_EBS_RESOLUTION_LINE) ?
+            prevShortName : NULL),
+            &newShortName, addAddress));
         // free the previous short name now.
         if (prevShortName) {
           free(prevShortName);
@@ -1028,7 +1133,7 @@ void Tau_sampling_finalizeProfile(int tid)
   //       we iterate TheFunctionDB()! Hence the candidates!
   for (vector<FunctionInfo*>::iterator fI_iter = TheFunctionDB().begin(); fI_iter != TheFunctionDB().end(); fI_iter++) {
     FunctionInfo * parentTauContext = *fI_iter;
-    char resolved_address[1024] = "";  
+    char resolved_address[1024] = "";
     /* If OMPT-TR6 is enabled, we need to resolve addresses that are embedded within the timer name, if they haven't already been
      * resolved. */
     if(strcmp(parentTauContext->GetPrimaryGroup(), "TAU_OPENMP") == 0 && !TauEnv_get_ompt_resolve_address_eagerly()) {
@@ -1123,7 +1228,7 @@ void Tau_sampling_finalizeProfile(int tid)
     fi_it = name2FuncInfoMap->find(iglstring);
     if (fi_it == name2FuncInfoMap->end()) {
       // Create the FunctionInfo object for the leaf Intermediate object.
-      intermediateGlobalLeaf = 
+      intermediateGlobalLeaf =
 	new FunctionInfo(iglstring,
 			 candidate->tauContext->GetType(),
 			 candidate->tauContext->GetProfileGroup(),
@@ -1145,7 +1250,7 @@ void Tau_sampling_finalizeProfile(int tid)
     fi_it = name2FuncInfoMap->find(iplstring);
     if (fi_it == name2FuncInfoMap->end()) {
       // Create the FunctionInfo object for the leaf Intermediate object.
-      intermediatePathLeaf = 
+      intermediatePathLeaf =
 	new FunctionInfo(iplstring,
 			 candidate->tauContext->GetType(),
 			 candidate->tauContext->GetProfileGroup(),
@@ -1201,7 +1306,7 @@ void Tau_sampling_finalizeProfile(int tid)
         sampleGlobalLeaf = (FunctionInfo*)fi_it->second;
       }
       RtsLayer::UnLockDB();
-      
+
       stringstream callSiteKeyName;
 	  callSiteKeyName << iplstring << " ";
 	  callSiteKeyName << candidate->tauContext->GetType() << " => ";
@@ -1320,7 +1425,9 @@ void Tau_sampling_handle_sampleProfile(void *pc, ucontext_t *context, int tid) {
     samplingContext = profiler->ThisFunction;
   }
 
+#ifndef __NEC__
   TAU_ASSERT(samplingContext != NULL, "samplingContext == NULL!");
+#endif
 
   /* Get the current metric values */
   double values[TAU_MAX_COUNTERS] = { 0.0 };
@@ -1360,7 +1467,7 @@ void Tau_sampling_handle_sampleProfile(void *pc, ucontext_t *context, int tid) {
     //printf("tid = %d, sampling previousTimestamp = %llu, period = %d\n", tid, tau_sampling_flags()->previousTimestamp[i], ebsPeriod); fflush(stdout);
   }
   //printf("tid = %d, Delta = %f, period = %d\n", tid, deltaValues[0], ebsPeriod); fflush(stdout);
-#if defined(TAU_OPENMP) && !defined(TAU_USE_OMPT_TR6)
+#if defined(TAU_OPENMP) && !defined(TAU_USE_OMPT_TR6) && !defined(TAU_USE_OMPT_TR7) && !defined(TAU_USE_OMPT_5_0)
   if (TauEnv_get_openmp_runtime_states_enabled() == 1) {
     // get the thread state, too!
 #if defined(TAU_USE_OMPT) || defined(TAU_IBM_OMPT)
@@ -1411,7 +1518,7 @@ void Tau_sampling_event_start(int tid, void **addresses)
 #endif /* TAU_UNWIND */
 
 /* Kevin here. This code is a bad idea. I have disabled it for now.
- * Sampling and instrumentation can play together just fine, if a 
+ * Sampling and instrumentation can play together just fine, if a
  * timer starts / stops, it shouldn't affect the timers for sampling.
  * And SHOULDN'T affect the timers for sampling.
  */
@@ -1577,10 +1684,17 @@ void Tau_sampling_papi_overflow_handler(int EventSet, void *address, x_int64 ove
   Tau_sampling_handle_sample(address, (ucontext_t *)context);
 }
 
+#if defined(TAU_USE_PGS)
+static void Tau_init_pgs_sampling_flags() {
+    pthread_key_create (&tau_sampling_tls_key, NULL);
+}
+#endif
+
 /*********************************************************************
  * Initialize the sampling trace system
  ********************************************************************/
-int Tau_sampling_init(int tid)
+// Set pid to 0 to determine dynamically
+int Tau_sampling_init(int tid, pid_t pid)
 {
   int ret;
 
@@ -1590,8 +1704,8 @@ int Tau_sampling_init(int tid)
   int threshold = TauEnv_get_ebs_period();
 
 #if defined(TAU_USE_PGS)
-  if (tid == 0)
-    pthread_key_create (&tau_sampling_tls_key, NULL);
+  static pthread_once_t key_initialized = PTHREAD_ONCE_INIT;
+  pthread_once(&key_initialized, Tau_init_pgs_sampling_flags);
   struct tau_sampling_flags *tau_sampling_tls_flags = (struct tau_sampling_flags*)malloc(sizeof(struct tau_sampling_flags));
   tau_sampling_tls_flags->samplingEnabled = 0;
   tau_sampling_tls_flags->suspendSampling = 0;
@@ -1642,122 +1756,151 @@ int Tau_sampling_init(int tid)
   if (strcmp(TauEnv_get_ebs_source(), "itimer") == 0 ||
       strcmp(TauEnv_get_ebs_source(), "TIME") == 0)
   {
-    // only thread 0 sets up the timer interrupts.
-    if (tid == 0) {
 #endif // TAU_BGQ
-  struct sigaction act;
 
-  // If TIME isn't on the list of TAU_METRICS, then do not sample.
-  // Eventually, we could employ a best-effort attempt to add
-  //   TAU_EBS_SOURCE to TAU_METRICS if TAU_EBS_SOURCE is not a
-  //   a member of TAU_METRICS.
-  int checkVal = TauMetrics_getMetricIndexFromName("TIME");
-  if (checkVal == -1) {
-    // *CWL* - Attempt other default (or pseudo-default) timer options.
-    //         This is probably not the best nor most efficient way.
-    //         The only saving grace is that these pseudo-default
-    //         timers are probably not going to overlap in the same run.
-    //
-    //         Essentially, we don't
-    //         really care what these timers do, if EBS_SOURCE=TIME, we
-    //         just want to find ANY time-based metric to latch the
-    //         data to.
-    const char *temp = NULL;
-    checkVal = TauMetrics_getMetricIndexFromName("TAUGPU_TIME");
-    if (checkVal != -1) {
-      temp = "TAUGPU_TIME";
-    }
-
-    checkVal = TauMetrics_getMetricIndexFromName("LINUX_TIMERS");
-    if (checkVal != -1) {
-      temp = "LINUX_TIMERS";
-    }
-
-    checkVal = TauMetrics_getMetricIndexFromName("BGL_TIMERS");
-    if (checkVal != -1) {
-      temp = "BGL_TIMERS";
-    }
-
-    checkVal = TauMetrics_getMetricIndexFromName("BGP_TIMERS");
-    if (checkVal != -1) {
-      temp = "BGP_TIMERS";
-    }
-
-    checkVal = TauMetrics_getMetricIndexFromName("BGQ_TIMERS");
-    if (checkVal != -1) {
-      temp = "BGQ_TIMERS";
-    }
-
-    checkVal = TauMetrics_getMetricIndexFromName("CRAY_TIMERS");
-    if (checkVal != -1) {
-      temp = "CRAY_TIMERS";
-    }
-
-    // If *some* pseudo-default timer is used, then override the EBS_SOURCE string.
-    //   The overriden value will eventually be used in the final EBS data resolution
-    //   phase to latch the EBS data to the appropriate metric data (which uses the
-    //   EBS_SOURCE string to figure out the metric index).
-    if (temp) {
-      TauEnv_override_ebs_source(temp);
-    } else {
-      fprintf(stderr, "TAU Sampling Warning: No time-related metric found in TAU_METRICS. "
-          "Sampling is disabled for TAU_EBS_SOURCE %s.\n", TauEnv_get_ebs_source());
-      return -1;
-    }
+  static bool sigaction_initialized = false;
+  // Deferred thread sampling init will happen inside this same function,
+  // so we only lock if we are NOT currently handling a deferred thread
+  if(pid == 0) {
+    RtsLayer::LockEnv();
   }
+  if(!sigaction_initialized) {
+    TAU_VERBOSE("sigaction is being initialized on thread %d\n", tid);
+    sigaction_initialized = true;
+    struct sigaction act;
 
-  memset(&act, 0, sizeof(struct sigaction));
-  ret = sigemptyset(&act.sa_mask);
-  if (ret != 0) {
-    fprintf(stderr, "TAU: Sampling error 1: %s\n", strerror(ret));
-    return -1;
-  }
-  ret = sigaddset(&act.sa_mask, TAU_ALARM_TYPE);
-  if (ret != 0) {
-    fprintf(stderr, "TAU: Sampling error 2: %s\n", strerror(ret));
-    return -1;
-  }
-  act.sa_sigaction = Tau_sampling_handler;
-  act.sa_flags = SA_SIGINFO | SA_RESTART;
+    // If TIME isn't on the list of TAU_METRICS, then do not sample.
+    // Eventually, we could employ a best-effort attempt to add
+    //   TAU_EBS_SOURCE to TAU_METRICS if TAU_EBS_SOURCE is not a
+    //   a member of TAU_METRICS.
+    int checkVal = TauMetrics_getMetricIndexFromName("TIME");
+    if (checkVal == -1) {
+        // *CWL* - Attempt other default (or pseudo-default) timer options.
+        //         This is probably not the best nor most efficient way.
+        //         The only saving grace is that these pseudo-default
+        //         timers are probably not going to overlap in the same run.
+        //
+        //         Essentially, we don't
+        //         really care what these timers do, if EBS_SOURCE=TIME, we
+        //         just want to find ANY time-based metric to latch the
+        //         data to.
+        const char *temp = NULL;
+        checkVal = TauMetrics_getMetricIndexFromName("TAUGPU_TIME");
+        if (checkVal != -1) {
+        temp = "TAUGPU_TIME";
+        }
 
-  // initialize the application signal action, so we can apply it
-  // after we run our signal handler
-  struct sigaction query_action;
-  ret = sigaction(TAU_ALARM_TYPE, NULL, &query_action);
-  if (ret != 0) {
-    fprintf(stderr, "TAU: Sampling error 3: %s\n", strerror(ret));
-    return -1;
-  }
-  if (query_action.sa_handler == SIG_DFL || query_action.sa_handler == SIG_IGN) {
-    ret = sigaction(TAU_ALARM_TYPE, &act, NULL);
-    if (ret != 0) {
-      fprintf(stderr, "TAU: Sampling error 4: %s\n", strerror(ret));
-      return -1;
-    }
-    //DEBUGMSG("sigaction called");
-    // the old handler was just the default or ignore.
-    memset(&application_sa, 0, sizeof(struct sigaction));
-    sigemptyset(&application_sa.sa_mask);
-    application_sa.sa_handler = query_action.sa_handler;
-  } else {
-    // FIRST! check if this is us! (i.e. we got initialized twice)
-    if (query_action.sa_sigaction == Tau_sampling_handler) {
-#ifndef TAU_BGQ // It's ok to have multiple init on BGQ
-      TAU_VERBOSE("[%d] WARNING! Tau_sampling_init called twice!\n", tid);
-#endif
-    } else {
-      TAU_VERBOSE("[%d] WARNING! Tau_sampling_init found another handler!\n", tid);
-      // install our handler, and save the old handler
-      ret = sigaction(TAU_ALARM_TYPE, &act, &application_sa);
-      if (ret != 0) {
-        fprintf(stderr, "TAU: Sampling error 5: %s\n", strerror(ret));
+        checkVal = TauMetrics_getMetricIndexFromName("LINUX_TIMERS");
+        if (checkVal != -1) {
+        temp = "LINUX_TIMERS";
+        }
+
+        checkVal = TauMetrics_getMetricIndexFromName("BGL_TIMERS");
+        if (checkVal != -1) {
+        temp = "BGL_TIMERS";
+        }
+
+        checkVal = TauMetrics_getMetricIndexFromName("BGP_TIMERS");
+        if (checkVal != -1) {
+        temp = "BGP_TIMERS";
+        }
+
+        checkVal = TauMetrics_getMetricIndexFromName("BGQ_TIMERS");
+        if (checkVal != -1) {
+        temp = "BGQ_TIMERS";
+        }
+
+        checkVal = TauMetrics_getMetricIndexFromName("CRAY_TIMERS");
+        if (checkVal != -1) {
+        temp = "CRAY_TIMERS";
+        }
+
+        // If *some* pseudo-default timer is used, then override the EBS_SOURCE string.
+        //   The overriden value will eventually be used in the final EBS data resolution
+        //   phase to latch the EBS data to the appropriate metric data (which uses the
+        //   EBS_SOURCE string to figure out the metric index).
+        if (temp != NULL) {
+        TauEnv_override_ebs_source(temp);
+        } else {
+        fprintf(stderr, "TAU Sampling Warning: No time-related metric found in TAU_METRICS. "
+            "Sampling is disabled for TAU_EBS_SOURCE %s.\n", TauEnv_get_ebs_source());
         return -1;
-      }
-      //DEBUGMSG("sigaction called");
+        }
     }
+
+    memset(&act, 0, sizeof(struct sigaction));
+    ret = sigemptyset(&act.sa_mask);
+    if (ret != 0) {
+        fprintf(stderr, "TAU: Sampling error 1: %s\n", strerror(ret));
+        return -1;
+    }
+    ret = sigaddset(&act.sa_mask, TAU_ALARM_TYPE);
+    if (ret != 0) {
+        fprintf(stderr, "TAU: Sampling error 2: %s\n", strerror(ret));
+        return -1;
+    }
+    act.sa_sigaction = Tau_sampling_handler;
+
+    // By default, we use SA_RESTART so that syscalls are automatically restarted instead of
+    // returning EINTR.
+    act.sa_flags = SA_RESTART | SA_SIGINFO;
+
+    // initialize the application signal action, so we can apply it
+    // after we run our signal handler
+    struct sigaction query_action;
+    // If the application explicitly disabled syscall restart, leave it that way.
+    ret = sigaction(TAU_ALARM_TYPE, NULL, &query_action);
+    if(query_action.sa_handler != SIG_DFL && query_action.sa_handler != SIG_IGN && !(query_action.sa_flags & SA_RESTART)) {
+        act.sa_flags = SA_SIGINFO;
+    }
+    if (ret != 0) {
+        fprintf(stderr, "TAU: Sampling error 3: %s\n", strerror(ret));
+        return -1;
+    }
+    if (query_action.sa_handler == SIG_DFL || query_action.sa_handler == SIG_IGN) {
+        ret = sigaction(TAU_ALARM_TYPE, &act, NULL);
+        if (ret != 0) {
+        fprintf(stderr, "TAU: Sampling error 4: %s\n", strerror(ret));
+        return -1;
+        }
+        //DEBUGMSG("sigaction called");
+        // the old handler was just the default or ignore.
+        memset(&application_sa, 0, sizeof(struct sigaction));
+        sigemptyset(&application_sa.sa_mask);
+        application_sa.sa_handler = query_action.sa_handler;
+    } else {
+        // FIRST! check if this is us! (i.e. we got initialized twice)
+        if (query_action.sa_sigaction == Tau_sampling_handler) {
+#ifndef TAU_BGQ // It's ok to have multiple init on BGQ
+        TAU_VERBOSE("[%d] WARNING! Tau_sampling_init called twice!\n", tid);
+#endif
+        } else {
+        TAU_VERBOSE("[%d] WARNING! Tau_sampling_init found another handler!\n", tid);
+        // install our handler, and save the old handler
+        ret = sigaction(TAU_ALARM_TYPE, &act, &application_sa);
+        if (ret != 0) {
+            fprintf(stderr, "TAU: Sampling error 5: %s\n", strerror(ret));
+            return -1;
+        }
+        //DEBUGMSG("sigaction called");
+        }
+    }
+    // Since we've now initialized sigaction, we can start the timer on any deferred threads
+    for(DeferredInitVector::iterator it = TheDeferredInitVector().begin(); it != TheDeferredInitVector().end(); ++it) {
+        if(!samplingThrInitialized[it->tid]) {
+            TAU_VERBOSE("Will create sampling timer for deferred thread %d\n", it->tid);
+            samplingThrInitialized[it->tid] = true;
+            Tau_sampling_init(it->tid, it->pid);
+        }
+    }
+  } else { //!sigaction_initialized
+      TAU_VERBOSE("In init on thread %d, sigaction already initialized; skipping\n", tid);
+  }
+  // Unlock if we're NOT handling a deferred thread (and therefore locked above)
+  if(pid == 0) {
+    RtsLayer::UnLockEnv();
   }
 #ifndef TAU_BGQ
-}    // if (tid == 0)
 #endif
 
 /* on Linux systems, we have the option of sampling based on the Wall clock
@@ -1772,13 +1915,18 @@ int Tau_sampling_init(int tid)
    sev.sigev_value.sival_ptr = &timerid;
 #ifndef TAU_ANDROID
 #ifndef TAU_FUJITSU
-   sev.sigev_notify_thread_id = syscall(__NR_gettid);
+   sev.sigev_notify_thread_id = (pid == 0 ? syscall(__NR_gettid) : pid);
 #endif /* TAU_FUJITSU */
 #else
    sev.sigev_notify_thread_id = JNIThreadLayer::GetThreadSid();
    TAU_VERBOSE(" *** (S%d) send alarm to %d\n", gettid(), sev.sigev_notify_thread_id);
 #endif
    ret = timer_create(CLOCK_REALTIME, &sev, &timerid);
+   {
+     std::lock_guard<std::mutex> guard(TheThreadTimerMapMutex());
+     TheThreadTimerMap()[pid == 0 ? RtsLayer::getTid() : pid] = timerid;
+   }
+   TAU_VERBOSE("Created sampling timer for TAU tid = %d, kernel TID = %jd\n", tid, (intmax_t)sev.sigev_notify_thread_id);
   if (ret != 0) {
     fprintf(stderr, "TAU: (%d, %d) Sampling error 6: %s\n", RtsLayer::myNode(), RtsLayer::myThread(), strerror(ret));
     return -1;
@@ -1806,6 +1954,11 @@ int Tau_sampling_init(int tid)
   itval.it_interval.tv_sec = itval.it_value.tv_sec = threshold / TAU_MILLION;
   //DEBUGMSG("threshold=%d, itimer=(%d, %d)", threshold, itval.it_interval.tv_usec, itval.it_interval.tv_sec);
 
+  if(pid != 0) {
+    fprintf(stderr, "TAU: WARNING: Sampling on thread %d was deferred, but this system does not have timer_create available."
+            "Sampling on thread %d will not occur!\n", tid, tid);
+    return -1;
+  }
   ret = setitimer(TAU_ITIMER_TYPE, &itval, &ovalue);
   if (ret != 0) {
     fprintf(stderr, "TAU: Sampling error 8: %s\n", strerror(ret));
@@ -1852,7 +2005,9 @@ int Tau_sampling_finalize(int tid)
 
   /* Disable sampling first */
   tau_sampling_flags()->samplingEnabled = 0;
-  collectingSamples = 0;
+  if(tid == 0) {
+    collectingSamples = 0;
+  }
 
   struct itimerval itval;
   int ret;
@@ -1892,13 +2047,48 @@ int Tau_sampling_finalize(int tid)
   return 0;
 }
 
+// When using MPI, we don't start sampling until MPI_Init is done.
+// However, some threads may be created before MPI_Init.
+// Any such threads are pushed onto TheDeferredInitVector, and
+// are initialized once MPI_Init happens.
+extern "C" void Tau_sampling_defer_init(void) {
+    TauInternalFunctionGuard protects_this_function;
+    const int tid = RtsLayer::localThreadId();
+#ifdef SIGEV_THREAD_ID
+#ifndef TAU_ANDROID
+#ifndef TAU_FUJITSU
+#ifdef SYS_gettid
+    const pid_t pid = syscall(SYS_gettid);
+#elif defined(__NR_gettid)
+    const pid_t pid = syscall(__NR_gettid);
+#endif
+#endif /* TAU_FUJITSU */
+#else
+    const pid_t pid = JNIThreadLayer::GetThreadSid();
+#endif
+#ifdef TAU_FX_AARCH64
+    const pid_t pid = syscall(__NR_gettid);
+#endif
+#else
+    fprintf(stderr, "TAU: WARNING: Thread %d was started before MPI_Init, but this system "
+            "doesn't support timer_create. Thread %d will not be sampled!\n", tid, tid);
+    const pid_t pid = 0;
+    return;
+#endif
+    const DeferredInit d = DeferredInit(tid, pid);
+
+    RtsLayer::LockEnv();
+    TheDeferredInitVector().push_back(d);
+    RtsLayer::UnLockEnv();
+    TAU_VERBOSE("Deferring sampling start on thread tid=%d pid=%jd\n", tid, (intmax_t)pid);
+}
+
 /* *CWL* - This is workaround code for MPI where mvapich2 on Hera was
  found to conflict with EBS sampling operations if EBS was initialized
  before MPI_Init().
  */
 extern "C" void Tau_sampling_init_if_necessary(void)
 {
-  static bool samplingThrInitialized[TAU_MAX_THREADS] = { false };
 
   // Protect TAU from itself
   TauInternalFunctionGuard protects_this_function;
@@ -1910,7 +2100,10 @@ extern "C" void Tau_sampling_init_if_necessary(void)
   // if MPI_Init hasn't been called yet. This is necessary to prevent
   // problems where the sampling signal interferes with MPI startup.
 #ifdef TAU_MPI
-  if(Tau_get_usesMPI() == 0) return;
+  if(Tau_get_usesMPI() == 0) {
+      Tau_sampling_defer_init();
+      return;
+  }
 #endif
 
   int tid = RtsLayer::localThreadId();
@@ -1950,13 +2143,13 @@ extern "C" void Tau_sampling_init_if_necessary(void)
     // do this for all threads
 	int dummy = 0;
 	int all_threads = omp_get_max_threads();
-#pragma omp parallel for ordered 
+#pragma omp parallel for ordered
     for (dummy = 0 ; dummy < all_threads ; dummy++) {
         // but do it sequentially.
 		//
         // Protect TAU from itself
         TauInternalFunctionGuard protects_this_function;
-#pragma omp ordered 
+#pragma omp ordered
       {
 #pragma omp critical (creatingtopleveltimer)
         {
@@ -1984,7 +2177,7 @@ extern "C" void Tau_sampling_init_if_necessary(void)
 // handle all other cases!
   if (!samplingThrInitialized[tid]) {
     samplingThrInitialized[tid] = true;
-    Tau_sampling_init(tid);
+    Tau_sampling_init(tid, 0);
     TAU_VERBOSE("Thread %d initialized sampling\n", tid);
   }
 #endif
@@ -2017,7 +2210,9 @@ void Tau_sampling_finalize_if_necessary(int tid)
       RtsLayer::LockEnv();
       // check again, someone else might already have finalized by now.
       if (!finalized) {
-        collectingSamples = 0;
+        if(tid == 0) {
+            collectingSamples = 0;
+        }
         finalized = true;
       }
       RtsLayer::UnLockEnv();
