@@ -142,7 +142,10 @@ int TauTraceGetFlushEvents() {
 
 /* Check that the trace file is initialized */
 static int checkTraceFileInitialized(int tid) {
-  if ( !(TraceFileInitialized[tid]) && (RtsLayer::myNode() > -1)) { 
+  if ( !(TraceFileInitialized[tid])){
+    if(RtsLayer::myNode() <= -1) {
+      fprintf (stderr,"ERROR: TAU is creating a trace file on a node less than 0.\n");
+    } 
     TraceFileInitialized[tid] = 1;
     const char *dirname;
     char tracefilename[1024];
@@ -321,7 +324,7 @@ int TauTraceInit(int tid)
    encountered for a multi-threaded program */ 
 void TauTraceReinitialize(int oldid, int newid, int tid) {
 #ifndef TAU_SETNODE0
-  printf("Inside TauTraceReinitialize : oldid = %d, newid = %d, tid = %d\n",
+  TAU_VERBOSE("Inside TauTraceReinitialize : oldid = %d, newid = %d, tid = %d\n",
 	oldid, newid, tid);
 #endif
   /* We should put a record in the trace that says that oldid is mapped to newid this 
@@ -515,7 +518,7 @@ TauTraceOffsetInfo *TheTauTraceOffsetInfo() {
 int TauTraceDumpEDF(int tid) {
   vector<FunctionInfo*>::iterator it;
   AtomicEventDB::iterator uit;
-  char filename[1024], errormsg[1024];
+  char filename[1024], errormsg[1064];
   const char *dirname;
   FILE* fp;
   int  numEvents, numExtra;
@@ -643,7 +646,7 @@ int TauTraceMergeAndConvertTracesIfNecessary(void) {
   
   const char *outdir;
   char *keepfiles;
-  char cmd[1024];
+  char cmd[4096];
   char rmcmd[256]; 
   char cdcmd[1024];
   const char *tauroot=TAUROOT;
@@ -695,21 +698,26 @@ int TauTraceMergeAndConvertTracesIfNecessary(void) {
 
 #ifdef TAU_GPU
 
-void TauTraceOneSidedMsg(int type, GpuEvent *gpu, int length, int threadId)
+void TauTraceOneSidedMsg(int type, GpuEvent *gpu, int length, int threadId, x_uint64 ts = 0UL)
 {
-		/* there are three user events that make up a one-sided msg */
-		if (type == MESSAGE_SEND)
-    	TauTraceEventSimple(TAU_ONESIDED_MESSAGE_SEND, length, threadId, TAU_TRACE_EVENT_KIND_COMM); 
-		else if (type == MESSAGE_RECV)
-    	TauTraceEventSimple(TAU_ONESIDED_MESSAGE_RECV, length, threadId, TAU_TRACE_EVENT_KIND_COMM); 
-		else if (type == MESSAGE_RECIPROCAL_SEND)
-    	TauTraceEventSimple(TAU_ONESIDED_MESSAGE_RECIPROCAL_SEND, length, threadId, TAU_TRACE_EVENT_KIND_COMM); 
-		else if (type == MESSAGE_RECIPROCAL_RECV)
-    	TauTraceEventSimple(TAU_ONESIDED_MESSAGE_RECIPROCAL_RECV, length, threadId, TAU_TRACE_EVENT_KIND_COMM); 
-		else
-    	TauTraceEventSimple(TAU_ONESIDED_MESSAGE_UNKNOWN, length, threadId, TAU_TRACE_EVENT_KIND_COMM); 
-    TauTraceEventSimple(TAU_ONESIDED_MESSAGE_ID_1, gpu->id_p1(), threadId, TAU_TRACE_EVENT_KIND_COMM); 
-    TauTraceEventSimple(TAU_ONESIDED_MESSAGE_ID_2, gpu->id_p2(), threadId, TAU_TRACE_EVENT_KIND_COMM); 
+    int use_ts = 0;
+    if (ts > 0UL) {
+        use_ts = 1;
+    }
+    /* there are three user events that make up a one-sided msg */
+    if (type == MESSAGE_SEND) {
+        TauTraceEvent(TAU_ONESIDED_MESSAGE_SEND, length, threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
+    } else if (type == MESSAGE_RECV) {
+        TauTraceEvent(TAU_ONESIDED_MESSAGE_RECV, length, threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
+    } else if (type == MESSAGE_RECIPROCAL_SEND) {
+        TauTraceEvent(TAU_ONESIDED_MESSAGE_RECIPROCAL_SEND, length, threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
+    } else if (type == MESSAGE_RECIPROCAL_RECV) {
+        TauTraceEvent(TAU_ONESIDED_MESSAGE_RECIPROCAL_RECV, length, threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
+    } else {
+        TauTraceEvent(TAU_ONESIDED_MESSAGE_UNKNOWN, length, threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM);
+    }
+    TauTraceEvent(TAU_ONESIDED_MESSAGE_ID_1, gpu->id_p1(), threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
+    TauTraceEvent(TAU_ONESIDED_MESSAGE_ID_2, gpu->id_p2(), threadId, ts, use_ts, TAU_TRACE_EVENT_KIND_COMM); 
 }
 
 #endif
@@ -804,6 +812,39 @@ void TauTraceSendMsgRemote(int type, int destination, int length, int remote_id)
   /* 0, 0 is for ts and use_ts so TAU generates the timestamp */
 }
 
+void TauTraceBarrierAllStart(int tag) {
+#ifdef TAU_OTF2
+  if(TauEnv_get_trace_format() == TAU_TRACE_FORMAT_OTF2) {
+      TauTraceOTF2BarrierAllStart(tag);
+  }
+#endif
+}
+
+void TauTraceBarrierAllEnd(int tag) {
+#ifdef TAU_OTF2
+  if(TauEnv_get_trace_format() == TAU_TRACE_FORMAT_OTF2) {
+      TauTraceOTF2BarrierAllEnd(tag);
+  }
+#endif
+}
+
+
+void TauTraceRMACollectiveBegin(int tag, int type, int start, int stride, int size, int data_in, int data_out, int root) {
+#ifdef TAU_OTF2
+  if(TauEnv_get_trace_format() == TAU_TRACE_FORMAT_OTF2) {
+      TauTraceOTF2RMACollectiveBegin(tag, type, start, stride, size, data_in, data_out, root);
+  }
+#endif
+}
+
+void TauTraceRMACollectiveEnd(int tag, int type, int start, int stride, int size, int data_in, int data_out, int root) {
+#ifdef TAU_OTF2
+  if(TauEnv_get_trace_format() == TAU_TRACE_FORMAT_OTF2) {
+      TauTraceOTF2RMACollectiveEnd(tag, type, start, stride, size, data_in, data_out, root);
+  }
+#endif
+}
+
 void TauTraceOTF2InitShmem_if_necessary() {
 #ifdef TAU_OTF2
     TauTraceOTF2InitShmem();
