@@ -18,6 +18,10 @@
 #ifndef _TAU_TF_DECL_H_
 #define _TAU_TF_DECL_H_
 
+#include <map>
+#include <utility>
+#include <functional>
+
 #include "Profile/tau_types.h"
 
 /* general declarations */
@@ -67,7 +71,7 @@ typedef struct {
   x_uint16           tid;   /* -- thread id       -- */
   x_int64            par;   /* -- event parameter -- */
   x_uint64           ti;    /* -- time [us]?      -- */
-} PCXX_EV32;
+} TAU_EV32;
 
 /* for 64 bit platforms */
 typedef struct {
@@ -77,11 +81,11 @@ typedef struct {
   x_uint32           padding; /*  space wasted for 8-byte aligning the next item */ 
   x_int64            par;   /* -- event parameter -- */
   x_uint64           ti;    /* -- time [us]?      -- */
-} PCXX_EV64;
+} TAU_EV64;
 
 
-typedef PCXX_EV32 EVENT;
-typedef PCXX_EV PCXX_EV_NATIVE;
+typedef TAU_EV32 EVENT;
+typedef TAU_EV TAU_EV_NATIVE;
 
 
 
@@ -104,12 +108,12 @@ typedef PCXX_EV PCXX_EV_NATIVE;
 
 
 
-typedef map< pair<int, int>, int, less < pair<int, int> > > NidTidMapT;
-typedef map< long int , Ttf_EventDescrT, less <long int> > EventIdMapT;
-typedef map< const char *, int, Ttf_ltstr > GroupIdMapT;
+typedef std::map< std::pair<int, int>, int, std::less < std::pair<int, int> > > NidTidMapT;
+typedef std::map< long int , Ttf_EventDescrT, std::less <long int> > EventIdMapT;
+typedef std::map< const char *, int, Ttf_ltstr > GroupIdMapT;
 
 /* for trace writing */
-typedef map< int, const char * > GroupNameMapT;
+typedef std::map< int, const char * > GroupNameMapT;
 
 typedef struct Ttf_file 
 {
@@ -132,6 +136,8 @@ typedef struct Ttf_file
   GroupNameMapT *groupNameMap;
   bool		initialized;
   double        lastTimestamp;
+
+  bool          forWriting;
 } Ttf_fileT;
 
 
@@ -147,6 +153,6 @@ extern "C" int isEventIDRegistered(Ttf_fileT *tFile, long int eid);
 
 /***************************************************************************
  * $RCSfile: TAU_tf_decl.h,v $   $Author: amorris $
- * $Revision: 1.5 $   $Date: 2005/10/11 19:16:27 $
- * TAU_VERSION_ID: $Id: TAU_tf_decl.h,v 1.5 2005/10/11 19:16:27 amorris Exp $ 
+ * $Revision: 1.7 $   $Date: 2009/02/19 22:30:03 $
+ * TAU_VERSION_ID: $Id: TAU_tf_decl.h,v 1.7 2009/02/19 22:30:03 amorris Exp $ 
  ***************************************************************************/
