@@ -16,6 +16,14 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 #include <Profile/TauIoWrap.h>
+#include <iowrap_metadata.h>
+
+// We should forward declare the TauEnv functions, but TAU_ASSERT is defined
+// in tau_internal.h, and variadic macros are not supported by pgcc.
+// So, for PGI, don't include this header.
+#ifndef __PGI
+#include <Profile/TauEnv.h>
+#endif
 
 #define TAU_MAX_SOCKET_LEN 1024
 #define TAU_READ TAU_IO
@@ -27,17 +35,27 @@ extern void Tau_iowrap_checkInit(void);
 typedef int64_t               off64_t;
 #endif /* __APPLE__ */
 
+/* FIX for others... */
+#ifndef off64_t
+typedef int64_t               off64_t;
+typedef struct stat               STRUCT_STAT64;
+#else
+typedef struct stat64               STRUCT_STAT64;
+#endif
+
 /*********************************************************************
  * fsync
  ********************************************************************/
+int __real_fsync( int fd);
 int __wrap_fsync( int fd)
 {
   int ret;
-  Tau_iowrap_checkInit();
+  
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fsync(fd); 
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fsync()", " ", TAU_IO);
   TAU_PROFILE_START(t);
   ret = __real_fsync(fd);
@@ -59,13 +77,14 @@ int __wrap_fsync( int fd)
 /*********************************************************************
  * open
  ********************************************************************/
+int __real_open(const char *pathname, int flags, ...);
 int __wrap_open(const char *pathname, int flags, ...)
 {
   int ret;
   int mode = 0;
   int mode_specified = 0; 
   TAU_VERBOSE("__wrap_open: pathname = %s\n", pathname); 
-  Tau_iowrap_checkInit();
+ 
   if (flags & O_CREAT) {
     va_list arg;
     va_start(arg, flags);
@@ -81,6 +100,10 @@ int __wrap_open(const char *pathname, int flags, ...)
       return __real_open(pathname, flags, mode); 
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "open()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -93,7 +116,7 @@ int __wrap_open(const char *pathname, int flags, ...)
   if (ret != -1) {
     Tau_iowrap_registerEvents(ret, pathname);
   }
-
+  
   if (TauEnv_get_track_io_params()) {
     TAU_REGISTER_EVENT(open_fd, "OPEN flags");
     TAU_REGISTER_EVENT(open_ret, "OPEN ret");
@@ -101,6 +124,9 @@ int __wrap_open(const char *pathname, int flags, ...)
     TAU_EVENT(open_ret, ret);
   }
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA(flags, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("Open call with pathname %s and flags %d: ret %d\n", pathname, flags, ret);
@@ -111,12 +137,13 @@ int __wrap_open(const char *pathname, int flags, ...)
 /*********************************************************************
  * open64
  ********************************************************************/
+int __real_open64(const char *pathname, int flags, ...);
 int __wrap_open64(const char *pathname, int flags, ...)
 {
   int ret;
   int mode = 0;
   int mode_specified = 0;
-  Tau_iowrap_checkInit();
+  
   if (flags & O_CREAT) {
     va_list arg;
     va_start(arg, flags);
@@ -132,6 +159,10 @@ int __wrap_open64(const char *pathname, int flags, ...)
       return __real_open64(pathname, flags, mode);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "open()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -152,6 +183,9 @@ int __wrap_open64(const char *pathname, int flags, ...)
     TAU_EVENT(open64_ret, ret);
   }
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA(flags, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("Open call with pathname %s and flags %d: ret %d\n", pathname, flags, ret);
@@ -162,15 +196,20 @@ int __wrap_open64(const char *pathname, int flags, ...)
 /*********************************************************************
  * creat
  ********************************************************************/
+int __real_creat(const char *pathname, mode_t mode);
 int __wrap_creat(const char *pathname, mode_t mode) 
 {
   int ret;
-  Tau_iowrap_checkInit();
+  
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_creat(pathname, mode);
   }
   Tau_global_incr_insideTAU();
+Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "creat()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -188,6 +227,9 @@ int __wrap_creat(const char *pathname, mode_t mode)
     TAU_EVENT(creat_ret, ret);
   }
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA(O_WRONLY, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("creat called on pathname %s with mode %d: ret %d\n", pathname, mode, ret);
@@ -198,15 +240,20 @@ int __wrap_creat(const char *pathname, mode_t mode)
 /*********************************************************************
  * creat64
  ********************************************************************/
+int __real_creat64(const char *pathname, mode_t mode);
 int __wrap_creat64(const char *pathname, mode_t mode)
 {
   int ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_creat64(pathname, mode);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "creat64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -224,6 +271,9 @@ int __wrap_creat64(const char *pathname, mode_t mode)
     TAU_EVENT(creat64_ret, ret);
   }
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA(O_WRONLY, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("creat called on pathname %s with mode %d: ret %d\n", pathname, mode, ret);
@@ -231,19 +281,23 @@ int __wrap_creat64(const char *pathname, mode_t mode)
   return ret;
 }
 
-FILE * __real_fopen(const char *pathname, const char * mode);
 /*********************************************************************
  * fopen
  ********************************************************************/
+FILE * __real_fopen(const char *pathname, const char * mode);
 FILE * __wrap_fopen(const char *pathname, const char * mode)
 {
   FILE * ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fopen(pathname, mode);
   }
   Tau_global_incr_insideTAU();
+    Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "fopen()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -261,6 +315,9 @@ FILE * __wrap_fopen(const char *pathname, const char * mode)
 */
 
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA_FOPEN(mode, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("fopen called with pathname=%s, mode=%d\n", pathname, mode);
@@ -268,19 +325,23 @@ FILE * __wrap_fopen(const char *pathname, const char * mode)
   return ret;
 }
 
-FILE*  __real_fopen64(const char *pathname, const char * mode);
 /*********************************************************************
  * fopen64
  ********************************************************************/
+FILE*  __real_fopen64(const char *pathname, const char * mode);
 FILE * __wrap_fopen64(const char *pathname, const char * mode)
 {
   FILE * ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fopen64(pathname, mode);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
+
+  /* get the name of the current timer, current thread and get a timestamp */
+  TAU_IOWRAPPER_METADATA_SETUP
 
   TAU_PROFILE_TIMER(t, "fopen64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -299,6 +360,9 @@ FILE * __wrap_fopen64(const char *pathname, const char * mode)
 */
 
   TAU_PROFILE_STOP(t);
+
+  TAU_IOWRAPPER_WRITE_FILE_METADATA_FOPEN(mode, pathname)
+
   Tau_global_decr_insideTAU();
 
   TAU_VERBOSE("fopen64 called with pathname=%s, mode=%d\n", pathname, mode);
@@ -313,16 +377,17 @@ FILE * __wrap_fopen64(const char *pathname, const char * mode)
 /*********************************************************************
  * pipe
  ********************************************************************/
+int __real_pipe(int filedes[2]);
 int __wrap_pipe(int filedes[2])
 {
   int ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_pipe(filedes);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "pipe()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -381,15 +446,16 @@ char * Tau_wrapper_get_socket_name(const struct sockaddr *sa, char *s, size_t le
 /*********************************************************************
  * socket
  ********************************************************************/
+int __real_socket(int domain, int type, int protocol);
 int __wrap_socket(int domain, int type, int protocol) {
   int ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_socket(domain, type, protocol);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "socket()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -420,15 +486,16 @@ int __wrap_socket(int domain, int type, int protocol) {
 /*********************************************************************
  * socketpair
  ********************************************************************/
+int __real_socketpair(int domain, int type, int protocol, int sv[2]);
 int __wrap_socketpair(int domain, int type, int protocol, int sv[2]) {
   int ret;
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_socketpair(domain, type, protocol, sv);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "socketpair()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -461,17 +528,18 @@ int __wrap_socketpair(int domain, int type, int protocol, int sv[2]) {
 /*********************************************************************
  * bind
  ********************************************************************/
+int __real_bind(int socket, const struct sockaddr *address, socklen_t address_len);
 int __wrap_bind(int socket, const struct sockaddr *address, socklen_t address_len) 
 {
   int ret;
-  Tau_iowrap_checkInit();
+
   char socketname[TAU_MAX_SOCKET_LEN];
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_bind(socket, address, address_len);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "bind()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -501,17 +569,18 @@ int __wrap_bind(int socket, const struct sockaddr *address, socklen_t address_le
 /*********************************************************************
  * connect
  ********************************************************************/
+int __real_connect(int socket, struct sockaddr *address, socklen_t address_len);
 int __wrap_connect(int socket, struct sockaddr *address, socklen_t address_len)
 {
   int ret;
-  Tau_iowrap_checkInit();
+
   char socketname[TAU_MAX_SOCKET_LEN];
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_connect(socket, address, address_len);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "connect()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -542,17 +611,18 @@ int __wrap_connect(int socket, struct sockaddr *address, socklen_t address_len)
 /*********************************************************************
  * accept
  ********************************************************************/
+int __real_accept(int socket, struct sockaddr *address, socklen_t* address_len);
 int __wrap_accept(int socket, struct sockaddr *address, socklen_t* address_len)
 {
   int ret;
-  Tau_iowrap_checkInit();
+
   char socketname[TAU_MAX_SOCKET_LEN];
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_accept(socket, address, address_len);
   }
   Tau_global_incr_insideTAU();
-
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "accept()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -580,6 +650,7 @@ int __wrap_accept(int socket, struct sockaddr *address, socklen_t* address_len)
 /*********************************************************************
  * fcntl
  ********************************************************************/
+int __real_fcntl(int fd, int cmd, ...) ;
 int __wrap_fcntl(int fd, int cmd, ...) 
 {
   va_list ap;
@@ -622,16 +693,18 @@ int __wrap_fcntl(int fd, int cmd, ...)
 /*********************************************************************
  * read
  ********************************************************************/
+size_t __real_read(int fd, void *buf, size_t nbytes);
 size_t __wrap_read(int fd, void *buf, size_t nbytes)
 {
   int ret;
   double currentRead = 0.0;
   struct timeval t1, t2; 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_read(fd, buf, nbytes);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "read()", " ", TAU_READ|TAU_IO);
   TAU_GET_IOWRAP_EVENT(re, READ_BW, fd);
   TAU_GET_IOWRAP_EVENT(bytesread, READ_BYTES, fd);
@@ -676,6 +749,7 @@ size_t __wrap_read(int fd, void *buf, size_t nbytes)
 /*********************************************************************
  * fread
  ********************************************************************/
+size_t __real_fread(void *ptr, size_t size, size_t nmemb, FILE *stream) ;
 size_t __wrap_fread(void *ptr, size_t size, size_t nmemb, FILE *stream) 
 {
   size_t ret;
@@ -684,11 +758,12 @@ size_t __wrap_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
   unsigned long long count; 
   int fd;
   fd = fileno(stream);
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fread(ptr, size, nmemb, stream);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fread()", " ", TAU_READ|TAU_IO);
   TAU_GET_IOWRAP_EVENT(re, READ_BW, fd);
   TAU_GET_IOWRAP_EVENT(bytesread, READ_BYTES, fd);
@@ -734,19 +809,21 @@ size_t __wrap_fread(void *ptr, size_t size, size_t nmemb, FILE *stream)
 /*********************************************************************
  * readv
  ********************************************************************/
+ssize_t __real_readv(int fd, const struct iovec *vec, int count) ;
 ssize_t __wrap_readv(int fd, const struct iovec *vec, int count) 
 {
   ssize_t ret;
   double currentRead = 0.0;
   struct timeval t1, t2;
 
-  Tau_iowrap_checkInit();
+
 
   if (Tau_iowrap_checkPassThrough()) {
     return __real_readv(fd, vec, count);
   }
 
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "readv()", " ", TAU_READ|TAU_IO);
   TAU_GET_IOWRAP_EVENT(re, READ_BW, fd);
   TAU_GET_IOWRAP_EVENT(bytesread, READ_BYTES, fd);
@@ -794,17 +871,19 @@ ssize_t __wrap_readv(int fd, const struct iovec *vec, int count)
 /*********************************************************************
  * write
  ********************************************************************/
+size_t __real_write(int fd, void *buf, size_t nbytes);
 size_t __wrap_write(int fd, void *buf, size_t nbytes)
 {
   int ret;
   double currentWrite = 0.0;
   struct timeval t1, t2; 
   double bw = 0.0; 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_write(fd, buf, nbytes);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "write()", " ", TAU_WRITE|TAU_IO);
   TAU_GET_IOWRAP_EVENT(wb, WRITE_BW, fd);
   TAU_GET_IOWRAP_EVENT(byteswritten, WRITE_BYTES, fd);
@@ -846,6 +925,7 @@ size_t __wrap_write(int fd, void *buf, size_t nbytes)
 /*********************************************************************
  * fwrite
  ********************************************************************/
+size_t __real_fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream);
 size_t __wrap_fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream)
 {
   size_t ret;
@@ -854,12 +934,13 @@ size_t __wrap_fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream)
   struct timeval t1, t2; 
   double bw = 0.0; 
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fwrite(ptr, size, nmemb, stream);
   }
 
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fwrite()", " ", TAU_WRITE|TAU_IO);
   TAU_GET_IOWRAP_EVENT(wb, WRITE_BW, fileno(stream));
   TAU_GET_IOWRAP_EVENT(byteswritten, WRITE_BYTES, fileno(stream));
@@ -903,17 +984,19 @@ size_t __wrap_fwrite( const void *ptr, size_t size, size_t nmemb, FILE *stream)
 /*********************************************************************
  * writev
  ********************************************************************/
+ssize_t __real_writev(int fd,  const struct iovec *vec, int count) ;
 ssize_t __wrap_writev(int fd,  const struct iovec *vec, int count) 
 {
   ssize_t ret;
   double currentWrite = 0.0;
   struct timeval t1, t2;
   double bw = 0.0;
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_writev(fd, vec, count);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "writev()", " ", TAU_WRITE|TAU_IO);
   TAU_GET_IOWRAP_EVENT(wb, WRITE_BW, fd);
   TAU_GET_IOWRAP_EVENT(byteswritten, WRITE_BYTES, fd);
@@ -955,17 +1038,19 @@ ssize_t __wrap_writev(int fd,  const struct iovec *vec, int count)
 /*********************************************************************
  * pwrite
  ********************************************************************/
+ssize_t __real_pwrite(int fd, void *buf, size_t nbytes, off_t offset);
 ssize_t __wrap_pwrite(int fd, void *buf, size_t nbytes, off_t offset)
 {
   ssize_t ret;
   double currentWrite = 0.0;
   struct timeval t1, t2; 
   double bw = 0.0; 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_pwrite(fd, buf, nbytes, offset);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "pwrite()", " ", TAU_WRITE|TAU_IO);
   TAU_GET_IOWRAP_EVENT(wb, WRITE_BW, fd);
   TAU_GET_IOWRAP_EVENT(byteswritten, WRITE_BYTES, fd);
@@ -1009,17 +1094,19 @@ ssize_t __wrap_pwrite(int fd, void *buf, size_t nbytes, off_t offset)
 /*********************************************************************
  * pwrite64
  ********************************************************************/
+ssize_t __real_pwrite64(int fd, void *buf, size_t nbytes, off64_t offset);
 ssize_t __wrap_pwrite64(int fd, void *buf, size_t nbytes, off64_t offset)
 {
   ssize_t ret;
   double currentWrite = 0.0;
   struct timeval t1, t2;
   double bw = 0.0;
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_pwrite64(fd, buf, nbytes, offset);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "pwrite64()", " ", TAU_WRITE|TAU_IO);
   TAU_GET_IOWRAP_EVENT(wb, WRITE_BW, fd);
   TAU_GET_IOWRAP_EVENT(byteswritten, WRITE_BYTES, fd);
@@ -1063,16 +1150,18 @@ ssize_t __wrap_pwrite64(int fd, void *buf, size_t nbytes, off64_t offset)
 /*********************************************************************
  * pread
  ********************************************************************/
+ssize_t __real_pread(int fd, void *buf, size_t nbytes, off_t offset);
 ssize_t __wrap_pread(int fd, void *buf, size_t nbytes, off_t offset)
 {
   ssize_t ret;
   double currentRead = 0.0;
   struct timeval t1, t2; 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_pread(fd, buf, nbytes, offset);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "pread()", " ", TAU_READ|TAU_IO);
   TAU_GET_IOWRAP_EVENT(re, READ_BW, fd);
   TAU_GET_IOWRAP_EVENT(bytesread, READ_BYTES, fd);
@@ -1119,16 +1208,18 @@ ssize_t __wrap_pread(int fd, void *buf, size_t nbytes, off_t offset)
 /*********************************************************************
  * pread64
  ********************************************************************/
+ssize_t __real_pread64(int fd, void *buf, size_t nbytes, off64_t offset);
 ssize_t __wrap_pread64(int fd, void *buf, size_t nbytes, off64_t offset)
 {
   ssize_t ret;
   double currentRead = 0.0;
   struct timeval t1, t2; 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_pread64(fd, buf, nbytes, offset);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "pread64()", " ", TAU_READ|TAU_IO);
   TAU_GET_IOWRAP_EVENT(re, READ_BW, fd);
   TAU_GET_IOWRAP_EVENT(bytesread, READ_BYTES, fd);
@@ -1175,14 +1266,16 @@ ssize_t __wrap_pread64(int fd, void *buf, size_t nbytes, off64_t offset)
 /*********************************************************************
  * close
  ********************************************************************/
+size_t __real_close(int fd);
 size_t __wrap_close(int fd)
 {
   int ret;
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_close(fd);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "close()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1206,16 +1299,18 @@ size_t __wrap_close(int fd)
 /*********************************************************************
  * fclose
  ********************************************************************/
+int __real_fclose(FILE *fp);
 int __wrap_fclose(FILE *fp)
 {
   int ret;
   int fd; 
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fclose(fp);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   fd = fileno(fp);
   TAU_PROFILE_TIMER(t, "fclose()", " ", TAU_IO);
   TAU_PROFILE_START(t);
@@ -1241,15 +1336,17 @@ int __wrap_fclose(FILE *fp)
 /*********************************************************************
  * fdatasync
  ********************************************************************/
+int __real_fdatasync(int fd);
 int __wrap_fdatasync(int fd)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fdatasync(fd);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fdatasync()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1274,15 +1371,17 @@ int __wrap_fdatasync(int fd)
 /*********************************************************************
  * lseek
  ********************************************************************/
+off_t __real_lseek(int fd, off_t offset, int whence) ;
 off_t __wrap_lseek(int fd, off_t offset, int whence) 
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_lseek(fd, offset, whence);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "lseek()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1308,15 +1407,17 @@ off_t __wrap_lseek(int fd, off_t offset, int whence)
 /*********************************************************************
  * lseek64
  ********************************************************************/
+off64_t __real_lseek64(int fd, off64_t offset, int whence);
 off64_t __wrap_lseek64(int fd, off64_t offset, int whence)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_lseek64(fd, offset, whence);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "lseek64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1342,15 +1443,17 @@ off64_t __wrap_lseek64(int fd, off64_t offset, int whence)
 /*********************************************************************
  * fseek
  ********************************************************************/
+off_t __real_fseek(FILE *stream, long offset, int whence) ;
 off_t __wrap_fseek(FILE *stream, long offset, int whence) 
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fseek(stream, offset, whence);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fseek()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1376,15 +1479,17 @@ off_t __wrap_fseek(FILE *stream, long offset, int whence)
 /*********************************************************************
  * stat
  ********************************************************************/
+int __real_stat(const char *path, struct stat *buf) ;
 int __wrap_stat(const char *path, struct stat *buf) 
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_stat(path, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "stat()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1407,15 +1512,17 @@ int __wrap_stat(const char *path, struct stat *buf)
 /*********************************************************************
  * stat64
  ********************************************************************/
+int __real_stat64(const char *path, struct stat *buf);
 int __wrap_stat64(const char *path, struct stat *buf)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_stat64(path, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "stat64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1438,15 +1545,17 @@ int __wrap_stat64(const char *path, struct stat *buf)
 /*********************************************************************
  * fstat
  ********************************************************************/
-int __wrap_fstat(int filedes, struct stat64 *buf)
+int __real_fstat(int filedes, STRUCT_STAT64 *buf);
+int __wrap_fstat(int filedes, STRUCT_STAT64 *buf)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fstat(filedes, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fstat()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1470,15 +1579,17 @@ int __wrap_fstat(int filedes, struct stat64 *buf)
 /*********************************************************************
  * fstat64
  ********************************************************************/
-int __wrap_fstat64(int filedes, struct stat64 *buf)
+int __real_fstat64(int filedes, STRUCT_STAT64 *buf);
+int __wrap_fstat64(int filedes, STRUCT_STAT64 *buf)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_fstat64(filedes, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "fstat64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1502,15 +1613,17 @@ int __wrap_fstat64(int filedes, struct stat64 *buf)
 /*********************************************************************
  * lstat
  ********************************************************************/
+int __real_lstat(const char *path, struct stat *buf);
 int __wrap_lstat(const char *path, struct stat *buf)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_lstat(path, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "lstat()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1532,15 +1645,17 @@ int __wrap_lstat(const char *path, struct stat *buf)
 /*********************************************************************
  * lstat64
  ********************************************************************/
-int __wrap_lstat64(const char *path, struct stat64 *buf)
+int __real_lstat64(const char *path, STRUCT_STAT64 *buf);
+int __wrap_lstat64(const char *path, STRUCT_STAT64 *buf)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_lstat64(path, buf);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "lstat64()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1562,15 +1677,17 @@ int __wrap_lstat64(const char *path, struct stat64 *buf)
 /*********************************************************************
  * dup
  ********************************************************************/
+int __real_dup(int filedes);
 int __wrap_dup(int filedes)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_dup(filedes);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "dup()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1595,15 +1712,17 @@ int __wrap_dup(int filedes)
 /*********************************************************************
  * dup2
  ********************************************************************/
+int __real_dup2(int filedes1, int filedes2);
 int __wrap_dup2(int filedes1, int filedes2)
 {
   int ret;
 
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_dup2(filedes1, filedes2);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "dup2()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
@@ -1631,15 +1750,18 @@ int __wrap_dup2(int filedes1, int filedes2)
 /*********************************************************************
  * select
  ********************************************************************/
+int __real_select(int nfds, fd_set *readfds, fd_set *writefds, 
+  fd_set *exceptfds, const struct timeval *timeout);
 int __wrap_select(int nfds, fd_set *readfds, fd_set *writefds, 
   fd_set *exceptfds, const struct timeval *timeout)
 {
   int ret;
-  Tau_iowrap_checkInit();
+
   if (Tau_iowrap_checkPassThrough()) {
     return __real_select(nfds, readfds, writefds, exceptfds, timeout);
   }
   Tau_global_incr_insideTAU();
+  Tau_iowrap_checkInit();
   TAU_PROFILE_TIMER(t, "select()", " ", TAU_IO);
   TAU_PROFILE_START(t);
 
