@@ -12,10 +12,13 @@ extern void Tau_profile_exit_all_threads(void);
 void taupreload_init() {
   Tau_init_initializeTAU();
   Tau_create_top_level_timer_if_necessary();
-  TAU_PROFILE_SET_NODE(0);
+  int tmp = TAU_PROFILE_GET_NODE();
+  if (tmp == -1) {
+    TAU_PROFILE_SET_NODE(0);
+  }
 }
 
 void taupreload_fini() {
-  Tau_destructor_trigger();
   Tau_profile_exit_all_threads();
+  Tau_destructor_trigger();
 }
