@@ -2,20 +2,18 @@ package edu.uoregon.tau.paraprof;
 
 import java.util.*;
 
-import edu.uoregon.tau.perfdmf.*;
-
 /**
  * ApplicationManager
  * This controls the adding of applications to the system.
  *  
  * 
- * <P>CVS $Id: ApplicationManager.java,v 1.13 2005/09/26 21:12:01 amorris Exp $</P>
+ * <P>CVS $Id: ApplicationManager.java,v 1.16 2007/05/29 20:27:20 amorris Exp $</P>
  * @author	Robert Bell
- * @version	$Revision: 1.13 $
+ * @version	$Revision: 1.16 $
  */
 public class ApplicationManager extends Observable {
 
-    private List applications = new ArrayList();
+    private List<ParaProfApplication> applications = new ArrayList<ParaProfApplication>();
 
     public ParaProfApplication addApplication() {
         ParaProfApplication application = new ParaProfApplication();
@@ -28,7 +26,7 @@ public class ApplicationManager extends Observable {
         applications.remove(application);
     }
 
-    public List getApplications() {
+    public List<ParaProfApplication> getApplications() {
         return applications;
     }
 
@@ -40,7 +38,7 @@ public class ApplicationManager extends Observable {
     }
 
     public ParaProfApplication getApplication(int applicationID) {
-        return (ParaProfApplication) applications.get(applicationID);
+        return applications.get(applicationID);
     }
 
     public ParaProfExperiment getExperiment(int applicationID, int experimentID) {
@@ -52,8 +50,8 @@ public class ApplicationManager extends Observable {
     }
 
     public boolean isApplicationPresent(String name) {
-        for (Iterator it = applications.iterator(); it.hasNext();) {
-            ParaProfApplication application = (ParaProfApplication) it.next();
+        for (Iterator<ParaProfApplication> it = applications.iterator(); it.hasNext();) {
+            ParaProfApplication application = it.next();
             if (name.compareTo(application.getName()) == 0)
                 return true;
         }
