@@ -22,6 +22,7 @@ import javax.swing.JTextField;
 
 import edu.uoregon.tau.perfdmf.DataSource;
 import edu.uoregon.tau.perfdmf.CubeDataSource;
+import edu.uoregon.tau.perfdmf.View;
 
 
 /**
@@ -55,122 +56,133 @@ public class LoadTrialWindow extends JFrame implements ActionListener {
 
     private JCheckBox monitorTrialCheckBox = new JCheckBox("Monitor Trial");
 
+	private View view;
+
+	private ParaProfView pview;
+
     public LoadTrialWindow(ParaProfManagerWindow paraProfManager, ParaProfApplication application, ParaProfExperiment experiment,
             boolean newApplication, boolean newExperiment) {
-        this.paraProfManagerWindow = paraProfManager;
+    	this(paraProfManager, !experiment.dBExperiment());
         this.application = application;
         this.experiment = experiment;
         this.newApplication = newApplication;
         this.newExperiment = newExperiment;
-
-        if (lastDirectory == null) {
-            lastDirectory = System.getProperty("user.dir");
-            dirLocationField.setText(lastDirectory);
-        }
-
-        //Window Stuff.
-        int windowWidth = 400;
-        int windowHeight = 200;
-
-        //Grab paraProfManager position and size.
-        Point parentPosition = paraProfManager.getLocationOnScreen();
-        Dimension parentSize = paraProfManager.getSize();
-        int parentWidth = parentSize.width;
-        int parentHeight = parentSize.height;
-
-        //Set the window to come up in the center of the screen.
-        int xPosition = (parentWidth - windowWidth) / 2;
-        int yPosition = (parentHeight - windowHeight) / 2;
-
-        xPosition = (int) parentPosition.getX() + xPosition;
-        yPosition = (int) parentPosition.getY() + yPosition;
-
-        this.setLocation(xPosition, yPosition);
-        setSize(new java.awt.Dimension(windowWidth, windowHeight));
-        setTitle("TAU: ParaProf: Load Trial");
-        ParaProfUtils.setFrameIcon(this);
-
-        addWindowListener(new java.awt.event.WindowAdapter() {
-            public void windowClosing(java.awt.event.WindowEvent evt) {
-                thisWindowClosing(evt);
-            }
-        });
-
-        selectButton = new JButton("Select Directory");
-        selectButton.addActionListener(this);
-
-        trialTypes = new JComboBox(DataSource.formatTypeStrings);
-        trialTypes.setMaximumRowCount(DataSource.formatTypeStrings.length);
-        trialTypes.addActionListener(this);
-        // must be after action listener
-        trialTypes.setSelectedIndex(defaultIndex);
-
-        Container contentPane = getContentPane();
-        GridBagLayout gbl = new GridBagLayout();
-        contentPane.setLayout(gbl);
-        GridBagConstraints gbc = new GridBagConstraints();
-        gbc.insets = new Insets(5, 5, 5, 5);
-
-        gbc.fill = GridBagConstraints.NONE;
-        gbc.anchor = GridBagConstraints.EAST;
-        gbc.weightx = 0;
-        gbc.weighty = 0;
-        addCompItem(new JLabel("Trial Type"), gbc, 0, 0, 1, 1);
-
-        gbc.fill = GridBagConstraints.BOTH;
-        gbc.anchor = GridBagConstraints.WEST;
-        gbc.weightx = 100;
-        gbc.weighty = 0;
-        addCompItem(trialTypes, gbc, 1, 0, 1, 1);
-
-        gbc.fill = GridBagConstraints.NONE;
-        gbc.anchor = GridBagConstraints.EAST;
-        gbc.weightx = 0;
-        gbc.weighty = 0;
-        addCompItem(selectButton, gbc, 0, 1, 1, 1);
-
-        gbc.fill = GridBagConstraints.BOTH;
-        gbc.anchor = GridBagConstraints.WEST;
-        gbc.weightx = 100;
-        gbc.weighty = 0;
-        addCompItem(dirLocationField, gbc, 1, 1, 2, 1);
-
-        gbc.fill = GridBagConstraints.NONE;
-        gbc.anchor = GridBagConstraints.EAST;
-        gbc.weightx = 0;
-        gbc.weighty = 0;
-
-        if (!experiment.dBExperiment()) {
-            monitorTrialCheckBox.addActionListener(this);
-            gbc.fill = GridBagConstraints.BOTH;
-            gbc.anchor = GridBagConstraints.CENTER;
-
-            addCompItem(monitorTrialCheckBox, gbc, 1, 2, 1, 1);
-
-            gbc.fill = GridBagConstraints.NONE;
-            gbc.anchor = GridBagConstraints.EAST;
-
-            JButton jButton = new JButton("Cancel");
-            jButton.addActionListener(this);
-            addCompItem(jButton, gbc, 0, 3, 1, 1);
-
-            jButton = new JButton("Ok");
-            jButton.addActionListener(this);
-            addCompItem(jButton, gbc, 2, 3, 1, 1);
-        } else {
-
-            JButton jButton = new JButton("Cancel");
-            jButton.addActionListener(this);
-            addCompItem(jButton, gbc, 0, 2, 1, 1);
-
-            jButton = new JButton("Ok");
-            jButton.addActionListener(this);
-            addCompItem(jButton, gbc, 2, 2, 1, 1);
-
-        }
-
     }
-    private static final String locationSeparator=", ";
+
+	public LoadTrialWindow(ParaProfManagerWindow paraProfManager, boolean addMonitor) {
+		this.paraProfManagerWindow = paraProfManager;
+
+		if (lastDirectory == null) {
+			lastDirectory = System.getProperty("user.dir");
+			dirLocationField.setText(lastDirectory);
+		}
+
+		// Window Stuff.
+		int windowWidth = 400;
+		int windowHeight = 200;
+
+		// Grab paraProfManager position and size.
+		Point parentPosition = paraProfManager.getLocationOnScreen();
+		Dimension parentSize = paraProfManager.getSize();
+		int parentWidth = parentSize.width;
+		int parentHeight = parentSize.height;
+
+		// Set the window to come up in the center of the screen.
+		int xPosition = (parentWidth - windowWidth) / 2;
+		int yPosition = (parentHeight - windowHeight) / 2;
+
+		xPosition = (int) parentPosition.getX() + xPosition;
+		yPosition = (int) parentPosition.getY() + yPosition;
+
+		this.setLocation(xPosition, yPosition);
+		setSize(new java.awt.Dimension(windowWidth, windowHeight));
+		setTitle("TAU: ParaProf: Load Trial");
+		ParaProfUtils.setFrameIcon(this);
+
+		addWindowListener(new java.awt.event.WindowAdapter() {
+			public void windowClosing(java.awt.event.WindowEvent evt) {
+				thisWindowClosing(evt);
+			}
+		});
+
+		selectButton = new JButton("Select Directory");
+		selectButton.addActionListener(this);
+
+		trialTypes = new JComboBox(DataSource.formatTypeStrings);
+		trialTypes.setMaximumRowCount(DataSource.formatTypeStrings.length);
+		trialTypes.addActionListener(this);
+		// must be after action listener
+		trialTypes.setSelectedIndex(defaultIndex);
+
+		Container contentPane = getContentPane();
+		GridBagLayout gbl = new GridBagLayout();
+		contentPane.setLayout(gbl);
+		GridBagConstraints gbc = new GridBagConstraints();
+		gbc.insets = new Insets(5, 5, 5, 5);
+
+		gbc.fill = GridBagConstraints.NONE;
+		gbc.anchor = GridBagConstraints.EAST;
+		gbc.weightx = 0;
+		gbc.weighty = 0;
+		addCompItem(new JLabel("Trial Type"), gbc, 0, 0, 1, 1);
+
+		gbc.fill = GridBagConstraints.BOTH;
+		gbc.anchor = GridBagConstraints.WEST;
+		gbc.weightx = 100;
+		gbc.weighty = 0;
+		addCompItem(trialTypes, gbc, 1, 0, 1, 1);
+
+		gbc.fill = GridBagConstraints.NONE;
+		gbc.anchor = GridBagConstraints.EAST;
+		gbc.weightx = 0;
+		gbc.weighty = 0;
+		addCompItem(selectButton, gbc, 0, 1, 1, 1);
+
+		gbc.fill = GridBagConstraints.BOTH;
+		gbc.anchor = GridBagConstraints.WEST;
+		gbc.weightx = 100;
+		gbc.weighty = 0;
+		addCompItem(dirLocationField, gbc, 1, 1, 2, 1);
+
+		gbc.fill = GridBagConstraints.NONE;
+		gbc.anchor = GridBagConstraints.EAST;
+		gbc.weightx = 0;
+		gbc.weighty = 0;
+
+		if (addMonitor) {
+			monitorTrialCheckBox.addActionListener(this);
+			gbc.fill = GridBagConstraints.BOTH;
+			gbc.anchor = GridBagConstraints.CENTER;
+
+			addCompItem(monitorTrialCheckBox, gbc, 1, 2, 1, 1);
+
+			gbc.fill = GridBagConstraints.NONE;
+			gbc.anchor = GridBagConstraints.EAST;
+
+			JButton jButton = new JButton("Cancel");
+			jButton.addActionListener(this);
+			addCompItem(jButton, gbc, 0, 3, 1, 1);
+
+			jButton = new JButton("Ok");
+			jButton.addActionListener(this);
+			addCompItem(jButton, gbc, 2, 3, 1, 1);
+		} else {
+
+			JButton jButton = new JButton("Cancel");
+			jButton.addActionListener(this);
+			addCompItem(jButton, gbc, 0, 2, 1, 1);
+
+			jButton = new JButton("Ok");
+			jButton.addActionListener(this);
+			addCompItem(jButton, gbc, 2, 2, 1, 1);
+
+		}
+	}
+	public LoadTrialWindow(ParaProfManagerWindow paraProfManagerWindow, View view) {
+		this(paraProfManagerWindow,true);
+		this.pview = new ParaProfView(view);
+	}
+	private static final String locationSeparator=", ";
     public void actionPerformed(ActionEvent evt) {
         try {
             String arg = evt.getActionCommand();
@@ -272,9 +284,14 @@ public class LoadTrialWindow extends JFrame implements ActionListener {
                             //return;
                         }
                     }
+                    if(pview != null){
+                    	 paraProfManagerWindow.addTrial(pview, files, trialTypes.getSelectedIndex(), false,
+                                 monitorTrialCheckBox.isSelected(), null);
+                    }else{
                     paraProfManagerWindow.addTrial(application, experiment, files, trialTypes.getSelectedIndex(), false,
                             monitorTrialCheckBox.isSelected());
                     }
+                    }
                 } else {
                     if (selectedFiles == null) {
                         selectedFiles = new File[1];
@@ -284,8 +301,14 @@ public class LoadTrialWindow extends JFrame implements ActionListener {
                             return;
                         }
                     }
-                    paraProfManagerWindow.addTrial(application, experiment, selectedFiles, trialTypes.getSelectedIndex(), false,
-                            monitorTrialCheckBox.isSelected());
+                    if(pview != null){
+                    paraProfManagerWindow.addTrial(pview, selectedFiles, trialTypes.getSelectedIndex(), false,
+                            monitorTrialCheckBox.isSelected(), null);
+                    }else{
+                        paraProfManagerWindow.addTrial(application, experiment, selectedFiles, trialTypes.getSelectedIndex(), false,
+                                monitorTrialCheckBox.isSelected());
+                    }
+                    
                 }
                 closeThisWindow();
             } else if (arg.equals("comboBoxChanged")) {
