@@ -27,6 +27,8 @@ import java.util.Vector;
 import javax.swing.tree.DefaultMutableTreeNode;
 import javax.swing.tree.TreePath;
 
+import edu.uoregon.tau.common.MetaDataMap;
+import edu.uoregon.tau.common.MetaDataMap.MetaDataKey;
 import edu.uoregon.tau.paraprof.script.ParaProfScript;
 import edu.uoregon.tau.paraprof.script.ParaProfTrialScript;
 import edu.uoregon.tau.paraprof.util.FileMonitor;
@@ -50,6 +52,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
 
     private DatabaseAPI dbAPI;
     private ParaProfExperiment experiment = null;
+    private ParaProfView view = null;
     private DefaultMutableTreeNode defaultMutableTreeNode = null;
     private TreePath treePath = null;
     private boolean dBTrial = false;
@@ -98,7 +101,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public Iterator<Function> getFunctions() {
-        return getDataSource().getFunctions();
+        return getDataSource().getFunctionIterator();
     }
 
     public Thread getMeanThread() {
@@ -132,6 +135,16 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     public void setID(int id) {
         trial.setID(id);
     }
+    
+    public void rename(String newName){
+    	if(dBTrial){
+			DatabaseAPI databaseAPI = getDatabaseAPI();
+			if (databaseAPI != null) {
+				this.trial.rename(databaseAPI.getDb(), newName);
+			}
+		}
+		trial.setName(newName);
+    }
 
     public String getName() {
         return trial.getName();
@@ -145,6 +158,14 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         this.experiment = experiment;
     }
 
+    public void setView(ParaProfView view) {
+        this.view = view;
+    }
+
+    public ParaProfView getView() {
+        return view;
+    }
+
     public ParaProfExperiment getExperiment() {
         return experiment;
     }
@@ -192,6 +213,8 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     public String getIDString() {
         if (experiment != null) {
             return (experiment.getIDString()) + ":" + (trial.getID());
+        } else if (view != null) {
+            return (view.getIDString()) + ":" + (trial.getID());
         } else {
             return ":" + (trial.getID());
         }
@@ -345,14 +368,20 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public void setMeanData(int metricID) {
+    	// save the old derived provided state
+    	boolean tmpVal = trial.getDataSource().isDerivedProvided();
+    	// set to false, so we compute new statistics for the new metric
+    	trial.getDataSource().setDerivedProvided(false);
         trial.getDataSource().generateStatistics(metricID, metricID);
+        // restore the old value
+    	trial.getDataSource().setDerivedProvided(tmpVal);
     }
 
     // return a vector of only those functions that are currently "displayed" (i.e. group masks, etc)
     public List<Function> getDisplayedFunctions() {
         List<Function> displayedFunctions = new ArrayList<Function>();
 
-        for (Iterator<Function> it = this.getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = this.getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             if (this.displayFunction(function)) {
                 displayedFunctions.add(function);
@@ -370,7 +399,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public void showGroup(Group group) {
-        for (Iterator<Function> it = getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             if (function.isGroupMember(group)) {
                 functionMask[function.getID()] = true;
@@ -382,7 +411,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public void hideGroup(Group group) {
-        for (Iterator<Function> it = getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             if (function.isGroupMember(group)) {
                 functionMask[function.getID()] = false;
@@ -394,7 +423,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public void showGroupOnly(Group group) {
-        for (Iterator<Function> it = getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             if (function.isGroupMember(group)) {
                 functionMask[function.getID()] = true;
@@ -408,7 +437,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public void showAllExcept(Group group) {
-        for (Iterator<Function> it = getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             if (function.isGroupMember(group)) {
                 functionMask[function.getID()] = false;
@@ -458,7 +487,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
             }
         }
 
-        for (Iterator<Function> it = getDataSource().getFunctions(); it.hasNext();) {
+        for (Iterator<Function> it = getDataSource().getFunctionIterator(); it.hasNext();) {
             Function function = it.next();
             String name = function.getName();
             if (caseSensitive) {
@@ -738,27 +767,40 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public Database getDatabase() {
-        if (experiment == null) {
-            return null;
+        if (experiment != null) {
+        	return experiment.getDatabase();
+        } else if (view != null) {
+            return view.getDatabase();
         }
-        return experiment.getDatabase();
+        return null;
     }
 
     public List<Thread> getThreads() {
         return getDataSource().getAllThreads();
     }
 
+    private static final String TAU_APPLICATION_NAME="TAU_APPLICATION_NAME";
     public List<String> getThreadNames() {
         List<String> threadNames = new ArrayList<String>();
 
         for (Iterator<Thread> it = getDataSource().getAllThreads().iterator(); it.hasNext();) {
             Thread thread = it.next();
+            
+            
+            
+
+            	MetaDataMap mdm = thread.getMetaData();
+            	String tan="";
+            	if(ParaProf.preferences.getAppNameLabels()==1&&mdm.containsKey(TAU_APPLICATION_NAME)){
+            		tan = ":"+mdm.get(TAU_APPLICATION_NAME);
+            	}
+            
             if (getDataSource().getExecutionType() == DataSource.EXEC_TYPE_MPI) {
-                threadNames.add(Integer.toString(thread.getNodeID()));
+                threadNames.add(Integer.toString(thread.getNodeID())+tan);
             } else if (getDataSource().getExecutionType() == DataSource.EXEC_TYPE_HYBRID) {
-                threadNames.add(thread.getNodeID() + ":" + thread.getThreadID());
+                threadNames.add(thread.getNodeID() + ":" + thread.getThreadID()+tan);
             } else {
-                threadNames.add(thread.getNodeID() + ":" + thread.getContextID() + ":" + thread.getThreadID());
+                threadNames.add(thread.getNodeID() + ":" + thread.getContextID() + ":" + thread.getThreadID()+tan);
             }
         }
         return threadNames;
@@ -801,14 +843,67 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
 	public Vector<String> getTopologyArray() {
-		Set<String> keys = getDataSource().getMetaData().keySet();
+		Set<MetaDataKey> keys = getDataSource().getMetaData().keySet();
 		Vector<String> topos = new Vector<String>();
-		for(Iterator<String> it = keys.iterator(); it.hasNext();){
-			String key = it.next();
-			if(key.contains(" isTorus")||key.contains(" Period")){
+		boolean foundTopo=false;
+		for(Iterator<MetaDataKey> it = keys.iterator(); it.hasNext();){
+			String key = it.next().name;
+			if(key.contains(" isTorus")||key.contains(" Period")||key.contains(" Dimension")){
 				topos.add(key.split(" ")[0]);
+				foundTopo=true;
+			}
+			
+		}
+		
+		if(!foundTopo){
+			Thread base =getDataSource().getThread(0, 0, 0);
+			if(base==null){
+				/*
+				 * If we don't have 0,0,0 get a list of all the threads. If any thread has more metadata than the first we assume that's the one with the core metadata. If any has less than the first we assume the first has the core metadata.
+				 */
+				List<Thread> threads = getDataSource().getThreads();
+				
+				Iterator<Thread> threadIt = threads.iterator();
+				if(!threadIt.hasNext()){
+					return topos;
+				}
+				Thread currentThread=threadIt.next();
+				base = currentThread;
+				int num=currentThread.getMetaData().keySet().size();
+				while(threadIt.hasNext()){
+					
+					currentThread=threadIt.next();
+					
+					int tmp =currentThread.getMetaData().keySet().size();
+					if(tmp>num){
+						base=currentThread;
+						break;
+					}else if(tmp<num){
+						
+						break;
+					}
+				}
+			}
+			if(base==null){
+				return topos;
+			}
+			boolean foundCray=false;
+			keys = base.getMetaData().keySet();
+			for(Iterator<MetaDataKey> it = keys.iterator(); it.hasNext();){
+				String key = it.next().name;
+				if((key.startsWith("Cray")&&key.contains("Nodename")))
+				{
+					topos.add(key.split(" ")[0]);
+					foundTopo=true;
+				}
+				if(!foundCray&&key.startsWith("CRAY_PMI")){
+					topos.add("CRAY_PMI");
+					foundTopo=true;
+					foundCray=true;
+				}
 			}
 		}
+		
 //		if(topos.size()==0){
 //			return new String[]{null};
 //		}
