@@ -32,7 +32,10 @@ import javax.swing.JSplitPane;
 
 import com.graphbuilder.math.VarMap;
 
+import edu.uoregon.tau.common.MetaDataMap.MetaDataKey;
+import edu.uoregon.tau.common.MetaDataMap.MetaDataValue;
 import edu.uoregon.tau.paraprof.ThreeDeeGeneralPlotUtils.CoordMap;
+import edu.uoregon.tau.paraprof.ThreeDeeGeneralPlotUtils.CrayTopology;
 import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.UserEventValueType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
@@ -362,7 +365,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 		float color = Float.NaN;
 
-		if (!settings.getAtomic(3)) {
+		if (settings.getDataType(3)==0) {
 			Function topoFunction = settings.getTopoFunction(3);
 			ValueType topoValueType = settings.getTopoValueType(3);
 			Metric topoMetricID = settings.getTopoMetric(3);
@@ -378,7 +381,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 					return 0;
 				}
 			}
-		} else {
+		} else if(settings.getDataType(3)==1){
 			UserEvent ue = settings.getTopoAtomic(3);
 			if (ue != null) {
 				UserEventProfile uep = thread.getUserEventProfile(ue);
@@ -390,6 +393,30 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 								ppTrial.getSelectedSnapshot());
 				}
 			}
+		}else if(settings.getDataType(3)==2){
+			MetaDataKey metaKey=settings.getTopoMetadata(3);//.toString();
+			if(metaKey!=null){
+				MetaDataValue mdVal=thread.getMetaData().get(metaKey);
+				
+				if(mdVal==null){
+					mdVal=ppTrial.getDataSource().getMetaData().get(metaKey);
+				}
+				if(mdVal==null){
+					//System.out.println(thread.getNodeID()+" null");	
+					return 0;
+				}
+				//System.out.println(mdVal.toString());		
+				
+				float tmp=Float.NaN;
+				String metaVal=mdVal.toString();
+				try{
+				tmp = Float.parseFloat(metaVal);
+				}catch(NumberFormatException e){
+					return tmp;
+				}
+				color=tmp;
+			}
+			//TODO: Handle non-numerical metadata?
 		}
 
 		return color;
@@ -450,6 +477,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 	int[] tsizes = { 0, 0, 0 };
 	boolean restrict = false;
 	Thread[][][] threadTopo=null;
+	private int[][] coords = null;
 
 	private float[][] defaultTopology(int numThreads) {
 		float[][] values = new float[numThreads][4];
@@ -517,14 +545,124 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 		} 
 		
-		else if (prefix.equals("Map")) {
-			String fileLoc = settings.getTopoMapFile();
-			if (fileLoc == null)
-				return null;
-			CoordMap map = ThreeDeeGeneralPlotUtils.parseMapFile(fileLoc);
+		else if (prefix.equals("Map")||prefix.equals("CRAY_PMI")) {
+			
+			CoordMap map = null;
+			
+			if(prefix.equals("CRAY_PMI"))
+			{
+					CrayTopology[] ctopo = new CrayTopology[ppTrial.getDataSource().getNumThreads()];
+					int i=0;
+					
+					//This metadata only shows up where it changes from the previous rank, so we need to check for it and use the previous value if it is absent.
+					
+					int coreid=0;
+					String nodename="";
+					int nodeid=0;
+					int x=0;
+					int y=0; 
+					int z=0;
+					
+					for (Iterator<Thread> it = ppTrial.getDataSource().getAllThreads()
+							.iterator(); it.hasNext();) {
+						Thread thread = it.next();
 
+						int mpirank = thread.getNodeID();
+						int threadID = thread.getThreadID();
+						//System.out.println("mpirank:"+mpirank+" threadid:"+threadID);
+						
+						String coreidS = thread.getMetaData().get("CRAY_CORE_ID");
+						if(coreidS!=null)
+						{
+							coreid = Integer.parseInt(coreidS);
+						}
+						else{
+							coreid=0;
+						}
+						if(thread.getMetaData().containsKey("CRAY_NODENAME"));
+						{
+							String tempnodename = thread.getMetaData().get("CRAY_NODENAME");
+							if(tempnodename!=null){
+								nodename=tempnodename;
+							}
+							else {
+								//In some cases, where these values are the same for all threads, thread metadata won't return them. They need to be acquired from top level metadata.
+								tempnodename = ppTrial.getDataSource().getMetaData().get("CRAY_NODENAME");
+								if(tempnodename!=null){
+									nodename=tempnodename;
+								}
+							}
+						}
+						String nodeids = thread.getMetaData().get("CRAY_PMI_NODEID");
+						if(nodeids==null) {
+							nodeids = ppTrial.getDataSource().getMetaData().get("CRAY_PMI_NODEID");
+						}
+						if(nodeids!=null)
+						{
+							nodeid = Integer.parseInt(nodeids);
+						}
+							
+						String xs = thread.getMetaData().get("CRAY_PMI_X");
+						if(xs==null) {
+							xs = ppTrial.getDataSource().getMetaData().get("CRAY_PMI_X");
+						}
+						if(xs!=null){
+							x = Integer.parseInt(xs);
+						}
+						
+						String ys = thread.getMetaData().get("CRAY_PMI_Y");
+						if(ys==null) {
+							ys = ppTrial.getDataSource().getMetaData().get("CRAY_PMI_Y");
+						}
+						if(ys!=null){
+							y = Integer.parseInt(ys);
+						}
+						
+						String zs = thread.getMetaData().get("CRAY_PMI_Z");
+						if(zs==null) {
+							zs = ppTrial.getDataSource().getMetaData().get("CRAY_PMI_Z");
+						}
+						if(zs!=null){
+							z = Integer.parseInt(zs);
+						}
+						
+						
+						//System.out.println(mpirank);
+						CrayTopology ct = new CrayTopology(mpirank,nodename,nodeid,x,y,z,coreid);
+						//System.out.println(ct.toString());
+						ctopo[i]=ct;
+						i++;
+					}
+					map = ThreeDeeGeneralPlotUtils.processCrayCoordinates(ctopo);
+					
+				
+			}
+			else{
+			String fileLoc = settings.getTopoMapFile();
+//			if(fileLoc.toLowerCase().endsWith(".json")){
+//				map=ThreeDeeGeneralPlotUtils.parseJsonMapFile(fileLoc);
+//			}
+//			else
+			{
+			if(settings.checkNewMapFile()||coords==null)
+			{
+				
+				if (fileLoc == null)
+					return null;
+				System.out.println("Reading map file");
+				coords = ThreeDeeGeneralPlotUtils.parseMapFile(fileLoc);
+			}
+			
+			if(coords==null){
+				return null;
+			}
+			System.out.println("Calculating map coords");
+			map = ThreeDeeGeneralPlotUtils.calculateCoreCoordinates(coords, settings.getCustomTopoCoreAxes());
+			}
+			}
 			for (int i = 0; i < 3; i++) {
-				maxScatterValues[i] = tsizes[i] = map.max[i];
+				maxScatterValues[i] = (int)map.max[i];
+				tsizes[i]=(int)map.max[i]-(int)map.min[i];
 				minScatterValues[i] = map.min[i];
 			}
 			// scatterPlot.setSize(tsizes[0],tsizes[1], tsizes[2]);
@@ -684,7 +822,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 				float[] topoVals = { 0, 0, 0, 0 };
 				for (int i = 0; i < 4; i++) {
-					if (!settings.getAtomic(i)) {
+					if (settings.getDataType(i)==0) {
 						Function f = settings.getTopoFunction(i);
 						if (f != null) {
 							FunctionProfile fp = thread.getFunctionProfile(f);
@@ -695,7 +833,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 										ppTrial.getSelectedSnapshot());
 							}
 						}
-					} else {
+					} else if(settings.getDataType(i)==1) {
 						UserEvent ue = settings.getTopoAtomic(i);
 						if (ue != null) {
 							UserEventProfile uep = thread
@@ -709,6 +847,20 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 							}
 						}
 					}
+					else if(settings.getDataType(i)==2) {
+						MetaDataKey metaKey=settings.getTopoMetadata(i);
+						if(metaKey!=null){
+							String metaVal=thread.getMetaData().get(metaKey).toString();
+							float tmp=Float.NaN;
+							try{
+							tmp = Float.parseFloat(metaVal);
+							}catch(NumberFormatException e){
+								
+							}
+							topoVals[i]=tmp;
+						}
+						//TODO: Metadata support
+					}
 				}
 				// float[] ueVal={0,0,0,0};
 				// for(int i=0;i<4;i++){
@@ -729,14 +881,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 				vm = ThreeDeeGeneralPlotUtils.getEvaluation(
 						rankIndex,
 						numThreads,
-						thread.getNodeID(),
-						thread.getContextID(),
-						thread.getThreadID(),
-						ppTrial.getDataSource().getNumberOfNodes(),
-						ppTrial.getDataSource().getNumberOfContexts(
-								thread.getNodeID()),
-						ppTrial.getDataSource().getNumberOfThreads(
-								thread.getNodeID(), thread.getContextID()),
+						thread,
+						ppTrial,
 						topoVals, varMins, varMaxs, varMeans, settings
 								.getCustomTopoAxes(), expressions);
 
@@ -808,7 +954,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 		} 
 
-			else {
+
+			else{
 				
 			String coord_key = prefix + " Coords";
 			String size_key = prefix + " Size";
@@ -821,7 +968,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 			}
 			
 			String tsize = ppTrial.getDataSource().getMetaData().get(size_key);
-			
+			if(tsize==null)
+			{
+				System.out.println("Topology sizes not defined");
+				return values;
+			}
+				
 			
 			int[] tempsizes = ThreeDeeGeneralPlotUtils.parseTuple(tsize);
 			
@@ -829,6 +981,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 			if(tempsizes.length>3){
 				
 				calculateTSize(tempsizes,offset);
+				if(tsizes[0]<0)
+					return values;
 			}
 			else{
 				tsizes = tempsizes;
@@ -844,6 +998,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 
 			int threadIndex = 0;
+			int coreCoordinate=0;//Used to generate final core coordinate mapping;
 			for (Iterator<Thread> it = ppTrial.getDataSource().getAllThreads()
 					.iterator(); it.hasNext();) {
 				Thread thread = it.next();
@@ -855,26 +1010,66 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 				
 				
 				int[] coords; 
+				
 				if(isBGQ){
 					coords = ThreeDeeGeneralPlotUtils.parseMPIProcName(coord);
+				}
+				else{
+					coords = ThreeDeeGeneralPlotUtils.parseTuple(coord);
+				}
+				if(coords.length<tempsizes.length){
+					int[] tempcoords=new int[tempsizes.length];
+					int i=0;
+					for(i=0;i<coords.length;i++){
+						tempcoords[i]=coords[i];
+					}
+					if(coreCoordinate==tempsizes[tempsizes.length-1])
+					{
+						coreCoordinate=0;
+					}
+					tempcoords[i]=coreCoordinate;
+					coreCoordinate++;
+					coords=tempcoords;
+				}
+				
+				
+				if(tempsizes.length==6&&isBGQ){
+					
 					values[threadIndex][0] = coords[4]*tempsizes[5]+coords[5]+offset*coords[4];
 					values[threadIndex][1] = coords[0]*tempsizes[1]+coords[1]+offset*coords[0];
 					values[threadIndex][2] = coords[2]*tempsizes[3]+coords[3]+offset*coords[2];
 					
 					for (int i = 0; i < 3; i++) {
-					maxScatterValues[i] = Math.max(maxScatterValues[i], values[threadIndex][i]);
-				}
-					
-					
-				}
-				
-				else{
-					coords = ThreeDeeGeneralPlotUtils.parseTuple(coord);
-
-					for (int i = 0; i < coords.length; i++) {
-						values[threadIndex][i] = coords[i];
+						maxScatterValues[i] = Math.max(maxScatterValues[i], values[threadIndex][i]);
 					}
 					
+					
+				}
+//				else if(tempsizes.length==5){
+//					values[threadIndex][0]=coords[4]*offset;
+//					values[threadIndex][1]=coords[0]*tempsizes[1]+coords[1]+offset*coords[0];;
+//					values[threadIndex][2]=coords[2]*tempsizes[3]+coords[3]+offset*coords[2];
+//				}else if (tempsizes.length==4){
+//					values[threadIndex][0]=coords[3]*offset;
+//					values[threadIndex][1]=coords[0]*tempsizes[1]+coords[1]+offset*coords[0];
+//					values[threadIndex][2]=coords[2]*offset;
+//				}
+				
+				else 
+					if(tempsizes.length<=3){
+					
+
+					for (int i = 0; i < coords.length; i++) {
+						
+						
+							values[threadIndex][i] = coords[i];
+						
+						
+					}
+					
+				}
+				else{
+					//System.out.println("Dimension "+coords.length+" Unsupported");
 				}
 				
 				int dex0=(int)values[threadIndex][0];
@@ -893,13 +1088,18 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 				}
 
 				threadIndex++;
-			}
+			}//Values populator loop
 			}
 
 		return values;
 	}
 	
 	private void calculateTSize(int[] tempSize, int offset){
+		
+//		for(int i=0;i<tempSize.length;i++){
+//			tempSize[i]=tempSize[i]+1;
+//		}
+		
 		if(tempSize.length==6){
 			
 			tsizes[0]=tempSize[4]*tempSize[5]+tempSize[4]*offset;
@@ -908,9 +1108,25 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 			
 		}
 		else
+//		else if(tempSize.length==5){
+//			tsizes[0]=tempSize[4]*offset;
+//			tsizes[1]=tempSize[0]*tempSize[1]+tempSize[0]*offset;
+//			tsizes[2]=tempSize[2]*tempSize[3]+tempSize[2]*offset;
+//		}
+//		else if(tempSize.length==4){
+//			tsizes[0]=tempSize[3]*offset;
+//			tsizes[1]=tempSize[0]*tempSize[1]+tempSize[0]*offset;
+//			tsizes[2]=tempSize[2]*offset;
+//		}
+//		else 
+			if(tempSize.length<=3)
 			for(int i=0;i<3;i++){
 				tsizes[i]=tempSize[i+3];
 			}
+		else{
+			System.out.println("Dimension "+tempSize.length+" Unsupported");
+			tsizes[0]=-1;
+		}
 	}
 
 	// String[] expressions =
@@ -2036,9 +2252,9 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 
 		int units = this.units;
 		
-		boolean atomic = settings.getAtomic(3);
+		int dataType = settings.getDataType(3);
 		
-		if(!atomic){
+		if(dataType==0){
 		
 		ParaProfMetric ppMetric = (ParaProfMetric) settings.getTopoMetric(3);
 		if (!ppMetric.isTimeMetric()
@@ -2050,9 +2266,14 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 				ppMetric.isTimeDenominator()).trim()
 				+ getUnitsString(units, settings.getTopoValueType(3), ppMetric);
 		}
-		else{
+		else if(dataType==1){
 			return val+" counts";
 		}
+		else if(dataType==2){
+			//TODO: metadata support
+			return val+" counts";
+		}
+		return val+" counts";
 	}
 
 	private String getUnitsString(int units, ValueType valueType,
@@ -2193,7 +2414,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 		String low = "none";
 		String high = "none";
 		
-		if (!settings.getAtomic(3)) {//If this is an interval value
+		if (settings.getDataType(3)==0) {//If this is an interval value
 			ParaProfMetric ppMetric = (ParaProfMetric) settings
 					.getTopoMetric(3);
 			int units = settings.getTopoValueType(3).getUnits(this.units,
@@ -2224,7 +2445,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 						ppMetric.isTimeDenominator()).trim();
 				
 			} 
-		} else {//if this is an atomic value
+		} else if(settings.getDataType(3)==1){//if this is an atomic value
 			UserEventValueType colorUEVT = settings.getTopoUserEventValueType(3);
 			UserEvent colorAtom = settings.getTopoAtomic(3);
 			if(colorAtom!=null){
@@ -2238,6 +2459,20 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 				high = Float.toString(maxScatterValues[3]);
 			}
 		}
+		else if(settings.getDataType(3)==2){			
+			MetaDataKey metaKey=settings.getTopoMetadata(3);
+		if(metaKey!=null){
+			String toDisplay = metaKey.toString();
+			if (toDisplay.length() > 30) {
+				toDisplay = toDisplay.substring(0, 30) + "...";
+			}
+			colorName = toDisplay ;//+ "\n("
+					//+ colorUEVT.toString()+ ")";
+			low = Float.toString(minScatterValues[3]);
+			high = Float.toString(maxScatterValues[3]);
+		}
+			//TODO: Metadata Support
+		}
 
 		colorScale.setStrings(low,	high, colorName);
 
@@ -2394,7 +2629,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
 	 */
 	public void keyPressed(KeyEvent e) {
-		// TODO Auto-generated method stub
 
 	}
 
@@ -2404,8 +2638,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
 	 */
 	public void keyReleased(KeyEvent e) {
-		// TODO Auto-generated method stub
-
 	}
 
 	/*
@@ -2414,7 +2646,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
 	 */
 	public void keyTyped(KeyEvent e) {
-		// TODO Auto-generated method stub
 		try {
 			// zoom in and out on +/-
 			if (e.getKeyChar() == '+') {
@@ -2444,7 +2675,6 @@ public class ThreeDeeWindow extends JFrame implements ActionListener,
 	}
 
 	public void createNewCanvas() {
-		// TODO Auto-generated method stub
 		visCanvas = new VisCanvas(visRenderer);
 
 		visCanvas.addKeyListener(this);
