@@ -1,10 +1,18 @@
 package edu.uoregon.tau.paraprof.treetable;
 
-import java.awt.*;
+import java.awt.Color;
+import java.awt.Component;
+import java.awt.Graphics;
+import java.awt.GridBagConstraints;
+import java.awt.GridBagLayout;
+import java.awt.Insets;
 import java.text.DecimalFormat;
 import java.text.NumberFormat;
 
-import javax.swing.*;
+import javax.swing.Icon;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JTable;
 import javax.swing.table.DefaultTableCellRenderer;
 import javax.swing.table.TableCellRenderer;
 
@@ -18,9 +26,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
  * 
  * This is starting to get messy and should be rethought
  *
- * <P>CVS $Id: TreeTableColumn.java,v 1.5 2009/06/26 00:43:50 amorris Exp $</P>
+ * <P>CVS $Id: TreeTableColumn.java,v 1.6 2010/03/16 02:16:28 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.5 $
+ * @version $Revision: 1.6 $
  */
 abstract public class TreeTableColumn {
     protected TreeTableWindow window;
@@ -35,9 +43,35 @@ abstract public class TreeTableColumn {
         this.window = window;
     }
 
-    protected Object adjustForUnits(double value, int metricID, boolean forSorting) {
-        if (forSorting)
+    
+    protected Object getValueString(double value) {
+        if (window.getDecimals() == -2) {
+            return Double.toString(value); 
+        } else if (window.getDecimals() == -1) {
             return new Double(value);
+        } else {
+            DecimalFormat format = null;
+            if (window.getDecimals() == 0) {
+                format = new DecimalFormat("#,###");
+            } else if (window.getDecimals() == 1) {
+                format = new DecimalFormat("#,##0.0");
+            } else if (window.getDecimals() == 2) {
+                format = new DecimalFormat("#,##0.00");
+            } else if (window.getDecimals() == 3) {
+                format = new DecimalFormat("#,##0.000");
+            } else if (window.getDecimals() == 4) {
+                format = new DecimalFormat("#,##0.0000");
+            }
+            String retval = format.format(value);
+            return retval;
+        }
+        //System.out.println("String for " + value + " is " + retval);
+    }
+    
+    protected Object adjustForUnits(double value, int metricID, boolean forSorting) {
+        if (forSorting) {
+            return new Double(value);
+        }
 
         if (window.getPPTrial().getMetric(metricID).isTimeMetric()) {
 
@@ -61,7 +95,9 @@ abstract public class TreeTableColumn {
                 }
             }
         }
-        return new Double(value);
+        
+        //return new Double(value);
+        return getValueString(value);
     }
 
     public static class ColorIcon implements Icon {
@@ -93,7 +129,11 @@ abstract public class TreeTableColumn {
     }
 
     static class ParaProfCellRenderer extends DefaultTableCellRenderer {
-        private NumberFormat formatter;
+        /**
+		 * 
+		 */
+		private static final long serialVersionUID = 5459926803089862887L;
+		private NumberFormat formatter;
         private TreeTableWindow window;
 
         public ParaProfCellRenderer(TreeTableWindow window) {
@@ -115,7 +155,7 @@ abstract public class TreeTableColumn {
             }
 
             column = table.convertColumnIndexToModel(column) - 1;
-            TreeTableColumn ttColumn = (TreeTableColumn) window.getColumns().get(column);
+            TreeTableColumn ttColumn = window.getColumns().get(column);
             Object result = ttColumn.getValueFor((TreeTableNode) value, false);
 
             setHorizontalAlignment(JLabel.RIGHT);
@@ -145,7 +185,11 @@ abstract public class TreeTableColumn {
     }
 
     static class RegularMetricCellRenderer extends JPanel implements TableCellRenderer {
-        private NumberFormat formatter;
+        /**
+		 * 
+		 */
+		private static final long serialVersionUID = -4911710251628231833L;
+		private NumberFormat formatter;
         private TreeTableWindow window;
 
         private JLabel iconLabel = new JLabel();
