@@ -12,11 +12,14 @@ import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Map;
 import java.util.Map.Entry;
 import java.util.Set;
 import java.util.zip.GZIPOutputStream;
 
+import edu.uoregon.tau.common.MetaDataMap;
+import edu.uoregon.tau.common.MetaDataMap.MetaDataKey;
+import edu.uoregon.tau.common.MetaDataMap.MetaDataValue;
+
 public class DataSourceExport {
 
 	private static int findGroupID(Group groups[], Group group) {
@@ -120,15 +123,17 @@ public class DataSourceExport {
 		BufferedOutputStream bw = new BufferedOutputStream(gzip);
 		DataOutputStream p = new DataOutputStream(bw);
 
-		Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
-		int numFunctions = 0;
+		//Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
+		int numFunctions = dataSource.getFunctions().size();//0;
 
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
-			Function function = it.next();
-			if (!function.isGroupMember(derived)) {
-				numFunctions++;
-			}
-		}
+		
+		
+//		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
+//			Function function = it.next();
+//			if (!function.isGroupMember(derived)) {
+//				numFunctions++;
+//			}
+//		}
 
 		int numMetrics = dataSource.getNumberOfMetrics();
 		int numUserEvents = dataSource.getNumUserEvents();
@@ -156,14 +161,14 @@ public class DataSourceExport {
 		if (dataSource.getMetaData() != null) {
 			// write out the trial meta-data, this data is normalized across all
 			// threads (i.e. it applies to all threads)
-			Map<String, String> metaData = dataSource.getMetaData();
+			MetaDataMap metaData = dataSource.getMetaData();
 			headerData.writeInt(metaData.size());
-			for (Iterator<String> it2 = metaData.keySet().iterator(); it2
+			for (Iterator<MetaDataKey> it2 = metaData.keySet().iterator(); it2
 					.hasNext();) {
-				String name = it2.next();
-				String value = metaData.get(name);
-				headerData.writeUTF(name);
-				headerData.writeUTF(value);
+				MetaDataKey key = it2.next();
+				MetaDataValue value = metaData.get(key);
+				headerData.writeUTF(key.name);
+				headerData.writeUTF(value.value.toString());
 			}
 		} else {
 			headerData.writeInt(0);
@@ -173,17 +178,17 @@ public class DataSourceExport {
 		for (Iterator<Thread> it = dataSource.getAllThreads().iterator(); it
 				.hasNext();) {
 			Thread thread = it.next();
-			Map<String, String> metaData = thread.getMetaData();
+			MetaDataMap metaData = thread.getMetaData();
 			headerData.writeInt(thread.getNodeID());
 			headerData.writeInt(thread.getContextID());
 			headerData.writeInt(thread.getThreadID());
 			headerData.writeInt(metaData.size());
-			for (Iterator<String> it2 = metaData.keySet().iterator(); it2
+			for (Iterator<MetaDataKey> it2 = metaData.keySet().iterator(); it2
 					.hasNext();) {
-				String name = it2.next();
-				String value = metaData.get(name);
-				headerData.writeUTF(name);
-				headerData.writeUTF(value);
+				MetaDataKey key = it2.next();
+				MetaDataValue value = metaData.get(key);
+				headerData.writeUTF(key.name);
+				headerData.writeUTF(value.value.toString());
 			}
 		}
 		headerData.close();
@@ -213,9 +218,9 @@ public class DataSourceExport {
 		Function functions[] = new Function[numFunctions];
 		idx = 0;
 		p.writeInt(numFunctions);
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
+		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
 			Function function = it.next();
-			if (!function.isGroupMember(derived)) {
+			//if (!function.isGroupMember(derived)) {
 
 				functions[idx++] = function;
 				p.writeUTF(function.getName());
@@ -230,14 +235,14 @@ public class DataSourceExport {
 						p.writeInt(findGroupID(groups, group));
 					}
 				}
-			}
+			//}
 		}
 
 		// write out user event names
 		UserEvent userEvents[] = new UserEvent[numUserEvents];
 		idx = 0;
 		p.writeInt(numUserEvents);
-		for (Iterator<UserEvent> it = dataSource.getUserEvents(); it.hasNext();) {
+		for (Iterator<UserEvent> it = dataSource.getUserEventIterator(); it.hasNext();) {
 			UserEvent userEvent = it.next();
 			userEvents[idx++] = userEvent;
 			p.writeUTF(userEvent.getName());
@@ -324,12 +329,12 @@ public class DataSourceExport {
 	}
 
 	private static void writeXMLSnippet(BufferedWriter bw,
-			Map<String, String> metaData) throws IOException {
-		for (Iterator<String> it2 = metaData.keySet().iterator(); it2.hasNext();) {
-			String name = it2.next();
-			String value = metaData.get(name);
-			bw.write("<attribute><name>" + xmlFixUp(name) + "</name><value>"
-					+ xmlFixUp(value) + "</value></attribute>");
+			MetaDataMap metaDataMap) throws IOException {
+		for (Iterator<MetaDataKey> it2 = metaDataMap.keySet().iterator(); it2.hasNext();) {
+			MetaDataKey key = it2.next();
+			MetaDataValue value = metaDataMap.get(key);
+			bw.write("<attribute><name>" + xmlFixUp(key.name) + "</name><value>"
+					+ xmlFixUp(value.value.toString()) + "</value></attribute>");
 		}
 	}
 
@@ -459,22 +464,22 @@ public class DataSourceExport {
 
 	public static void writeMetaDataSummary(DataSource dataSource)
 			throws IOException {
-		Map<String, String> m = dataSource.getMetaData();
+		MetaDataMap m = dataSource.getMetaData();
 		System.out.println("Metadata:");
 
-		Iterator<String> iks = m.keySet().iterator();
+		Iterator<MetaDataKey> iks = m.keySet().iterator();
 		int maxKeyLength = 0;
 		while (iks.hasNext()) {
-			String k = iks.next();
+			String k = iks.next().name;
 			maxKeyLength = Math.max(maxKeyLength, k.length());
 		}
 
-		Set<Entry<String, String>> ms = m.entrySet();
-		Iterator<Entry<String, String>> ims = ms.iterator();
+		Set<Entry<MetaDataKey, MetaDataValue>> ms = m.entrySet();
+		Iterator<Entry<MetaDataKey, MetaDataValue>> ims = ms.iterator();
 		while (ims.hasNext()) {
-			Entry<String, String> ems = ims.next();
-			System.out.format("%-" + maxKeyLength + "s%s", ems.getKey(), ": "
-					+ ems.getValue());
+			Entry<MetaDataKey, MetaDataValue> ems = ims.next();
+			System.out.format("%-" + maxKeyLength + "s%s", ems.getKey().name, ": "
+					+ ems.getValue().value.toString());
 			System.out.println();
 			// System.out.println(ems.getKey() + " = " + ems.getValue());
 		}
@@ -489,16 +494,16 @@ public class DataSourceExport {
 
 		int idx = 0;
 
-		Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
+		//Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
 
-		int numFunctions = 0;
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
-			Function function = it.next();
-			if (function.isGroupMember(derived)) {
-				continue;
-			}
-			numFunctions++;
-		}
+		int numFunctions = dataSource.getFunctions().size();//0;
+//		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
+//			Function function = it.next();
+//			if (function.isGroupMember(derived)) {
+//				continue;
+//			}
+//			numFunctions++;
+//		}
 
 		// write out group names
 		Group groups[] = new Group[numGroups];
@@ -513,10 +518,10 @@ public class DataSourceExport {
 		idx = 0;
 
 		// write out function names
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
+		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
 			Function function = it.next();
 
-			if (!function.isGroupMember(derived)) {
+			//if (!function.isGroupMember(derived)) {
 				functions[idx] = function;
 
 				List<Group> thisGroups = function.getGroups();
@@ -539,13 +544,13 @@ public class DataSourceExport {
 					groupStrings[idx] = groupStrings[idx].trim();
 				}
 				idx++;
-			}
+			//}
 		}
 
 		UserEvent userEvents[] = new UserEvent[numUserEvents];
 		idx = 0;
 		// collect user event names
-		for (Iterator<UserEvent> it = dataSource.getUserEvents(); it.hasNext();) {
+		for (Iterator<UserEvent> it = dataSource.getUserEventIterator(); it.hasNext();) {
 			UserEvent userEvent = it.next();
 			userEvents[idx++] = userEvent;
 		}
@@ -557,11 +562,13 @@ public class DataSourceExport {
 			for (int i = 0; i < numMetrics; i++) {
 				String name = "MULTI__"
 						+ createSafeMetricName(dataSource.getMetricName(i));
-				boolean success = (new File(name).mkdir());
+				boolean success = (new File(directory + File.separator + name)
+						.mkdir());
 				if (!success) {
 					System.err.print("Failed to create directory: " + name);
 				} else {
-					writeMetric(new File(directory + "/" + name), dataSource,
+					writeMetric(new File(directory + File.separator + name),
+							dataSource,
 							i, functions, groupStrings, userEvents, threads);
 				}
 			}
@@ -578,16 +585,16 @@ public class DataSourceExport {
 
 		int idx = 0;
 
-		Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
+		//Group derived = dataSource.getGroup("TAU_CALLPATH_DERIVED");
 
-		int numFunctions = 0;
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
-			Function function = it.next();
-			if (function.isGroupMember(derived)) {
-				continue;
-			}
-			numFunctions++;
-		}
+		int numFunctions = dataSource.getFunctions().size();//0;
+//		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
+//			Function function = it.next();
+//			if (function.isGroupMember(derived)) {
+//				continue;
+//			}
+//			numFunctions++;
+//		}
 
 		// // write out group names
 		// Group groups[] = new Group[numGroups];
@@ -602,10 +609,10 @@ public class DataSourceExport {
 		idx = 0;
 
 		// write out function names
-		for (Iterator<Function> it = dataSource.getFunctions(); it.hasNext();) {
+		for (Iterator<Function> it = dataSource.getFunctionIterator(); it.hasNext();) {
 			Function function = it.next();
 
-			if (!function.isGroupMember(derived)) {
+			//if (!function.isGroupMember(derived)) {
 				functions[idx] = function;
 
 				List<Group> thisGroups = function.getGroups();
@@ -628,13 +635,13 @@ public class DataSourceExport {
 					groupStrings[idx] = groupStrings[idx].trim();
 				}
 				idx++;
-			}
+			//}
 		}
 
 		UserEvent userEvents[] = new UserEvent[numUserEvents];
 		idx = 0;
 		// collect user event names
-		for (Iterator<UserEvent> it = dataSource.getUserEvents(); it.hasNext();) {
+		for (Iterator<UserEvent> it = dataSource.getUserEventIterator(); it.hasNext();) {
 			UserEvent userEvent = it.next();
 			userEvents[idx++] = userEvent;
 		}
@@ -711,13 +718,13 @@ public class DataSourceExport {
 			metType = metType + " (Counts)";
 
 		int metListLen = 15;
-		String headerFormat = "%-8s%-8s%-8s%-8s%-8s%-8s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s";
+		String headerFormat = "%-12s%-12s%-12s%-12s%-12s%-12s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s%-16s";
 		if (suppress) {
 			metListLen = 9;
-			System.out.format("%-24s%-48s%-48s%-8s", "   Calls",
+			System.out.format("%-36s%-48s%-48s%-8s", "   Calls",
 					"   Inclusive " + metType, "   Bytes Transferred",
 					"   Name");
-			headerFormat = "%-8s%-8s%-8s%-16s%-16s%-16s%-16s%-16s%-16s";
+			headerFormat = "%-12s%-12s%-12s%-16s%-16s%-16s%-16s%-16s%-16s";
 		} else {
 			System.out.format("%-24s%-24s%-48s%-48s%-48s%-8s", "   Calls",
 					"   Child Calls", "   Exclusive " + metType,
@@ -805,12 +812,12 @@ public class DataSourceExport {
 
 		if (suppress) {
 			System.out.format(
-					"%8.3f%8.3f%8.3f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f",
+					"%12.3f%12.3f%12.3f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f",
 					values[0], values[1], values[2], values[9], values[10],
 					values[11], values[12], values[13], values[14]);
 		} else {
 			System.out
-					.format("%8.3f%8.3f%8.3f%8.3f%8.3f%8.3f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f",
+					.format("%12.3f%12.3f%12.3f%12.3f%12.3f%12.3f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f%16.5f",
 							values[0], values[1], values[2], values[3],
 							values[4], values[5], values[6], values[7],
 							values[8], values[9], values[10], values[11],
@@ -820,5 +827,6 @@ public class DataSourceExport {
 
 		System.out.println();
 	}
+	
 
 }
