@@ -1,6 +1,7 @@
 package edu.uoregon.tau.perfdmf;
 
 import java.awt.Color;
+import java.io.File;
 import java.io.Serializable;
 import java.util.ArrayList;
 import java.util.Iterator;
@@ -18,7 +19,7 @@ import java.util.List;
 /**
  * @author amorris
  *
- * TODO ...
+ *
  */
 public class Function implements Serializable, Comparable<Function> {
 
@@ -27,6 +28,7 @@ public class Function implements Serializable, Comparable<Function> {
 	 */
 	private static final long serialVersionUID = 362090098221172924L;
 	private String name;
+	private String short_name;
     private String reversedName;
     private int id = -1;
     private List<Group> groups = new ArrayList<Group>();
@@ -34,6 +36,7 @@ public class Function implements Serializable, Comparable<Function> {
     private Function actualPhase;
     private Function parentPhase;
     //private boolean phaseSet = false;
+    private int databaseID = 0;
 
     boolean callpathFunction = false;
     boolean callpathFunctionSet = false;
@@ -44,6 +47,8 @@ public class Function implements Serializable, Comparable<Function> {
     private FunctionProfile totalProfile;
     private FunctionProfile minProfile;
     private FunctionProfile maxProfile;
+    private FunctionProfile meanAllProfile;
+    private FunctionProfile stddevAllProfile;
 
     // color settings
     private boolean colorFlag = false;
@@ -52,16 +57,22 @@ public class Function implements Serializable, Comparable<Function> {
 
     // source code link
     private SourceRegion sourceLink;
+    
+    
 
     public Function(String name, int id, int numMetrics) {
         this.name = name;
-        this.id = id;
+        this.setID(id);
     }
 
     public int getID() {
         return id;
     }
 
+    public int getDatabaseID() {
+        return databaseID;
+    }
+
     public String getName() {
         return name;
     }
@@ -71,6 +82,15 @@ public class Function implements Serializable, Comparable<Function> {
         this.reversedName = null;
     }
 
+    public String getCallpathNodeName() {
+    	if (!this.isCallPathFunction()) {
+    		return this.name;
+    	}
+        String s = name;
+        int location = s.lastIndexOf("=>");
+        return s.substring(location + 3);
+    }
+    
     /**
      * Retrieve the reversed (callpath) name
      * If the function's name is "A => B => C", this will return "C <= B <= A"
@@ -113,7 +133,20 @@ public class Function implements Serializable, Comparable<Function> {
     public SourceRegion getSourceLink() {
         if (this.sourceLink == null) {
             if (isCallPathFunction(name)){//(isGroupMember("TAU_CALLPATH_DERIVED")) {
-                this.sourceLink = getSourceLink(UtilFncs.getLeftSide(name));
+            	String tmpName = name;
+            	SourceRegion tmpReg = getSourceLink(UtilFncs.getRightMost(tmpName));
+            	int start = tmpReg.getStartLine();
+            	while(start<=-1){
+            		tmpReg = getSourceLink(UtilFncs.getRightMost(tmpName));
+                    start=tmpReg.getStartLine();
+                    String tmpTmpName=tmpName;
+                    tmpName=UtilFncs.getAllButRightMost(tmpName);
+                    if(tmpName.equals(tmpTmpName)){
+                    	break;
+                    }
+            	}
+                
+                this.sourceLink=tmpReg;
             } else {
                 this.sourceLink = getSourceLink(this.name);
             }
@@ -123,6 +156,9 @@ public class Function implements Serializable, Comparable<Function> {
 
     public static SourceRegion getSourceLink(String name) {
         SourceRegion sourceLink = new SourceRegion();
+        
+        // initialize to the full name for now
+        sourceLink.setShortName(name);
 
         if (name.indexOf("OpenMP location:") != -1) { // opari instrumentation points
             // parse source location with format:
@@ -133,7 +169,14 @@ public class Function implements Serializable, Comparable<Function> {
             int comma = name.indexOf(",");
             int right = name.indexOf(">");
 
-            sourceLink.setFilename(name.substring(fileIndex + 5, left).trim());
+            sourceLink.setShortName(name.substring(0,fileIndex));
+            String filename = name.substring(fileIndex + 5, left).trim();
+			File sourceFile = new File(filename);
+			if (!sourceFile.exists() && filename.contains("/"))
+            {
+            	filename = filename.substring(filename.lastIndexOf("/") + 1);
+            }
+            sourceLink.setFilename(filename);
             sourceLink.setStartLine(Integer.parseInt(name.substring(left + 1, comma).trim()));
             sourceLink.setEndLine(Integer.parseInt(name.substring(comma + 1, right).trim()));
         }
@@ -167,6 +210,41 @@ public class Function implements Serializable, Comparable<Function> {
             name = name.substring(name.lastIndexOf("=>") + 2);
         }
 
+        if(name.startsWith("!$omp")){
+        	int at = name.lastIndexOf('@');
+        	int colon = name.lastIndexOf(':');
+        	if(at<0||colon<0||colon<at){
+        		return sourceLink;
+        	}
+        	String filename = name.substring(at+1, colon);
+        	String lineString = name.substring(colon+1);
+        	sourceLink.setFilename(filename);
+        	int linenumber = Integer.parseInt(lineString);
+        	sourceLink.setStartLine(linenumber);
+        	sourceLink.setEndLine(linenumber);
+        	return sourceLink;
+        }
+        
+        // check for parameter based profile name
+    	List<Parameter> parameters = new ArrayList<Parameter>();
+        int parameterStart = name.indexOf("[ <");
+        while (parameterStart != -1) {
+        	
+        	// find the end of the name
+        	int parameterEnd = name.indexOf("> = <", parameterStart);
+        	String pname = name.substring(parameterStart+3, parameterEnd).trim();
+        	// find the start of the value
+        	parameterStart = name.indexOf("<", parameterEnd);
+        	// and the end of the value
+        	parameterEnd = name.indexOf(">", parameterStart);
+        	String pval = name.substring(parameterStart+1, parameterEnd).trim();
+        	// create a parameter and put it in the list.
+        	Parameter param = new Parameter(pname, pval, -1);
+        	parameters.add(param);
+            parameterStart = name.indexOf(", <", parameterEnd);
+        }        
+    	sourceLink.setParameters(parameters);
+
         int filenameStart = name.indexOf("[{");
         if (filenameStart == -1) {
             return sourceLink;
@@ -176,8 +254,10 @@ public class Function implements Serializable, Comparable<Function> {
             // quit, it's not valid
             return sourceLink;
         }
-
+        
         int openbracket1 = name.indexOf("{", filenameEnd + 1);
+        sourceLink.setShortName(name.substring(0,filenameStart).trim());
+        
         int comma1 = name.indexOf(",", filenameEnd + 1);
         int closebracket1 = name.indexOf("}", filenameEnd + 1);
         int dash = name.indexOf("-", closebracket1 + 1);
@@ -186,13 +266,24 @@ public class Function implements Serializable, Comparable<Function> {
         int closebracket2 = name.indexOf("}", closebracket1 + 1);
 
         String filename = name.substring(filenameStart + 2, filenameEnd);
-        filename = filename.substring(filename.lastIndexOf("/") + 1);
+        File sourceFile=new File(filename);
+        if(!sourceFile.exists())
+        {
+        	filename = filename.substring(filename.lastIndexOf("/") + 1);
+        }
 
         sourceLink.setFilename(filename);
-
+        
         if (openbracket1 == -1) {
             return sourceLink;
         }
+        
+        if(comma1>closebracket1){
+        	comma1=-1;
+        }
+        if(dash>closebracket2){
+        	dash=-1;
+        }
 
         if (dash == -1) {
             // fortran (e.g. "foo [{foo.cpp} {1,1}]")
@@ -206,7 +297,9 @@ public class Function implements Serializable, Comparable<Function> {
                     return sourceLink;
                 }
             }
-            int linenumber = Integer.parseInt(name.substring(openbracket1 + 1, comma1));
+            
+            String lineno=name.substring(openbracket1 + 1, comma1);
+            int linenumber = Integer.parseInt(lineno);
             sourceLink.setStartLine(linenumber);
             sourceLink.setEndLine(linenumber);
             return sourceLink;
@@ -385,10 +478,18 @@ public class Function implements Serializable, Comparable<Function> {
         this.stddevProfile = fp;
     }
 
+    public void setStddevAllProfile(FunctionProfile fp) {
+        this.stddevAllProfile = fp;
+    }
+
     public FunctionProfile getStddevProfile() {
         return stddevProfile;
     }
 
+    public FunctionProfile getStddevAllProfile() {
+        return stddevAllProfile;
+    }
+
     // mean section
     public void setMeanProfile(FunctionProfile fp) {
         this.meanProfile = fp;
@@ -398,6 +499,14 @@ public class Function implements Serializable, Comparable<Function> {
         return meanProfile;
     }
 
+    public void setMeanAllProfile(FunctionProfile fp) {
+        this.meanAllProfile = fp;
+    }
+
+    public FunctionProfile getMeanAllProfile() {
+        return meanAllProfile;
+    }
+
     public double getMeanInclusive(int metric) {
         return meanProfile.getInclusive(metric);
     }
@@ -479,7 +588,7 @@ public class Function implements Serializable, Comparable<Function> {
     }
     
     public int compareTo(Function o) {
-        return this.id -  o.getID();
+        return this.getID() -  o.getID();
     }
 
     public boolean isPhase() {
@@ -519,4 +628,32 @@ public class Function implements Serializable, Comparable<Function> {
         this.parentPhase = parentPhase;
     }
 
+    public void setSourceRegion(SourceRegion sourceRegion) {
+    	this.sourceLink = sourceRegion;
+    	if(sourceRegion.getShortName()==null){
+    		this.sourceLink=Function.getSourceLink(this.name);
+    	}
+    	if(this.short_name==null){
+    		this.short_name=this.sourceLink.getShortName();
+    	}
+    }
+
+	public String getShortName() {
+		return short_name;
+	}
+
+	public void setShortName(String short_name) {
+		if(short_name==null){
+			System.out.println("null short name");
+		}
+		this.short_name = short_name;
+	}
+
+	public void setDatabaseID(int id) {
+		this.databaseID = id;
+	}
+
+	public void setID(int id) {
+		this.id = id;
+	}
 }
\ No newline at end of file
