@@ -6,19 +6,27 @@
 
 package edu.uoregon.tau.perfdmf;
 
-import java.io.*;
-import java.util.*;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStreamReader;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.List;
+import java.util.StringTokenizer;
 
 public class HPMToolkitDataSource extends DataSource {
 
-    private Object initializeObject;
+    private List<File[]> initializeObject;
 
     //Frequently used items.
     private int metric = 0;
     private Function function = null;
     private FunctionProfile functionProfile = null;
-    private Node node = null;
-    private Context context = null;
+    //private Node node = null;
+    //private Context context = null;
     private edu.uoregon.tau.perfdmf.Thread thread = null;
     private int nodeID = -1;
     private int contextID = -1;
@@ -27,7 +35,7 @@ public class HPMToolkitDataSource extends DataSource {
     private String inputString = null;
     private BufferedReader br = null;
     boolean initialized = false;
-    Hashtable eventNames = new Hashtable();
+    Hashtable<String, Integer> eventNames = new Hashtable<String, Integer>();
 
     //Instance data.
     private LineData header1 = new LineData();
@@ -35,7 +43,7 @@ public class HPMToolkitDataSource extends DataSource {
     private LineData header3 = new LineData();
     private LineData header4 = new LineData();
     
-    public HPMToolkitDataSource(Object initializeObject) {
+    public HPMToolkitDataSource(List<File[]> initializeObject) {
         super();
         this.initializeObject = initializeObject;
     }
@@ -51,9 +59,9 @@ public class HPMToolkitDataSource extends DataSource {
 
     public void load() throws FileNotFoundException, IOException {
 
-        boolean firstFile = true;
-        List v = (List) initializeObject;
-        for (Iterator e = v.iterator(); e.hasNext();) {
+        //boolean firstFile = true;
+        List<File[]> v =  initializeObject;
+        for (Iterator<File[]> e = v.iterator(); e.hasNext();) {
             File files[] = (File[]) e.next();
             for (int i = 0; i < files.length; i++) {
                 long time = System.currentTimeMillis();
@@ -66,7 +74,7 @@ public class HPMToolkitDataSource extends DataSource {
                 nodeID++;
 
                 // clear the event names
-                eventNames = new Hashtable();
+                eventNames = new Hashtable<String, Integer>();
 
                 //####################################
                 //First Line
@@ -231,7 +239,7 @@ public class HPMToolkitDataSource extends DataSource {
         // ID
         // otherwise, reset the thread ID to 0.
         if (eventNames.containsKey(header1.s0)) {
-            Integer tmpID = (Integer) eventNames.get(header1.s0);
+            Integer tmpID = eventNames.get(header1.s0);
             threadID = tmpID.intValue();
             threadID++;
             eventNames.put(header1.s0, new Integer(threadID));
@@ -362,8 +370,8 @@ public class HPMToolkitDataSource extends DataSource {
             //double tmpValue = iEventValue / ((double) (header3.i0));
             //functionProfile.setInclusivePerCall(metric, tmpValue);
         }
-        functionProfile.setExclusivePercent(metric, 0);
-        functionProfile.setInclusivePercent(metric, 0);
+        //functionProfile.setExclusivePercent(metric, 0);
+        //functionProfile.setInclusivePercent(metric, 0);
         functionProfile.setNumCalls(header3.i0);
         functionProfile.setNumSubr(0);
 
@@ -390,11 +398,11 @@ public class HPMToolkitDataSource extends DataSource {
         metricName = "Time";
 
         String tmpStr = st1.nextToken().trim();
-        int metricCount = 0, newMetricCount = 0;
+        //int metricCount = 0, newMetricCount = 0;
         // need to clean stuff out of the value, like % and M and whatnot
         st1 = new StringTokenizer(tmpStr, " ");
         double dEventValue = 0.0;
-        int iEventValue = 0;
+        //int iEventValue = 0;
         tmpStr = st1.nextToken().trim();
 
         try {
@@ -447,26 +455,26 @@ public class HPMToolkitDataSource extends DataSource {
 //        if (function.getMaxInclusivePerCall(metric) < (dEventValue / header3.i0))
 //            function.setMaxInclusivePerCall(metric, (dEventValue / header3.i0));
 
-        functionProfile.setExclusivePercent(metric, 0);
-        functionProfile.setInclusivePercent(metric, 0);
+        //functionProfile.setExclusivePercent(metric, 0);
+        //functionProfile.setInclusivePercent(metric, 0);
         functionProfile.setNumCalls(header3.i0);
         functionProfile.setNumSubr(0);
 
     }
 
-    private String getMetricName(String inString) {
-        String tmpString = null;
-        int tmpInt = inString.indexOf("_MULTI_");
-
-        if (tmpInt > 0) {
-            //We are reading data from a multiple counter run.
-            //Grab the counter name.
-            tmpString = inString.substring(tmpInt + 7);
-            return tmpString;
-        }
-        //We are not reading data from a multiple counter run.
-        return tmpString;
-    }
+//    private String getMetricName(String inString) {
+//        String tmpString = null;
+//        int tmpInt = inString.indexOf("_MULTI_");
+//
+//        if (tmpInt > 0) {
+//            //We are reading data from a multiple counter run.
+//            //Grab the counter name.
+//            tmpString = inString.substring(tmpInt + 7);
+//            return tmpString;
+//        }
+//        //We are not reading data from a multiple counter run.
+//        return tmpString;
+//    }
 
     //####################################
     //End - Private Section.
