@@ -1,12 +1,13 @@
 package edu.uoregon.tau.perfdmf.loader;
 
-import edu.uoregon.tau.perfdmf.database.*;
-
 import java.sql.SQLException;
+
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
 import org.xml.sax.helpers.DefaultHandler;
 
+import edu.uoregon.tau.perfdmf.database.DB;
+
 /*** SAX Handler which creates SQL to load a xml document into the database. ***/
 
 public class AppLoadHandler extends DefaultHandler {
@@ -145,6 +146,8 @@ public class AppLoadHandler extends DefaultHandler {
                     buf.delete(0, buf.toString().length());
                     if (getDB().getDBType().compareTo("mysql") == 0)
                         buf.append("select LAST_INSERT_ID();");
+                    else if (getDB().getDBType().compareTo("sqlite") == 0)
+                        buf.append("select seq from sqlite_sequence where name = 'application'");
                     else if (getDB().getDBType().compareTo("db2") == 0)
                         buf.append("select IDENTITY_VAL_LOCAL() from application ");
                     else if (getDB().getDBType().compareTo("derby") == 0)
