@@ -3,7 +3,9 @@ package edu.uoregon.tau.perfexplorer.client;
 import java.awt.BasicStroke;
 import java.awt.Color;
 import java.text.DecimalFormat;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.NoSuchElementException;
 import java.util.StringTokenizer;
 
@@ -24,12 +26,14 @@ import org.jfree.data.xy.DefaultTableXYDataset;
 import org.jfree.data.xy.XYDataset;
 import org.jfree.data.xy.XYSeries;
 import org.jfree.data.xy.XYSeriesCollection;
+import org.jfree.chart.renderer.category.CategoryItemRenderer;
 
 import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.perfexplorer.common.ChartDataType;
 import edu.uoregon.tau.perfexplorer.common.CustomChartFactory;
 import edu.uoregon.tau.perfexplorer.common.RMIChartData;
 import edu.uoregon.tau.perfexplorer.common.RMIGeneralChartData;
+import edu.uoregon.tau.perfexplorer.common.PEChartColor;
 
 public class PerfExplorerChart extends PerfExplorerChartWindow {
 
@@ -122,12 +126,28 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 
 		DefaultTableXYDataset dataset = new DefaultTableXYDataset();
 		List<String> rowLabels = rawData.getRowLabels();
+		
+		//Get normalization factors
+		Map<Double,Double> totals = new HashMap<Double,Double>();
+		for (int y = 0 ; y < rawData.getRows() ; y++) {
+			List<double[]> row = rawData.getRowData(y);
+			for (int x = 0 ; x < row.size() ; x++) {
+				double[] values = (row.get(x));
+				Double tmpTotal= totals.get(values[0]);
+				if(tmpTotal==null){
+					tmpTotal=0.0;
+				}
+				totals.put(values[0], tmpTotal+values[1]/100.0);
+			}
+		}
+		
+		
 		for (int y = 0 ; y < rawData.getRows() ; y++) {
 			List<double[]> row = rawData.getRowData(y);
 			XYSeries s = new XYSeries(shortName(rowLabels.get(y)), true, false);
 			for (int x = 0 ; x < row.size() ; x++) {
 				double[] values = (row.get(x));
-				s.add(values[0], values[1]);
+				s.add(values[0], values[1]/totals.get(values[0]));
 			}
 			dataset.addSeries(s);
 		}
@@ -895,6 +915,12 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		return new PerfExplorerChart(chart, "Total " + PerfExplorerModel.getModel().getMetricName() + " Breakdown");
 	}
 
+    public static void customizeColors (CategoryItemRenderer renderer) {
+	  Color[] colors = PEChartColor.createDefaultColorArray();
+	  for (int i = 0 ; i < colors.length ; i++) {
+	    renderer.setSeriesPaint(i, colors[i]);
+	  }
+	}
 
 	static void customizeChart (JFreeChart chart, int rows, boolean lastLineIdeal) {
 		// set the chart to a common style
@@ -905,7 +931,7 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		if (lastLineIdeal)
 			renderer = new SpeedupXYLineAndShapeRenderer(rows);
 		else
-			renderer = new XYLineAndShapeRenderer();
+			renderer = new GoodXYLineAndShapeRenderer();
 		renderer.setBaseShapesFilled(true);
 		//renderer.setPlotShapes(true);
 		renderer.setBaseItemLabelsVisible(true);
@@ -945,6 +971,8 @@ public class PerfExplorerChart extends PerfExplorerChartWindow {
 		Utility.applyDefaultChartTheme(chart);
 
 		CategoryPlot plot = (CategoryPlot)chart.getPlot();
+		CategoryItemRenderer renderer = plot.getRenderer();
+		customizeColors(renderer);
 
 		// change the auto tick unit selection to integer units only...
 		NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
