@@ -5,27 +5,32 @@
 
 int main (int argc, char** argv) {
    printf("Connecting...\n");
-   PGconn* connection = NULL;
-   connection = taudb_connect_config("facets");
+   TAUDB_CONNECTION* connection = NULL;
+   int trialid = 1;
+   int items=0;  int* taudb_numItems=&items;
+   if (argc >= 2) {
+     connection = taudb_connect_config(argv[1],taudb_numItems);
+	 trialid = atoi(argv[2]);
+   } else {
+     fprintf(stderr, "Please specify a TAUdb config file.\n");
+	 exit(1);
+   }
    printf("Checking connection...\n");
    taudb_check_connection(connection);
    printf("Testing queries...\n");
 
-   int i = 0;
-   int j = 0;
-   int a, e, t;
+   int t;
 
    if (taudb_version == TAUDB_2005_SCHEMA) {
      // test the "find trials" method to populate the trial
      TAUDB_TRIAL* filter = taudb_create_trials(1);
-     //filter->id = 216;
-     filter->id = 209;
-     TAUDB_TRIAL* trials = taudb_query_trials(connection, FALSE, filter);
-     int numTrials = taudb_numItems;
+     filter->id = trialid;
+     TAUDB_TRIAL* trials = taudb_query_trials(connection, FALSE, filter,taudb_numItems);
+     int numTrials = *taudb_numItems;
      for (t = 0 ; t < numTrials ; t = t+1) {
         printf("  Trial name: '%s', id: %d\n", trials[t].name, trials[t].id);
-	    dump_metadata(trials[t].primary_metadata, trials[t].primary_metadata_count);
-        dump_threads(connection, &(trials[t]));
+	    dump_metadata(trials[t].primary_metadata);
+        dump_threads(connection, &(trials[t]), TRUE);
      }
    }
 
