@@ -5,26 +5,31 @@
 
 int main (int argc, char** argv) {
    printf("Connecting...\n");
-   char* config = "facets";
-   PGconn* connection = taudb_connect_config(config);
+   TAUDB_CONNECTION* connection = NULL;
+   int items=0;
+   int* taudb_numItems=&items;
+   if (argc >= 2) {
+     connection = taudb_connect_config(argv[1],taudb_numItems);
+   } else {
+     fprintf(stderr, "Please specify a TAUdb config file.\n");
+     exit(1);
+   }
    printf("Checking connection...\n");
    taudb_check_connection(connection);
    printf("Testing queries...\n");
 
-   int i = 0;
-   int j = 0;
-   int a, e, t;
+   int t;
 
    // test the "find trials" method to populate the trial
    TAUDB_TRIAL* filter = taudb_create_trials(1);
-   //filter->id = 216;
-   filter->id = 209;
-   TAUDB_TRIAL* trials = taudb_query_trials(connection, TRUE, filter);
-   int numTrials = taudb_numItems;
+   filter->id = atoi(argv[2]);
+   TAUDB_TRIAL* trials = taudb_query_trials(connection, TRUE, filter,taudb_numItems);
+   int numTrials = *taudb_numItems;
    for (t = 0 ; t < numTrials ; t = t+1) {
       printf("  Trial name: '%s', id: %d\n", trials[t].name, trials[t].id);
-      dump_metadata(trials[t].primary_metadata, trials[t].primary_metadata_count);
-      dump_trial(connection, &(trials[t]));
+      dump_metadata(trials[t].primary_metadata);
+      dump_secondary_metadata(trials[t].secondary_metadata);
+      dump_trial(connection, &(trials[t]), TRUE);
    }
 
    printf("Disconnecting...\n");
