@@ -175,6 +175,11 @@ ARCHminix
 ARCHi386_linux
 #endif
 
+#if defined(__x86_64) && defined(linux)
+# define _havehosttype_
+ARCHx86_64
+#endif
+
 #if defined(i386) && defined(__EMX__)
 # define _havehosttype_
 ARCHi386_emx
@@ -346,7 +351,11 @@ ARCHalpha
 
 #if defined(__APPLE__)
 #  define _havehosttype_ 
+#if defined(__arm64__)
+ARCHarm64_apple
+#else
 ARCHapple 
+#endif /* apple */
 # endif 
 
 #if defined(__hiuxmpp)
