@@ -22,6 +22,7 @@
 #include <stack>
 using namespace std;
 int debugPrint = 0;
+int remoteThread = -1;
 bool multiThreaded = false;
 #define dprintf if (debugPrint) printf
 
@@ -299,6 +300,9 @@ int EventTrigger( void *userData, double time,
   int cpuid = GlobalId (nodeToken, threadToken); /* GID */
   dprintf("EventTrigger: time %g, cpuid %d event id %d triggered value %lld \n", time, cpuid, userEventToken, userEventValue);
 
+	if(userEventToken==7004){
+		remoteThread=(int) userEventValue;
+	}
 
   /* write the sample data */
   OTF_Writer_writeCounter((OTF_Writer*)userData, TauGetClockTicksInGHz(time), cpuid, userEventToken, userEventValue); 
@@ -320,6 +324,12 @@ int SendMessage( void *userData, double time,
 		unsigned int messageComm)
 {
 
+	if(remoteThread>-1)
+	{
+		destinationThreadToken=remoteThread;
+		remoteThread=-1;
+	}
+
   int source = GlobalId(sourceNodeToken, sourceThreadToken);
   int dest   = GlobalId(destinationNodeToken, destinationThreadToken);
 
@@ -348,6 +358,12 @@ int RecvMessage( void *userData, double time,
 		unsigned int messageComm)
 {
 
+	if(remoteThread>-1)
+	{
+		sourceThreadToken=remoteThread;
+		remoteThread=-1;
+	}
+
   int source = GlobalId(sourceNodeToken, sourceThreadToken);
   int dest   = GlobalId(destinationNodeToken, destinationThreadToken);
 
@@ -384,6 +400,7 @@ int main(int argc, char **argv)
   Ttf_FileHandleT fh;
   OTF_FileManager* manager;
   int num_streams = 1;
+  int num_nodes = -1;
   int recs_read;
   char *trace_file;
   char *edf_file;
@@ -393,7 +410,7 @@ int main(int argc, char **argv)
   OTF_FileCompression compression = OTF_FILECOMPRESSION_UNCOMPRESSED; 
   int i; 
   /* main program: Usage app <trc> <edf> [-a] [-nomessage] */
-  if (argc < 3)
+  if (argc < 4)
   {
     printf("Usage: %s <TAU trace> <edf file> <out file> [-n streams] [-nomessage]  [-z] [-v]\n", 
 		    argv[0]);
@@ -428,6 +445,11 @@ int main(int argc, char **argv)
 	  num_streams = atoi(argv[i+1]); 
 	  i++; 
         }
+	if (strcmp(argv[i], "-s")==0)
+        {
+	  num_nodes = atoi(argv[i+1]); 
+	  i++; 
+        }
 	if (strcmp(argv[i], "-nomessage")==0)
 	{
 	  no_message_flag = 1;
@@ -482,43 +504,55 @@ int main(int argc, char **argv)
   OTF_Writer_writeDefCreator((OTF_Writer *)fcb, TAU_GLOBAL_STREAM_ID, "tau2otf converter version 2.15.x");
   OTF_Writer_writeDefCounterGroup((OTF_Writer *)fcb, TAU_GLOBAL_STREAM_ID, sampclassid, "TAU counter data");
 
-  /* in the first pass, we determine the no. of cpus and other group related
-   * information. In the second pass, we look at the function entry/exits */ 
 
-  Ttf_CallbacksT firstpass;
-  /* In the first pass, we just look for node/thread ids and def records */
-  firstpass.UserData = fcb;
-  firstpass.DefThread = DefThread;
-  firstpass.EndTrace = EndTrace;
-  firstpass.DefClkPeriod = ClockPeriod;
-  firstpass.DefThread = DefThread;
-  firstpass.DefStateGroup = DefStateGroup;
-  firstpass.DefState = DefState;
-  firstpass.SendMessage = 0; /* Important to declare these as null! */
-  firstpass.RecvMessage = 0; /* Important to declare these as null! */
-  firstpass.DefUserEvent = 0;
-  firstpass.EventTrigger = 0; /* these events are ignored in the first pass */
-  firstpass.EnterState = 0;   /* these events are ignored in the first pass */
-  firstpass.LeaveState = 0;   /* these events are ignored in the first pass */
+  int totalnidtids;
 
+  if (num_nodes == -1) {
+    /* in the first pass, we determine the no. of cpus and other group related
+     * information. In the second pass, we look at the function entry/exits */ 
+    
+    Ttf_CallbacksT firstpass;
+    /* In the first pass, we just look for node/thread ids and def records */
+    firstpass.UserData = fcb;
+    firstpass.DefThread = DefThread;
+    firstpass.EndTrace = EndTrace;
+    firstpass.DefClkPeriod = ClockPeriod;
+    firstpass.DefThread = DefThread;
+    firstpass.DefStateGroup = DefStateGroup;
+    firstpass.DefState = DefState;
+    firstpass.SendMessage = 0; /* Important to declare these as null! */
+    firstpass.RecvMessage = 0; /* Important to declare these as null! */
+    firstpass.DefUserEvent = 0;
+    firstpass.EventTrigger = 0; /* these events are ignored in the first pass */
+    firstpass.EnterState = 0;   /* these events are ignored in the first pass */
+    firstpass.LeaveState = 0;   /* these events are ignored in the first pass */
+    
 
-  /* Go through all trace records */
-  do {
-    recs_read = Ttf_ReadNumEvents(fh,firstpass, 1024);
+    /* Go through all trace records */
+    do {
+      recs_read = Ttf_ReadNumEvents(fh,firstpass, 1024);
 #ifdef DEBUG 
-    if (recs_read != 0)
-      cout <<"Read "<<recs_read<<" records"<<endl;
+      if (recs_read != 0)
+	cout <<"Read "<<recs_read<<" records"<<endl;
 #endif 
-  }
-  while ((recs_read >=0) && (!EndOfTrace));
-  /* reset the position of the trace to the first record */
-  for (map< pair<int,int>, int, less< pair<int,int> > >:: iterator it = 
-		  EOF_Trace.begin(); it != EOF_Trace.end(); it++)
-  { /* Explicilty mark end of trace to be not over */ 
-    (*it).second = 0;
+    }
+    while ((recs_read >0) && (!EndOfTrace));
+    
+
+    /* reset the position of the trace to the first record */
+    for (map< pair<int,int>, int, less< pair<int,int> > >:: iterator it = 
+	   EOF_Trace.begin(); it != EOF_Trace.end(); it++)
+      { /* Explicilty mark end of trace to be not over */ 
+	(*it).second = 0;
+      }
+    totalnidtids = EOF_Trace.size(); 
+  } else {
+    totalnidtids = num_nodes;
+    for (i=0; i<num_nodes; i++) {
+      numthreads[i] = 1; 
+    }
   }
 
-  int totalnidtids = EOF_Trace.size(); 
   /* This is ok for single threaded programs. For multi-threaded programs
    * we'll need to modify the way we describe the cpus/threads */
 /* THERE'S NO NEED TO WRITE THE TOTAL NO. OF CPUS in OTF */
@@ -636,7 +670,7 @@ int main(int argc, char **argv)
       cout <<"Read "<<recs_read<<" records"<<endl;
 #endif /* DEBUG */
   }
-  while ((recs_read >=0) && (!EndOfTrace));
+  while ((recs_read >0) && (!EndOfTrace));
 
   /* dummy records */
   Ttf_CloseFile(fh);
@@ -650,9 +684,9 @@ int main(int argc, char **argv)
 
 
 /***************************************************************************
- * $RCSfile: tau2otf.cpp,v $   $Author: sameer $
- * $Revision: 1.3 $   $Date: 2006/06/21 11:43:44 $
- * VERSION_ID: $Id: tau2otf.cpp,v 1.3 2006/06/21 11:43:44 sameer Exp $
+ * $RCSfile: tau2otf.cpp,v $   $Author: amorris $
+ * $Revision: 1.5 $   $Date: 2008/05/28 21:21:27 $
+ * VERSION_ID: $Id: tau2otf.cpp,v 1.5 2008/05/28 21:21:27 amorris Exp $
  ***************************************************************************/
 
 
